	PROGRAM NMHDECAYSLHA

*  Program to compute the NMSSM Higgs masses, couplings and
*  Branching ratios, with experimental and theoretical constraints
*
*   During the run:	
*
*	PAR(1) = lambda
*	PAR(2) = kappa
*	PAR(3) = tan(beta)
*	PAR(4) = mu (effective mu term = lambda*s)
*	PAR(5) = Alambda
*	PAR(6) = Akappa
*	PAR(7) = mQ3		
*	PAR(8) = mU3	
*	PAR(9) = mD3
*	PAR(10) = mL3
*	PAR(11) = mE3
*	PAR(12) = AU3
*	PAR(13) = AD3
*	PAR(14) = AE3
*	PAR(15) = mQ
*	PAR(16) = mU
*	PAR(17) = mD
*	PAR(18) = mL
*	PAR(19) = mE
*	PAR(20) = M1
*	PAR(21) = M2
*	PAR(22) = M3
*
*   On output:
*
*	SMASS(1-3): CP-even masses (ordered)
*
*	SCOMP(1-3,1-3): Mixing angles: if HB(I) are the bare states,
*	  HB(I) = Re(H1), Re(H2), Re(S), and HM(I) are the mass eigenstates, 
*	  the convention is HB(I) = SUM_(J=1,3) SCOMP(J,I)*HM(J)
*	  which is equivalent to HM(I) = SUM_(J=1,3) SCOMP(I,J)*HB(J)
*
*	PMASS(1-2): CP-odd masses (ordered)
*
*	PCOMP(1-2,1-2): Mixing angles: if AB(I) are the bare states,
*	  AB(I) = Im(H1), Im(H2), Im(S), and AM(I) are the mass eigenstates, 
*	  the convention is 
*	  AM(I) = PCOMP(I,1)*(COSBETA*AB(1)+SINBETA*AB(2))
*			+ PCOMP(I,2)*AB(3)
*
*	CMASS: Charged Higgs mass
*
*	WIDTH(i) Total decay width of h1,h2,h3,a1,a2 (i=1..5)
*		with the following branching ratios:
*	BRJJ(i)  h1,h2,h3,a1,a2	-> gluon gluon
*	BRMM(i)		"	-> mu mu
*	BRLL(i)		"	-> tau tau
*	BRSS(i)		"	-> ss
*	BRCC(i)		"	-> cc
*	BRBB(i)		"	-> bb
*	BRTT(i)		"	-> tt
*	BRWW(i)		"	-> WW (BRWW(4,5)=0)
*	BRZZ(i)		"	-> ZZ (BRZZ(4,5)=0)
*	BRGG(i)		"	-> gamma gamma
*	BRZG(i)         "       -> Z gamma
*	BRHAA(i,j)	hi -> a1a1, a1a2, a2a2 (j=1,2,3)
*	BRHCHC(i)	hi -> h+h-
*	BRHAZ(i,j)	hi -> Zaj
*	BRHCW(i)	h1,h2,h3 -> h+W- (i=1,3), a1,a2 -> h+W- (i=4,5)
*	BRHHH(i)	h2 -> h1h1, h3-> h1h1, h1h2, h2h2 (i=1,2,3,4)
*	BRAHA(i)	a2 -> a1hi
*	BRAHZ(i,j)	ai -> Zhj
*	BRNEUNEU(i,j,k)	-> neutralinos j,k
*	BRCHACHA(i,j)	-> charginos 11, 12, 22 (j=1,2,3)
*
*	HCWIDTH		Total decay width of the charged Higgs
*		       with the following branching ratios:
*	HCBRM     h+ ->	mu nu_mu
*	HCBRL		tau nu_tau
*	HCBRSU		s u
*	HCBRBU		b u
*	HCBRSC		s c
*	HCBRBC		b c
*	HCBRBT		b t
*	HCBRWH(i)	h1W+, h2W+, h3W+, a1W+, a2W+ (i=1..5)
*	HCBRSUSY(i,j)	neutalino i chargino j
*
*	MNEU(i)	 Mass of neutralino chi_i
*	NEU(i,j) chi_i components on bino, wino, higgsino 1&2, singlino (j=1..5)
*
*
*	MCHA(i)		Chargino masses
*	U(i,j),V(i,j)	Chargino mixing matrices
*
*       ERRORS: IFAIL=0..9
*
*	IFAIL =	0	OK
*		1,3,5,7	m_S1^2 < 0
*		2,3,6,7	m_P1^2 < 0
*		4,5,6,7	m_Hc^2 < 0
*		8	m_stop^2 or m_sbottom^2 < 0
*		9	Violation of at least one phenomenological constraint
*
*       Phenomenological constraints:
*
*		PROB(I) = 0, I = 1 - 14: OK
*		
*		PROB(1) = 1	chargino too light
*		PROB(2) = 1	excluded by Z -> neutralinos
*		PROB(3) = 1	charged Higgs too light
*		PROB(4) = 1	excluded by ee -> hZ 
*		PROB(5) = 1	excluded by ee -> hZ, h -> bb
*		PROB(6) = 1	excluded by ee -> hZ, h -> tautau
*		PROB(7) = 1	excluded by ee -> hZ, h -> invisible 
*		PROB(8) = 1	excluded by ee -> hZ, h -> 2jets
*		PROB(9) = 1	excluded by ee -> hZ, h -> 2photons
*		PROB(10) = 1	excluded by ee -> hZ, h -> AA -> 4bs
*		PROB(11) = 1	excluded by ee -> hA -> 4bs
*		PROB(12) = 1	excluded by ee -> hA -> 4taus
*		PROB(13) = 1	excluded by ee -> hA -> AAA -> 6bs
*		PROB(14) = 1    lightest scalar can decay into squarks
*		PROB(15) = 1    second scalar can decay into squarks
*		PROB(16) = 1    third scalar can decay into squarks
*		PROB(17) = 1    light. p.-scalar can decay into squarks
*		PROB(18) = 1    sec. p.-scalar can decay into squarks
*		PROB(19) = 1    charged Higgs can decay into squarks
*		PROB(20) = 1	Landau Pole in lambda, kappa, ht, hb below MGUT
*		PROB(21) = 1	Unphysical global minimum
*               PROB(22) = 1    excluded by ee -> Zh -> ZAA -> Z+light pair

	IMPLICIT NONE

	INTEGER NFL,NPROB,NPAR
	PARAMETER (NFL=9,NPROB=22,NPAR=22)
	INTEGER NFAIL(NFL),PROB(NPROB)
	INTEGER I,IFAIL

	DOUBLE PRECISION PAR(NPAR)
	DOUBLE PRECISION LM,KM,TM,MM,ALM,AKM
	DOUBLE PRECISION LN,LNN,KN,KNN,TN,TNN,MN,MNN,ALN,ALNN,AKN,AKNN

	COMMON/INPAR/LM,KM,TM,MM,ALM,AKM

*   Reading of the input parameters

	CALL INPUT(PAR,NPAR)

*   Initialization


	DO I=1,NFL
	 NFAIL(I)=0
	ENDDO

	PAR(1)=LM
	PAR(2)=KM
	PAR(3)=TM
	PAR(4)=MM
	PAR(5)=ALM
	PAR(6)=AKM

	DO I=1,NPROB
	 PROB(I)=0
	ENDDO	

*   Computation of Higgs masses

	CALL MHIGGS(PAR,IFAIL)
	IF(IFAIL.NE.0)GOTO 11
	
*   Computation of chargino masses

	CALL CHARGINO(PAR)

*   Computation of neutralino masses

	CALL NEUTRALINO(PAR)

*   Computation of Higgs branching ratios

	CALL DECAY(PAR)

*   LEP constraints

	CALL SUBLEP(PAR,PROB)

*   Landau Pole

	CALL RGES(PAR,PROB,NPROB)

*   Global minimum?

	CALL CHECKMIN(PAR,PROB,NPROB)

	DO I=1,NPROB
	 IF(PROB(I).NE.0)IFAIL=9
	ENDDO

11	CALL OUTPUT(PAR,IFAIL,PROB)

	END



	SUBROUTINE INPUT(PAR,NPAR)

	IMPLICIT NONE

	INTEGER N0,NLOOP,NBER,NPAR
	INTEGER NhZind,NhZbb,NhZll,NhZinv,NhZjj,NhZgg
	INTEGER NhA4b,NhA4tau,NAAA6b,NAAZ4b,I,J
	INTEGER Ncccc02,Ncccc04,Ncccc05,Ncccc06,Ncccc08,Ncccc1
	INTEGER Nccgg02,Nccgg04,Nccgg05,Nccgg06,Nccgg08,Nccgg1
	INTEGER Ncctt02,Ncctt04,Ncctt05,Ncctt06,Ncctt08,Ncctt1
	INTEGER Ngggg02,Ngggg04,Ngggg05,Ngggg06,Ngggg08,Ngggg1
	INTEGER Nttgg02,Nttgg04,Nttgg05,Nttgg06,Nttgg08,Nttgg1
	INTEGER Ntttt02,Ntttt04,Ntttt05,Ntttt06,Ntttt08,Ntttt1

	DOUBLE PRECISION PAR(*)
	DOUBLE PRECISION LM,KM,TM,MM,ALM,AKM
	DOUBLE PRECISION ACC,XITLA,XLAMBDA,MC0,MB0,MT0
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION GZINV,GZINVMAX,MCHAMIN,MCMIN,SIGNEU1,SIGNEU
	DOUBLE PRECISION hZind(1000,2),hZbb(1000,2),hZll(1000,2)
	DOUBLE PRECISION hZinv(1000,2),hZjj(1000,2),hZgg(1000,2)
	DOUBLE PRECISION hA4b(10000,3),hA4tau(10000,3)
	DOUBLE PRECISION AAA6b(10000,3),AAZ4b(10000,3)
	DOUBLE PRECISION cccc02(1000,2),cccc04(1000,2),cccc05(1000,2)
	DOUBLE PRECISION cccc06(1000,2),cccc08(1000,2),cccc1(1000,2)
	DOUBLE PRECISION ccgg02(1000,2),ccgg04(1000,2),ccgg05(1000,2)
	DOUBLE PRECISION ccgg06(1000,2),ccgg08(1000,2),ccgg1(1000,2)
	DOUBLE PRECISION cctt02(1000,2),cctt04(1000,2),cctt05(1000,2)
	DOUBLE PRECISION cctt06(1000,2),cctt08(1000,2),cctt1(1000,2)
	DOUBLE PRECISION gggg02(1000,2),gggg04(1000,2),gggg05(1000,2)
	DOUBLE PRECISION gggg06(1000,2),gggg08(1000,2),gggg1(1000,2)
	DOUBLE PRECISION ttgg02(1000,2),ttgg04(1000,2),ttgg05(1000,2)
	DOUBLE PRECISION ttgg06(1000,2),ttgg08(1000,2),ttgg1(1000,2)
	DOUBLE PRECISION tttt02(1000,2),tttt04(1000,2),tttt05(1000,2)
	DOUBLE PRECISION tttt06(1000,2),tttt08(1000,2),tttt1(1000,2)
	DOUBLE PRECISION VUS,VCB,VUB
*  PARAMETERS FOR READING THE SLHA INPUT FILE
	DOUBLE PRECISION VAL
	CHARACTER CHINL*120,CHBLCK*60,CHDUM*120,FILE*20
	INTEGER NLINE,INL,ICH,IX,IVAL,INMSSM

	COMMON/INPAR/LM,KM,TM,MM,ALM,AKM
	COMMON/CONTR/GZINVMAX,MCHAMIN,MCMIN,SIGNEU1,SIGNEU
	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/CKM/VUS,VCB,VUB
	COMMON/ALS/XLAMBDA,MC0,MB0,MT0,N0
	COMMON/LEP/hZind,hZbb,hZll,hZinv,hZjj,hZgg,
     C		hA4b,hA4tau,AAA6b,AAZ4b,
     C          cccc02,cccc04,cccc05,cccc06,cccc08,cccc1,
     C          ccgg02,ccgg04,ccgg05,ccgg06,ccgg08,ccgg1,
     C          cctt02,cctt04,cctt05,cctt06,cctt08,cctt1,
     C          gggg02,gggg04,gggg05,gggg06,gggg08,gggg1,
     C          ttgg02,ttgg04,ttgg05,ttgg06,ttgg08,ttgg1,
     C          tttt02,tttt04,tttt05,tttt06,tttt08,tttt1,     
     C		NhZind,NhZbb,NhZll,NhZinv,NhZjj,NhZgg,
     C		NhA4b,NhA4tau,NAAA6b,NAAZ4b,
     C          Ncccc02,Ncccc04,Ncccc05,Ncccc06,Ncccc08,Ncccc1,
     C          Nccgg02,Nccgg04,Nccgg05,Nccgg06,Nccgg08,Nccgg1,
     C          Ncctt02,Ncctt04,Ncctt05,Ncctt06,Ncctt08,Ncctt1,
     C          Ngggg02,Ngggg04,Ngggg05,Ngggg06,Ngggg08,Ngggg1,
     C          Nttgg02,Nttgg04,Nttgg05,Nttgg06,Nttgg08,Nttgg1,
     C          Ntttt02,Ntttt04,Ntttt05,Ntttt06,Ntttt08,Ntttt1


*   I/O files

	OPEN(15,FILE='slhainp.dat', STATUS= 'UNKNOWN')

*       DEFAULT VALUES FOR INMSSM (DEFAULT: FALSE)

	INMSSM=0
	
*   DEFAULT VALUES FOR SMINPUTS

	ALP0=  1.D0/137.036
	GF=     1.16639D-5
	ALSMZ=  0.12
	MZ=     91.187
	MB=     4.24	
	MT=     175.
	MTAU=   1.7771

*    THE FOLLOWING SM PARAMETERS ARE NOT READ IN 
*    AND HAVE NECESSARILY TO BE DEFINED	

	MMUON=  0.105658389
	MW=     80.42
	MS=     0.19
	MC=     1.61
	VUS= 0.222D0
	VCB= 0.041D0
	VUB= 0.0036D0
	
*   DEFAULT VALUES FOR LIMITS ON INV. Z WIDTH (GZINVMAX),
*   LOWER LIMIT ON CHARGINO MASS (MCHAMIN), CHARGED HIGGS MASS (MCMIN)

	GZINVMAX=1.76D-3
	MCHAMIN=103.5D0
	MCMIN=78.6D0
	SIGNEU1=1.D-2
	SIGNEU=1.D-1
	
	
*   INITIALIZE READ LOOP
	NLINE=0
	CHBLCK=' '
	
*   START TO READ NEW LINE INTO CHINL
 21	CHINL=' '

*   LINE NUMBER
	NLINE=NLINE+1

	READ(15,'(A120)',END=29,ERR=999) CHINL
		
*   CHECK FOR EMPTY OR COMMENT LINES
	IF(CHINL.EQ.' '.OR.CHINL(1:1).EQ.'#'
     C   .OR.CHINL(1:1).EQ.'*') GOTO 21
	
*   FORCE UPPER CASE LETTERS IN CHINL (AS REQUIRED BELOW)
	INL=0
 22	INL=INL+1
	IF(CHINL(INL:INL).NE.'#') THEN
	 DO ICH=97,122
	  IF(CHINL(INL:INL).EQ.CHAR(ICH)) CHINL(INL:INL)=CHAR(ICH-32)	
	 END DO
	 IF(INL.LT.120) GOTO 22
	ENDIF
	
*   CHECK FOR BLOCK STATEMENT
	IF(CHINL(1:1).EQ.'B') THEN
	 READ(CHINL,'(A6,A)',ERR=999) CHDUM,CHBLCK
	 GOTO 21
	ENDIF
	
*   CHECK FOR NMSSM MODEL IN MODSEL
	IF(CHBLCK(1:6).EQ.'MODSEL') THEN
	 READ(CHINL,*,ERR=999) IX,IVAL
	 IF((IX.EQ.3).AND.(IVAL.EQ.1)) INMSSM=1
	ENDIF
	
	IF(INMSSM.NE.1) THEN
	 WRITE(*,*) 'SWITCH IN MODSEL DOES NOT CORRESPOND TO THE NMSSM'
         STOP
	ENDIF
	
*   READ SMINPUTS
	IF(CHBLCK(1:8).EQ.'SMINPUTS') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.1) ALP0=1.D0/VAL
	 IF(IX.EQ.2) GF=VAL
	 IF(IX.EQ.3) ALSMZ=VAL
	 IF(IX.EQ.4) MZ=VAL
	 IF(IX.EQ.5) MB=VAL
	 IF(IX.EQ.6) MT=VAL
	 IF(IX.EQ.7) MTAU=VAL
	 
*   READ TANBETA
	ELSEIF(CHBLCK(1:6).EQ.'MINPAR') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.3) TM=VAL
	 
*   READ EXTPAR
	ELSEIF(CHBLCK(1:6).EQ.'EXTPAR') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.1) PAR(20)=VAL
	 IF(IX.EQ.2) PAR(21)=VAL
	 IF(IX.EQ.3) PAR(22)=VAL
	 IF(IX.EQ.11) PAR(12)=VAL
	 IF(IX.EQ.12) PAR(13)=VAL
	 IF(IX.EQ.13) PAR(14)=VAL
	 IF(IX.EQ.23) MM=VAL
	 IF(IX.EQ.31) PAR(18)=VAL
	 IF(IX.EQ.32) PAR(18)=VAL
	 IF(IX.EQ.33) PAR(10)=VAL
	 IF(IX.EQ.34) PAR(19)=VAL
	 IF(IX.EQ.35) PAR(19)=VAL
	 IF(IX.EQ.36) PAR(11)=VAL
	 IF(IX.EQ.41) PAR(15)=VAL
	 IF(IX.EQ.42) PAR(15)=VAL
	 IF(IX.EQ.43) PAR(7)=VAL
	 IF(IX.EQ.44) PAR(16)=VAL
	 IF(IX.EQ.45) PAR(16)=VAL
	 IF(IX.EQ.46) PAR(8)=VAL
	 IF(IX.EQ.47) PAR(17)=VAL
	 IF(IX.EQ.48) PAR(17)=VAL
	 IF(IX.EQ.49) PAR(9)=VAL
	 
	 IF(IX.EQ.61) LM=VAL
	 IF(IX.EQ.62) KM=VAL
	 IF(IX.EQ.63) ALM=VAL
	 IF(IX.EQ.64) AKM=VAL
	 
	ENDIF
	
*   END OF READING FROM INPUT FILE
	GOTO 21
 29     CONTINUE

*   LEP constraints

	OPEN(11,FILE='LEPCON/hZind.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZind.dat'
	I=1
 111	READ(11,*,END=112,ERR=998)(hZind(I,J),J=1,2)
	I=I+1
	GOTO 111
 112	NhZind=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hZbb.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZbb.dat'
	I=1
 211	READ(11,*,END=212,ERR=998)(hZbb(I,J),J=1,2)
	I=I+1
	GOTO 211
 212	NhZbb=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hZll.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZll.dat'
	I=1
 311	READ(11,*,END=312,ERR=998)(hZll(I,J),J=1,2)
	I=I+1
	GOTO 311
 312	NhZll=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hZinv.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZinv.dat'
	I=1
 411	READ(11,*,END=412,ERR=998)(hZinv(I,J),J=1,2)
	I=I+1
	GOTO 411
 412	NhZinv=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hZjj.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZjj.dat'
	I=1
 511	READ(11,*,END=512,ERR=998)(hZjj(I,J),J=1,2)
	I=I+1
	GOTO 511
 512	NhZjj=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hZgg.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hZgg.dat'
	I=1
 611	READ(11,*,END=612,ERR=998)(hZgg(I,J),J=1,2)
	I=I+1
	GOTO 611
 612	NhZgg=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hA4b.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hA4b.dat'
	I=1
 711	READ(11,*,END=712,ERR=998)(hA4b(I,J),J=1,3)
	I=I+1
	GOTO 711
 712	NhA4b=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/hA4tau.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/hA4tau.dat'
	I=1
 811	READ(11,*,END=812,ERR=998)(hA4tau(I,J),J=1,3)
	I=I+1
	GOTO 811
 812	NhA4tau=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/AAA6b.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/AAA6b.dat'
	I=1
 911	READ(11,*,END=912,ERR=998)(AAA6b(I,J),J=1,3)
	I=I+1
	GOTO 911
 912	NAAA6b=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/AAZ4b.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/AAZ4b.dat'
	I=1
 1101	READ(11,*,END=1102,ERR=998)(AAZ4b(I,J),J=1,3)
	I=I+1
	GOTO 1101
 1102	NAAZ4b=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc02.dat'
	I=1
 1201	READ(11,*,END=1202) cccc02(I,1)
        READ(11,*,END=1202,ERR=998) cccc02(I,2)
	I=I+1
	GOTO 1201
 1202	Ncccc02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc04.dat'
	I=1
 1211	READ(11,*,END=1212) cccc04(I,1)
        READ(11,*,END=1212,ERR=998) cccc04(I,2)
	I=I+1
	GOTO 1211
 1212	Ncccc04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc05.dat'
	I=1
 1221	READ(11,*,END=1222) cccc05(I,1)
        READ(11,*,END=1222,ERR=998) cccc05(I,2)
	I=I+1
	GOTO 1221
 1222	Ncccc05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc06.dat'
	I=1
 1231	READ(11,*,END=1232) cccc06(I,1)
        READ(11,*,END=1232,ERR=998) cccc06(I,2)
	I=I+1
	GOTO 1231
 1232	Ncccc06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc08.dat'
	I=1
 1241	READ(11,*,END=1242) cccc08(I,1)
        READ(11,*,END=1242,ERR=998) cccc08(I,2)
	I=I+1
	GOTO 1241
 1242	Ncccc08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cccc1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cccc1.dat'
	I=1
 1251	READ(11,*,END=1252) cccc1(I,1)
        READ(11,*,END=1252,ERR=998) cccc1(I,2)
	I=I+1
	GOTO 1251
 1252	Ncccc1=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg02.dat'
	I=1
 1301	READ(11,*,END=1302) ccgg02(I,1)
        READ(11,*,END=1302,ERR=998) ccgg02(I,2)
	I=I+1
	GOTO 1301
 1302	Nccgg02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg04.dat'
	I=1
 1311	READ(11,*,END=1312) ccgg04(I,1)
        READ(11,*,END=1312,ERR=998) ccgg04(I,2)
	I=I+1
	GOTO 1311
 1312	Nccgg04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg05.dat'
	I=1
 1321	READ(11,*,END=1322) ccgg05(I,1)
        READ(11,*,END=1322,ERR=998) ccgg05(I,2)
	I=I+1
	GOTO 1321
 1322	Nccgg05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg06.dat'
	I=1
 1331	READ(11,*,END=1332) ccgg06(I,1)
        READ(11,*,END=1332,ERR=998) ccgg06(I,2)
	I=I+1
	GOTO 1331
 1332	Nccgg06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg08.dat'
	I=1
 1341	READ(11,*,END=1342) ccgg08(I,1)
        READ(11,*,END=1342,ERR=998) ccgg08(I,2)
	I=I+1
	GOTO 1341
 1342	Nccgg08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ccgg1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ccgg1.dat'
	I=1
 1351	READ(11,*,END=1352) ccgg1(I,1)
        READ(11,*,END=1352,ERR=998) ccgg1(I,2)
	I=I+1
	GOTO 1351
 1352	Nccgg1=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt02.dat'
	I=1
 1401	READ(11,*,END=1402) cctt02(I,1)
        READ(11,*,END=1402,ERR=998) cctt02(I,2)
	I=I+1
	GOTO 1401
 1402	Ncctt02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt04.dat'
	I=1
 1411	READ(11,*,END=1412) cctt04(I,1)
        READ(11,*,END=1412,ERR=998) cctt04(I,2)
	I=I+1
	GOTO 1411
 1412	Ncctt04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt05.dat'
	I=1
 1421	READ(11,*,END=1422) cctt05(I,1)
        READ(11,*,END=1422,ERR=998) cctt05(I,2)
	I=I+1
	GOTO 1421
 1422	Ncctt05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt06.dat'
	I=1
 1431	READ(11,*,END=1432) cctt06(I,1)
        READ(11,*,END=1432,ERR=998) cctt06(I,2)
	I=I+1
	GOTO 1431
 1432	Ncctt06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt08.dat'
	I=1
 1441	READ(11,*,END=1442) cctt08(I,1)
        READ(11,*,END=1442,ERR=998) cctt08(I,2)
	I=I+1
	GOTO 1441
 1442	Ncctt08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/cctt1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/cctt1.dat'
	I=1
 1451	READ(11,*,END=1452) cctt1(I,1)
        READ(11,*,END=1452,ERR=998) cctt1(I,2)
	I=I+1
	GOTO 1451
 1452	Ncctt1=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg02.dat'
	I=1
 1501	READ(11,*,END=1502) gggg02(I,1)
        READ(11,*,END=1502,ERR=998) gggg02(I,2)
	I=I+1
	GOTO 1501
 1502	Ngggg02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg04.dat'
	I=1
 1511	READ(11,*,END=1512) gggg04(I,1)
        READ(11,*,END=1512,ERR=998) gggg04(I,2)
	I=I+1
	GOTO 1511
 1512	Ngggg04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg05.dat'
	I=1
 1521	READ(11,*,END=1522) gggg05(I,1)
        READ(11,*,END=1522,ERR=998) gggg05(I,2)
	I=I+1
	GOTO 1521
 1522	Ngggg05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg06.dat'
	I=1
 1531	READ(11,*,END=1532) gggg06(I,1)
        READ(11,*,END=1532,ERR=998) gggg06(I,2)
	I=I+1
	GOTO 1531
 1532	Ngggg06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg08.dat'
	I=1
 1541	READ(11,*,END=1542) gggg08(I,1)
        READ(11,*,END=1542,ERR=998) gggg08(I,2)
	I=I+1
	GOTO 1541
 1542	Ngggg08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/gggg1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/gggg1.dat'
	I=1
 1551	READ(11,*,END=1552) gggg1(I,1)
        READ(11,*,END=1552,ERR=998) gggg1(I,2)
	I=I+1
	GOTO 1551
 1552	Ngggg1=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg02.dat'
	I=1
 1601	READ(11,*,END=1602) ttgg02(I,1)
        READ(11,*,END=1602,ERR=998) ttgg02(I,2)
	I=I+1
	GOTO 1601
 1602	Nttgg02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg04.dat'
	I=1
 1611	READ(11,*,END=1612) ttgg04(I,1)
        READ(11,*,END=1612,ERR=998) ttgg04(I,2)
	I=I+1
	GOTO 1611
 1612	Nttgg04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg05.dat'
	I=1
 1621	READ(11,*,END=1622) ttgg05(I,1)
        READ(11,*,END=1622,ERR=998) ttgg05(I,2)
	I=I+1
	GOTO 1621
 1622	Nttgg05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg06.dat'
	I=1
 1631	READ(11,*,END=1632) ttgg06(I,1)
        READ(11,*,END=1632,ERR=998) ttgg06(I,2)
	I=I+1
	GOTO 1631
 1632	Nttgg06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg08.dat'
	I=1
 1641	READ(11,*,END=1642) ttgg08(I,1)
        READ(11,*,END=1642,ERR=998) ttgg08(I,2)
	I=I+1
	GOTO 1641
 1642	Nttgg08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/ttgg1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/ttgg1.dat'
	I=1
 1651	READ(11,*,END=1652) ttgg1(I,1)
        READ(11,*,END=1652,ERR=998) ttgg1(I,2)
	I=I+1
	GOTO 1651
 1652	Nttgg1=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt02.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt02.dat'
	I=1
 1701	READ(11,*,END=1702) tttt02(I,1)
        READ(11,*,END=1702,ERR=998) tttt02(I,2)
	I=I+1
	GOTO 1701
 1702	Ntttt02=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt04.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt04.dat'
	I=1
 1711	READ(11,*,END=1712) tttt04(I,1)
        READ(11,*,END=1712,ERR=998) tttt04(I,2)
	I=I+1
	GOTO 1711
 1712	Ntttt04=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt05.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt05.dat'
	I=1
 1721	READ(11,*,END=1722) tttt05(I,1)
        READ(11,*,END=1722,ERR=998) tttt05(I,2)
	I=I+1
	GOTO 1721
 1722	Ntttt05=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt06.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt06.dat'
	I=1
 1731	READ(11,*,END=1732) tttt06(I,1)
        READ(11,*,END=1732,ERR=998) tttt06(I,2)
	I=I+1
	GOTO 1731
 1732	Ntttt06=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt08.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt08.dat'
	I=1
 1741	READ(11,*,END=1742) tttt08(I,1)
        READ(11,*,END=1742,ERR=998) tttt08(I,2)
	I=I+1
	GOTO 1741
 1742	Ntttt08=I-1
	CLOSE(11)

	OPEN(11,FILE='LEPCON/tttt1.dat', STATUS= 'UNKNOWN')
	FILE='LEPCON/tttt1.dat'
	I=1
 1751	READ(11,*,END=1752) tttt1(I,1)
        READ(11,*,END=1752,ERR=998) tttt1(I,2)
	I=I+1
	GOTO 1751
 1752	Ntttt1=I-1
	CLOSE(11)

*   Initialization for ALPHAS and RUNM

	MSB=MS
	MC0=MC
	MB0=MB
	MT0=MT

	N0=5
	NLOOP=2
	NBER=18
	ACC=1.D-8
	XLAMBDA=XITLA(NLOOP,ALSMZ,ACC)
	CALL ALSINI(ACC)
	CALL BERNINI(NBER)
	
*    g1,g2 in the on-shell scheme in terms of GF, MZ(pole)
*    and MW(pole); ALP0=ALPEM(0) is used only in Higgs -> gamma gamma;
*    subsequently g2 = g_2^2, g1 = g_1^2

	g2=4.*DSQRT(2.D0)*GF*MW**2
	g1=4.*DSQRT(2.D0)*GF*(MZ**2-MW**2)
	

9999    RETURN

* ERROR CATCH

 998    WRITE(*,*) 'LEP INPUT READ ERROR IN ', FILE
 	STOP

 999    WRITE(*,*) 'inslha.dat READ ERROR ON LINE:', NLINE
        WRITE(*,*) CHINL(1:80)
	STOP

	END


	SUBROUTINE OUTPUT(PAR,IFAIL,PROB)

	IMPLICIT NONE

	INTEGER I,IFAIL,PROB(*)

	DOUBLE PRECISION PAR(*),SMASS(3),PMASS(2)
	DOUBLE PRECISION CMASS,SCOMP(3,3),PCOMP(2,2)
	DOUBLE PRECISION MCHA(2),U(2,2),V(2,2),MNEU(5),NEU(5,5)
	DOUBLE PRECISION CU(5),CD(5),CV(3),CG(5),CGA(5)
	DOUBLE PRECISION BRJJ(5),BRMM(5),BRLL(5),BRSS(5),BRCC(5)
	DOUBLE PRECISION BRBB(5),BRTT(5),BRWW(3),BRZZ(3),BRGG(5),BRZG(5)
	DOUBLE PRECISION BRHHH(4),BRHAA(3,3),BRHCHC(3),BRHAZ(3,2)
	DOUBLE PRECISION BRAHA(3),BRAHZ(2,3),BRHCW(5),BRHIGGS(5)
	DOUBLE PRECISION BRNEU(5,5,5),BRCHA(5,3),BRSUSY(5),WIDTH(5)
	DOUBLE PRECISION HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC
	DOUBLE PRECISION HCBRBT,HCBRWH(5),HCBRWHT,HCBRSUSY(5,2)
	DOUBLE PRECISION HCBRSUSYT,HCWIDTH
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION VUS,VCB,VUB
	DOUBLE PRECISION LM,KM,TM,MM,ALM,AKM
	DOUBLE PRECISION MST1,MST2,MSB1,MSB2
	

	COMMON/BRN/BRJJ,BRMM,BRLL,BRSS,BRCC,BRBB,BRTT,BRWW,BRZZ,BRGG,
     C		BRZG,BRHHH,BRHAA,BRHCHC,BRHAZ,BRAHA,BRAHZ,BRHCW,
     C		BRHIGGS,BRNEU,BRCHA,BRSUSY,WIDTH
	COMMON/BRC/HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC,
     C		HCBRBT,HCBRWH,HCBRWHT,HCBRSUSY,HCBRSUSYT,HCWIDTH
	COMMON/REDCOUP/CU,CD,CV,CG,CGA
	COMMON/HIGGSPEC/SMASS,SCOMP,PMASS,PCOMP,CMASS
	COMMON/SUSYSPEC/MCHA,U,V,MNEU,NEU
	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/CKM/VUS,VCB,VUB
	COMMON/INPAR/LM,KM,TM,MM,ALM,AKM
	COMMON/SQSPEC/MST1,MST2,MSB1,MSB2
 
	OPEN(17,FILE='spectr.dat', STATUS= 'UNKNOWN')
	
	WRITE(17,899) "# NMHDECAY OUTPUT IN SLHA FORMAT"
	WRITE(17,899) "# Info about spectrum calculator"
	WRITE(17,899) "BLOCK SPINFO        # Program information"
	WRITE(17,900) 1,"NMHDECAY   # spectrum calculator"
	WRITE(17,900) 2,"1.1        # version number"
	
	IF(PROB(1).NE.0) 
     C   WRITE(17,900) 3,"# chargino too light"
	IF(PROB(2).NE.0) 
     C   WRITE(17,900) 3,"# excluded by Z -> neutralinos"
        IF(PROB(3).NE.0) 
     C   WRITE(17,900) 3,"# charged Higgs too light"
        IF(PROB(4).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ"
        IF(PROB(5).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> bb"
        IF(PROB(6).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> tautau"
        IF(PROB(7).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> invisible"
        IF(PROB(8).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> 2jets"
        IF(PROB(9).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> 2photons"
        IF(PROB(10).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hZ, h -> AA -> 4bs"
        IF(PROB(11).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hA -> 4bs"
        IF(PROB(12).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hA -> 4taus"
        IF(PROB(13).NE.0) 
     C   WRITE(17,900) 3,"# excluded by e+e- -> hA -> AAA -> 6bs"
        IF(PROB(14).NE.0) 
     C   WRITE(17,900) 3,"# lightest scalar can decay into squarks"
        IF(PROB(15).NE.0) 
     C   WRITE(17,900) 3,"# second scalar can decay into squarks"
        IF(PROB(16).NE.0) 
     C   WRITE(17,900) 3,"# third scalar can decay into squarks"
        IF(PROB(17).NE.0) 
     C   WRITE(17,900) 3,"# light. p.-scalar can decay into squarks"
        IF(PROB(18).NE.0) 
     C   WRITE(17,900) 3,"# sec. p.-scalar can decay into squarks"
        IF(PROB(19).NE.0) 
     C   WRITE(17,900) 3,"# charged Higgs can decay into squarks"
        IF(PROB(20).NE.0) 
     C   WRITE(17,900) 3,"# Landau Pole below MGUT"
        IF(PROB(21).NE.0) 
     C   WRITE(17,900) 3,"# Not a global minimum"
        IF(PROB(22).NE.0) 
     C   WRITE(17,900) 3,"# This point elim. by special Zh->Zaa modes"
     
	IF(IFAIL.EQ.1 .OR. IFAIL.EQ.3 .OR. IFAIL.EQ.5 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900)4,"# M_H1^2<0"
	IF(IFAIL.EQ.2 .OR. IFAIL.EQ.3 .OR. IFAIL.EQ.6 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900)4,"# M_A1^2<0"
	IF(IFAIL.EQ.4 .OR. IFAIL.EQ.5 .OR. IFAIL.EQ.6 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900)4,"# M_HC^2<0"
	IF (IFAIL.EQ.8) 
     C 	   WRITE(17,900) 4,"# NEGATIVE STOP OR SBOTTOM MASS SQUARED"
	
	WRITE(17,899) "# Input parameters"
	WRITE(17,899) "BLOCK MODSEL"
	WRITE(17,899) "    3     1         # NMSSM PARTICLE CONTENT"
	WRITE(17,899) "BLOCK SMINPUTS"
	WRITE(17,901) 1,1./ALP0,"ALPHA_EM^-1"
	WRITE(17,901) 2,GF,"GF"
	WRITE(17,901) 3,ALSMZ,"ALPHA_S(MZ)"
	WRITE(17,901) 4,MZ,"MZ"
	WRITE(17,901) 5,MB,"MB(MB)"
	WRITE(17,901) 6,MT,"MTOP (POLE MASS)"
	WRITE(17,901) 7,MTAU,"MTAU"
	WRITE(17,899) "# SMINPUTS Beyond SLHA:"
	WRITE(17,906) "MW:",MW
	WRITE(17,906) "MS:",MS
	WRITE(17,906) "MC:",MC
	WRITE(17,906) "VUS:",VUS
	WRITE(17,906) "VCB:",VCB
	WRITE(17,906) "VUB:",VUB
	WRITE(17,899) "BLOCK MINPAR"
	WRITE(17,901) 3,PAR(3),"TANBETA"
	WRITE(17,899) "BLOCK EXTPAR"
	WRITE(17,901) 1,PAR(20),"M1"
	WRITE(17,901) 2,PAR(21),"M2"
	WRITE(17,901) 3,PAR(22),"M3"
	WRITE(17,901) 11,PAR(12),"ATOP"
	WRITE(17,901) 12,PAR(13),"ABOTTOM"
	WRITE(17,901) 13,PAR(14),"ATAU"
	WRITE(17,901) 23,PAR(4),"MU"
	WRITE(17,901) 33,PAR(18),"LEFT STAU"
	WRITE(17,901) 36,PAR(19),"RIGHT STAU"
	WRITE(17,901) 43,PAR(7),"LEFT 3RD GEN. SQUARKS"
	WRITE(17,901) 46,PAR(8),"RIGHT T-SQUARKS"
	WRITE(17,901) 49,PAR(9),"RIGHT B-SQUARKS"
	WRITE(17,901) 61,PAR(1),"LAMBDA"
	WRITE(17,901) 62,PAR(2),"KAPPA"
	WRITE(17,901) 63,PAR(5),"A_LAMBDA"
	WRITE(17,901) 64,PAR(6),"A_KAPPA"
	
	IF((1.LE.IFAIL).AND.(IFAIL.LE.8)) GOTO 1000
	
	WRITE(17,899) "# "
	WRITE(17,899) "BLOCK MASS   # Mass spectrum "
	WRITE(17,899) "#  PDG Code     mass             particle "
	WRITE(17,902) 25,SMASS(1),"lightest neutral scalar"
	WRITE(17,902) 35,SMASS(2),"second neutral scalar"
	WRITE(17,902) 45,SMASS(3),"third neutral scalar"
	WRITE(17,902) 36,PMASS(1),"lightest pseudoscalar"
	WRITE(17,902) 46,PMASS(2),"second pseudoscalar"
	WRITE(17,902) 37,CMASS,"charged Higgs"
	
	WRITE(17,902) 1000022,MNEU(1),"neutralino(1)"
	WRITE(17,902) 1000023,MNEU(2),"neutralino(2)"
	WRITE(17,902) 1000025,MNEU(3),"neutralino(3)"
	WRITE(17,902) 1000035,MNEU(4),"neutralino(4)"
	WRITE(17,902) 1000045,MNEU(5),"neutralino(5)"
	WRITE(17,902) 1000024,MCHA(1),"chargino(1)"
	WRITE(17,902) 1000037,MCHA(2),"chargino(2)"
	
	WRITE(17,902) 1000005,MSB1," ~b_1"
	WRITE(17,902) 1000006,MST1," ~t_1"
	WRITE(17,902) 2000005,MSB2," ~b_2"
	WRITE(17,902) 2000006,MST2," ~t_2"

	WRITE(17,899) "# 3*3 Higgs mixing"
	WRITE(17,899) "BLOCK HIGMIX"
	WRITE(17,903) 1,1,SCOMP(2,2),"S_(1,1)"
	WRITE(17,903) 1,2,SCOMP(2,1),"S_(1,2)"
	WRITE(17,903) 1,3,SCOMP(2,3),"S_(1,3)"
	WRITE(17,903) 2,1,SCOMP(1,2),"S_(2,1)"
	WRITE(17,903) 2,2,SCOMP(1,1),"S_(2,2)"
	WRITE(17,903) 2,3,SCOMP(1,3),"S_(2,3)"
	WRITE(17,903) 3,1,SCOMP(3,2),"S_(3,1)"
	WRITE(17,903) 3,2,SCOMP(3,1),"S_(3,2)"
	WRITE(17,903) 3,3,SCOMP(3,3),"S_(3,3)"
	
	WRITE(17,899) "# 2*2 Pseudoscalar Higgs mixing"
	WRITE(17,899) "BLOCK AMIX"
	WRITE(17,903) 1,1,PCOMP(1,1),"P'_(1,1)"
	WRITE(17,903) 1,2,PCOMP(1,2),"P'_(1,2)"
	WRITE(17,903) 2,1,PCOMP(2,1),"P'_(2,1)"
	WRITE(17,903) 2,2,PCOMP(2,2),"P'_(2,2)"
	
	WRITE(17,899) "# Gaugino-Higgsino mixing"
	WRITE(17,899) "BLOCK NMIX  # 5*5 Neutralino Mixing Matrix"
	WRITE(17,903) 1,1,NEU(1,1),"N_(1,1)"
	WRITE(17,903) 1,2,NEU(1,2),"N_(1,2)"
	WRITE(17,903) 1,3,NEU(1,4),"N_(1,3)"
	WRITE(17,903) 1,4,NEU(1,3),"N_(1,4)"
	WRITE(17,903) 1,5,NEU(1,5),"N_(1,5)"
	WRITE(17,903) 2,1,NEU(2,1),"N_(2,1)"
	WRITE(17,903) 2,2,NEU(2,2),"N_(2,2)"
	WRITE(17,903) 2,3,NEU(2,4),"N_(2,3)"
	WRITE(17,903) 2,4,NEU(2,3),"N_(2,4)"
	WRITE(17,903) 2,5,NEU(2,5),"N_(2,5)"
	WRITE(17,903) 3,1,NEU(3,1),"N_(3,1)"
	WRITE(17,903) 3,2,NEU(3,2),"N_(3,2)"
	WRITE(17,903) 3,3,NEU(3,4),"N_(3,3)"
	WRITE(17,903) 3,4,NEU(3,3),"N_(3,4)"
	WRITE(17,903) 3,5,NEU(3,5),"N_(3,5)"
	WRITE(17,903) 4,1,NEU(4,1),"N_(4,1)"
	WRITE(17,903) 4,2,NEU(4,2),"N_(4,2)"
	WRITE(17,903) 4,3,NEU(4,4),"N_(4,3)"
	WRITE(17,903) 4,4,NEU(4,3),"N_(4,4)"
	WRITE(17,903) 4,5,NEU(4,5),"N_(4,5)"
	WRITE(17,903) 5,1,NEU(5,1),"N_(5,1)"
	WRITE(17,903) 5,2,NEU(5,2),"N_(5,2)"
	WRITE(17,903) 5,3,NEU(5,4),"N_(5,3)"
	WRITE(17,903) 5,4,NEU(5,3),"N_(5,4)"
	WRITE(17,903) 5,5,NEU(5,5),"N_(5,5)"

	WRITE(17,899) "BLOCK UMIX  # Chargino U Mixing Matrix"
	WRITE(17,903) 1,1,U(1,1),"U_(1,1)"
	WRITE(17,903) 1,2,U(1,2),"U_(1,2)"
	WRITE(17,903) 2,1,U(2,1),"U_(2,1)"
	WRITE(17,903) 2,2,U(2,2),"U_(2,2)"

	WRITE(17,899) "BLOCK VMIX  # Chargino V Mixing Matrix"
	WRITE(17,903) 1,1,V(1,1),"V_(1,1)"
	WRITE(17,903) 1,2,V(1,2),"V_(1,2)"
	WRITE(17,903) 2,1,V(2,1),"V_(2,1)"
	WRITE(17,903) 2,2,V(2,2),"V_(2,2)"

	OPEN(18,FILE='decay.dat', STATUS= 'UNKNOWN')
	
	WRITE(18,899) "# HIGGS BRANCHING RATIOS IN SLHA FORMAT"
	WRITE(18,899) "# Info about decay package"
	WRITE(18,899) "BLOCK DCINFO        # Program information"
	WRITE(18,900) 1,"NMHDECAY   # Decay package"
	WRITE(18,900) 2,"1.0        # Version number"
	WRITE(18,899) "#           PDG          Width"
	
	WRITE(18,904) 25,WIDTH(1),"Lightest neutral Higgs scalar"
	WRITE(18,905) BRJJ(1),2,21,21,"BR(H_1 -> gluon gluon)"
	WRITE(18,905) BRMM(1),2,13,-13,"BR(H_1 -> muon muon)"
	WRITE(18,905) BRLL(1),2,15,-15,"BR(H_1 -> tau tau)"
	WRITE(18,905) BRSS(1),2,3,-3,"BR(H_1 -> s sbar)"
	WRITE(18,905) BRCC(1),2,4,-4,"BR(H_1 -> c cbar)"
	WRITE(18,905) BRBB(1),2,5,-5,"BR(H_1 -> b bbar)"
	WRITE(18,905) BRTT(1),2,6,-6,"BR(H_1 -> t tbar)"
	WRITE(18,905) BRWW(1),2,24,-24,"BR(H_1 -> W+ W-)"
	WRITE(18,905) BRZZ(1),2,23,23,"BR(H_1 -> Z Z)"
	WRITE(18,905) BRGG(1),2,22,22,"BR(H_1 -> gamma gamma)"
	WRITE(18,905) BRZG(1),2,23,22,"BR(H_1 -> Z gamma)"
	WRITE(18,905) BRHAA(1,1),2,36,36,"BR(H_1 -> A_1 A_1)"
	WRITE(18,905) BRHAA(1,2),2,36,46,"BR(H_1 -> A_1 A_2)"
	WRITE(18,905) BRHAA(1,3),2,46,46,"BR(H_1 -> A_2 A_2)"
	WRITE(18,905) BRHAZ(1,1),2,23,36,"BR(H_1 -> A_1 Z)"
	WRITE(18,905) BRNEU(1,1,1),2,1000022,1000022,"BR(H_1 -> neu_1 neu_1)"
	WRITE(18,905) BRNEU(1,1,2),2,1000022,1000023,"BR(H_1 -> neu_1 neu_2)"
	WRITE(18,905) BRNEU(1,1,4),2,1000022,1000025,"BR(H_1 -> neu_1 neu_3)"
	WRITE(18,905) BRNEU(1,1,3),2,1000022,1000035,"BR(H_1 -> neu_1 neu_4)"
	WRITE(18,905) BRNEU(1,1,5),2,1000022,1000045,"BR(H_1 -> neu_1 neu_5)"
	WRITE(18,905) BRNEU(1,2,2),2,1000023,1000023,"BR(H_1 -> neu_2 neu_2)"
	WRITE(18,905) BRNEU(1,2,4),2,1000023,1000025,"BR(H_1 -> neu_2 neu_3)"
	WRITE(18,905) BRNEU(1,2,3),2,1000023,1000035,"BR(H_1 -> neu_2 neu_4)"
	WRITE(18,905) BRNEU(1,2,5),2,1000023,1000045,"BR(H_1 -> neu_2 neu_5)"
	WRITE(18,905) BRNEU(1,4,4),2,1000025,1000025,"BR(H_1 -> neu_3 neu_3)"
	WRITE(18,905) BRNEU(1,4,3),2,1000025,1000035,"BR(H_1 -> neu_3 neu_4)"
	WRITE(18,905) BRNEU(1,4,5),2,1000025,1000045,"BR(H_1 -> neu_3 neu_5)"
	WRITE(18,905) BRNEU(1,3,3),2,1000035,1000035,"BR(H_1 -> neu_4 neu_4)"
	WRITE(18,905) BRNEU(1,3,5),2,1000035,1000045,"BR(H_1 -> neu_4 neu_5)"
	WRITE(18,905) BRNEU(1,5,5),2,1000045,1000045,"BR(H_1 -> neu_5 neu_5)"
	WRITE(18,905) BRCHA(1,1),2,1000024,-1000024,"BR(H_1 -> cha_1 cha_1)"
	WRITE(18,905) BRCHA(1,2),2,1000024,-1000037,"BR(H_1 -> cha_1 cha_2)"
	WRITE(18,905) BRCHA(1,3),2,1000037,-1000037,"BR(H_1 -> cha_2 cha_2)"

	WRITE(18,904) 35,WIDTH(2),"2nd neutral Higgs scalar"
	WRITE(18,905) BRJJ(2),2,21,21,"BR(H_2 -> gluon gluon)"
	WRITE(18,905) BRMM(2),2,13,-13,"BR(H_2 -> muon muon)"
	WRITE(18,905) BRLL(2),2,15,-15,"BR(H_2 -> tau tau)"
	WRITE(18,905) BRSS(2),2,3,-3,"BR(H_2 -> s sbar)"
	WRITE(18,905) BRCC(2),2,4,-4,"BR(H_2 -> c cbar)"
	WRITE(18,905) BRBB(2),2,5,-5,"BR(H_2 -> b bbar)"
	WRITE(18,905) BRTT(2),2,6,-6,"BR(H_2 -> t tbar)"
	WRITE(18,905) BRWW(2),2,24,-24,"BR(H_2 -> W+ W-)"
	WRITE(18,905) BRZZ(2),2,23,23,"BR(H_2 -> Z Z)"
	WRITE(18,905) BRGG(2),2,22,22,"BR(H_2 -> gamma gamma)"
	WRITE(18,905) BRZG(2),2,23,22,"BR(H_2 -> Z gamma)"
	WRITE(18,905) BRHHH(1),2,25,25,"BR(H_2 -> H_1 H_1)"
	WRITE(18,905) BRHAA(2,1),2,36,36,"BR(H_2 -> A_1 A_1)"
	WRITE(18,905) BRHAA(2,2),2,36,46,"BR(H_2 -> A_1 A_2)"
	WRITE(18,905) BRHAA(2,3),2,46,46,"BR(H_2 -> A_2 A_2)"
	WRITE(18,905) BRHAZ(2,1),2,23,36,"BR(H_2 -> A_1 Z)"
	WRITE(18,905) BRNEU(2,1,1),2,1000022,1000022,"BR(H_2 -> neu_1 neu_1)"
	WRITE(18,905) BRNEU(2,1,2),2,1000022,1000023,"BR(H_2 -> neu_1 neu_2)"
	WRITE(18,905) BRNEU(2,1,4),2,1000022,1000025,"BR(H_2 -> neu_1 neu_3)"
	WRITE(18,905) BRNEU(2,1,3),2,1000022,1000035,"BR(H_2 -> neu_1 neu_4)"
	WRITE(18,905) BRNEU(2,1,5),2,1000022,1000045,"BR(H_2 -> neu_1 neu_5)"
	WRITE(18,905) BRNEU(2,2,2),2,1000023,1000023,"BR(H_2 -> neu_2 neu_2)"
	WRITE(18,905) BRNEU(2,2,4),2,1000023,1000025,"BR(H_2 -> neu_2 neu_3)"
	WRITE(18,905) BRNEU(2,2,3),2,1000023,1000035,"BR(H_2 -> neu_2 neu_4)"
	WRITE(18,905) BRNEU(2,2,5),2,1000023,1000045,"BR(H_2 -> neu_2 neu_5)"
	WRITE(18,905) BRNEU(2,4,4),2,1000025,1000025,"BR(H_2 -> neu_3 neu_3)"
	WRITE(18,905) BRNEU(2,4,3),2,1000025,1000035,"BR(H_2 -> neu_3 neu_4)"
	WRITE(18,905) BRNEU(2,4,5),2,1000025,1000045,"BR(H_2 -> neu_3 neu_5)"
	WRITE(18,905) BRNEU(2,3,3),2,1000035,1000035,"BR(H_2 -> neu_4 neu_4)"
	WRITE(18,905) BRNEU(2,3,5),2,1000035,1000045,"BR(H_2 -> neu_4 neu_5)"
	WRITE(18,905) BRNEU(2,5,5),2,1000045,1000045,"BR(H_2 -> neu_5 neu_5)"
	WRITE(18,905) BRCHA(2,1),2,1000024,-1000024,"BR(H_2 -> cha_1 cha_1)"
	WRITE(18,905) BRCHA(2,2),2,1000024,-1000037,"BR(H_2 -> cha_1 cha_2)"
	WRITE(18,905) BRCHA(2,3),2,1000037,-1000037,"BR(H_2 -> cha_2 cha_2)"

	WRITE(18,904) 45,WIDTH(3),"3rd neutral Higgs scalar"
	WRITE(18,905) BRJJ(3),2,21,21,"BR(H_3 -> gluon gluon)"
	WRITE(18,905) BRMM(3),2,13,-13,"BR(H_3 -> muon muon)"
	WRITE(18,905) BRLL(3),2,15,-15,"BR(H_3 -> tau tau)"
	WRITE(18,905) BRSS(3),2,3,-3,"BR(H_3 -> s sbar)"
	WRITE(18,905) BRCC(3),2,4,-4,"BR(H_3 -> c cbar)"
	WRITE(18,905) BRBB(3),2,5,-5,"BR(H_3 -> b bbar)"
	WRITE(18,905) BRTT(3),2,6,-6,"BR(H_3 -> t tbar)"
	WRITE(18,905) BRWW(3),2,24,-24,"BR(H_3 -> W+ W-)"
	WRITE(18,905) BRZZ(3),2,23,23,"BR(H_3 -> Z Z)"
	WRITE(18,905) BRGG(3),2,22,22,"BR(H_3 -> gamma gamma)"
	WRITE(18,905) BRZG(3),2,23,22,"BR(H_3 -> Z gamma)"
	WRITE(18,905) BRHHH(2),2,25,25,"BR(H_3 -> H_1 H_1)"
	WRITE(18,905) BRHHH(3),2,25,35,"BR(H_3 -> H_1 H_2)"
	WRITE(18,905) BRHHH(4),2,35,35,"BR(H_3 -> H_2 H_2)"
	WRITE(18,905) BRHAA(3,1),2,36,36,"BR(H_3 -> A_1 A_1)"
	WRITE(18,905) BRHAA(3,2),2,36,46,"BR(H_3 -> A_1 A_2)"
	WRITE(18,905) BRHAA(3,3),2,46,46,"BR(H_3 -> A_2 A_2)"
	WRITE(18,905) BRHAZ(3,1),2,23,36,"BR(H_3 -> Z A_1)"
	WRITE(18,905) BRHAZ(3,2),2,23,46,"BR(H_3 -> Z A_2)"
	WRITE(18,905) BRHCHC(3),2,37,-37,"BR(H_3 -> H+ H-)"
	WRITE(18,905) BRHCW(3),2,24,-37,"BR(H_3 -> W+ H-)"
	WRITE(18,905) BRNEU(3,1,1),2,1000022,1000022,"BR(H_3 -> neu_1 neu_1)"
	WRITE(18,905) BRNEU(3,1,2),2,1000022,1000023,"BR(H_3 -> neu_1 neu_2)"
	WRITE(18,905) BRNEU(3,1,4),2,1000022,1000025,"BR(H_3 -> neu_1 neu_3)"
	WRITE(18,905) BRNEU(3,1,3),2,1000022,1000035,"BR(H_3 -> neu_1 neu_4)"
	WRITE(18,905) BRNEU(3,1,5),2,1000022,1000045,"BR(H_3 -> neu_1 neu_5)"
	WRITE(18,905) BRNEU(3,2,2),2,1000023,1000023,"BR(H_3 -> neu_2 neu_2)"
	WRITE(18,905) BRNEU(3,2,4),2,1000023,1000025,"BR(H_3 -> neu_2 neu_3)"
	WRITE(18,905) BRNEU(3,2,3),2,1000023,1000035,"BR(H_3 -> neu_2 neu_4)"
	WRITE(18,905) BRNEU(3,2,5),2,1000023,1000045,"BR(H_3 -> neu_2 neu_5)"
	WRITE(18,905) BRNEU(3,4,4),2,1000025,1000025,"BR(H_3 -> neu_3 neu_3)"
	WRITE(18,905) BRNEU(3,4,3),2,1000025,1000035,"BR(H_3 -> neu_3 neu_4)"
	WRITE(18,905) BRNEU(3,4,5),2,1000025,1000045,"BR(H_3 -> neu_3 neu_5)"
	WRITE(18,905) BRNEU(3,3,3),2,1000035,1000035,"BR(H_3 -> neu_4 neu_4)"
	WRITE(18,905) BRNEU(3,3,5),2,1000035,1000045,"BR(H_3 -> neu_4 neu_5)"
	WRITE(18,905) BRNEU(3,5,5),2,1000045,1000045,"BR(H_3 -> neu_5 neu_5)"
	WRITE(18,905) BRCHA(3,1),2,1000024,-1000024,"BR(H_3 -> cha_1 cha_1)"
	WRITE(18,905) BRCHA(3,2),2,1000024,-1000037,"BR(H_3 -> cha_1 cha_2)"
	WRITE(18,905) BRCHA(3,3),2,1000037,-1000037,"BR(H_3 -> cha_2 cha_2)"

	WRITE(18,904) 36,WIDTH(4),"Lightest pseudoscalar"
	WRITE(18,905) BRJJ(4),2,21,21,"BR(A_1 -> gluon gluon)"
	WRITE(18,905) BRMM(4),2,13,-13,"BR(A_1 -> muon muon)"
	WRITE(18,905) BRLL(4),2,15,-15,"BR(A_1 -> tau tau)"
	WRITE(18,905) BRSS(4),2,3,-3,"BR(A_1 -> s sbar)"
	WRITE(18,905) BRCC(4),2,4,-4,"BR(A_1 -> c cbar)"
	WRITE(18,905) BRBB(4),2,5,-5,"BR(A_1 -> b bbar)"
	WRITE(18,905) BRTT(4),2,6,-6,"BR(A_1 -> t tbar)"
	WRITE(18,905) BRGG(4),2,22,22,"BR(A_1 -> gamma gamma)"
	WRITE(18,905) BRZG(4),2,23,22,"BR(A_1 -> Z gamma)"
	WRITE(18,905) BRAHZ(1,1),2,23,25,"BR(A_1 -> Z H_1)"
	WRITE(18,905) BRAHZ(1,2),2,23,35,"BR(A_1 -> Z H_2)"
	WRITE(18,905) BRNEU(4,1,1),2,1000022,1000022,"BR(A_1 -> neu_1 neu_1)"
	WRITE(18,905) BRNEU(4,1,2),2,1000022,1000023,"BR(A_1 -> neu_1 neu_2)"
	WRITE(18,905) BRNEU(4,1,4),2,1000022,1000025,"BR(A_1 -> neu_1 neu_3)"
	WRITE(18,905) BRNEU(4,1,3),2,1000022,1000035,"BR(A_1 -> neu_1 neu_4)"
	WRITE(18,905) BRNEU(4,1,5),2,1000022,1000045,"BR(A_1 -> neu_1 neu_5)"
	WRITE(18,905) BRNEU(4,2,2),2,1000023,1000023,"BR(A_1 -> neu_2 neu_2)"
	WRITE(18,905) BRNEU(4,2,4),2,1000023,1000025,"BR(A_1 -> neu_2 neu_3)"
	WRITE(18,905) BRNEU(4,2,3),2,1000023,1000035,"BR(A_1 -> neu_2 neu_4)"
	WRITE(18,905) BRNEU(4,2,5),2,1000023,1000045,"BR(A_1 -> neu_2 neu_5)"
	WRITE(18,905) BRNEU(4,4,4),2,1000025,1000025,"BR(A_1 -> neu_3 neu_3)"
	WRITE(18,905) BRNEU(4,4,3),2,1000025,1000035,"BR(A_1 -> neu_3 neu_4)"
	WRITE(18,905) BRNEU(4,4,5),2,1000025,1000045,"BR(A_1 -> neu_3 neu_5)"
	WRITE(18,905) BRNEU(4,3,3),2,1000035,1000035,"BR(A_1 -> neu_4 neu_4)"
	WRITE(18,905) BRNEU(4,3,5),2,1000035,1000045,"BR(A_1 -> neu_4 neu_5)"
	WRITE(18,905) BRNEU(4,5,5),2,1000045,1000045,"BR(A_1 -> neu_5 neu_5)"
	WRITE(18,905) BRCHA(4,1),2,1000024,-1000024,"BR(A_1 -> cha_1 cha_1)"
	WRITE(18,905) BRCHA(4,2),2,1000024,-1000037,"BR(A_1 -> cha_1 cha_2)"
	WRITE(18,905) BRCHA(4,3),2,1000037,-1000037,"BR(A_1 -> cha_2 cha_2)"

	WRITE(18,904) 46,WIDTH(5),"2nd pseudoscalar"
	WRITE(18,905) BRJJ(5),2,21,21,"BR(A_2 -> gluon gluon)"
	WRITE(18,905) BRMM(5),2,13,-13,"BR(A_2 -> muon muon)"
	WRITE(18,905) BRLL(5),2,15,-15,"BR(A_2 -> tau tau)"
	WRITE(18,905) BRSS(5),2,3,-3,"BR(A_2 -> s sbar)"
	WRITE(18,905) BRCC(5),2,4,-4,"BR(A_2 -> c cbar)"
	WRITE(18,905) BRBB(5),2,5,-5,"BR(A_2 -> b bbar)"
	WRITE(18,905) BRTT(5),2,6,-6,"BR(A_2 -> t tbar)"
	WRITE(18,905) BRGG(5),2,22,22,"BR(A_2 -> gamma gamma)"
	WRITE(18,905) BRZG(5),2,23,22,"BR(A_2 -> Z gamma)"
	WRITE(18,905) BRAHA(1),2,36,25,"BR(A_2 -> A_1 H_1)"
	WRITE(18,905) BRAHA(2),2,36,35,"BR(A_2 -> A_1 H_2)"
	WRITE(18,905) BRAHA(3),2,36,45,"BR(A_2 -> A_1 H_3)"
	WRITE(18,905) BRAHZ(2,1),2,23,25,"BR(A_2 -> Z H_1)"
	WRITE(18,905) BRAHZ(2,2),2,23,35,"BR(A_2 -> Z H_2)"
	WRITE(18,905) BRHCW(5),2,24,-37,"BR(A_2 -> W+ H-)"
	WRITE(18,905) BRNEU(5,1,1),2,1000022,1000022,"BR(A_2 -> neu_1 neu_1)"
	WRITE(18,905) BRNEU(5,1,2),2,1000022,1000023,"BR(A_2 -> neu_1 neu_2)"
	WRITE(18,905) BRNEU(5,1,4),2,1000022,1000025,"BR(A_2 -> neu_1 neu_3)"
	WRITE(18,905) BRNEU(5,1,3),2,1000022,1000035,"BR(A_2 -> neu_1 neu_4)"
	WRITE(18,905) BRNEU(5,1,5),2,1000022,1000045,"BR(A_2 -> neu_1 neu_5)"
	WRITE(18,905) BRNEU(5,2,2),2,1000023,1000023,"BR(A_2 -> neu_2 neu_2)"
	WRITE(18,905) BRNEU(5,2,4),2,1000023,1000025,"BR(A_2 -> neu_2 neu_3)"
	WRITE(18,905) BRNEU(5,2,3),2,1000023,1000035,"BR(A_2 -> neu_2 neu_4)"
	WRITE(18,905) BRNEU(5,2,5),2,1000023,1000045,"BR(A_2 -> neu_2 neu_5)"
	WRITE(18,905) BRNEU(5,4,4),2,1000025,1000025,"BR(A_2 -> neu_3 neu_3)"
	WRITE(18,905) BRNEU(5,4,3),2,1000025,1000035,"BR(A_2 -> neu_3 neu_4)"
	WRITE(18,905) BRNEU(5,4,5),2,1000025,1000045,"BR(A_2 -> neu_3 neu_5)"
	WRITE(18,905) BRNEU(5,3,3),2,1000035,1000035,"BR(A_2 -> neu_4 neu_4)"
	WRITE(18,905) BRNEU(5,3,5),2,1000035,1000045,"BR(A_2 -> neu_4 neu_5)"
	WRITE(18,905) BRNEU(5,5,5),2,1000045,1000045,"BR(A_2 -> neu_5 neu_5)"
	WRITE(18,905) BRCHA(5,1),2,1000024,-1000024,"BR(A_2 -> cha_1 cha_1)"
	WRITE(18,905) BRCHA(5,2),2,1000024,-1000037,"BR(A_2 -> cha_1 cha_2)"
	WRITE(18,905) BRCHA(4,3),2,1000037,-1000037,"BR(A_2 -> cha_2 cha_2)"

	WRITE(18,904) 37,HCWIDTH,"Charged Higgs"
	WRITE(18,905) HCBRM,2,-13,14,"BR(H+ -> muon nu_muon)"
	WRITE(18,905) HCBRL,2,-15,16,"BR(H+ -> tau nu_tau)"
	WRITE(18,905) HCBRSU,2,2,-3,"BR(H+ -> u sbar)"
	WRITE(18,905) HCBRSC,2,4,-3,"BR(H+ -> c sbar)"
	WRITE(18,905) HCBRBU,2,2,-5,"BR(H+ -> u bbar)"
	WRITE(18,905) HCBRBC,2,4,-5,"BR(H+ -> c bbar)"
	WRITE(18,905) HCBRBT,2,6,-5,"BR(H+ -> t bbar)"
	WRITE(18,905) HCBRWH(1),2,24,25,"BR(H+ -> W+ H_1)"
	WRITE(18,905) HCBRWH(2),2,24,35,"BR(H+ -> W+ H_2)"
	WRITE(18,905) HCBRWH(4),2,24,36,"BR(H+ -> W+ A_1)"
	WRITE(18,905) HCBRSUSY(1,1),2,1000024,1000022,"BR(H+ -> cha_1 neu_1)"
	WRITE(18,905) HCBRSUSY(2,1),2,1000024,1000023,"BR(H+ -> cha_1 neu_2)"
	WRITE(18,905) HCBRSUSY(3,1),2,1000024,1000025,"BR(H+ -> cha_1 neu_3)"
	WRITE(18,905) HCBRSUSY(4,1),2,1000024,1000035,"BR(H+ -> cha_1 neu_4)"
	WRITE(18,905) HCBRSUSY(5,1),2,1000024,1000045,"BR(H+ -> cha_1 neu_5)"
	WRITE(18,905) HCBRSUSY(1,2),2,1000037,1000022,"BR(H+ -> cha_2 neu_1)"
	WRITE(18,905) HCBRSUSY(2,2),2,1000037,1000023,"BR(H+ -> cha_2 neu_2)"
	WRITE(18,905) HCBRSUSY(3,2),2,1000037,1000025,"BR(H+ -> cha_2 neu_3)"
	WRITE(18,905) HCBRSUSY(4,2),2,1000037,1000035,"BR(H+ -> cha_2 neu_4)"
	WRITE(18,905) HCBRSUSY(5,2),2,1000037,1000045,"BR(H+ -> cha_2 neu_5)"

1000    CONTINUE
	
 899    FORMAT(A)
 900    FORMAT(1X,I5,3X,A)
 901    FORMAT(1X,I5,3X,1P,E16.8,0P,3X,'#',1X,A)
 902    FORMAT(1X,I9,3X,1P,E16.8,0P,3X,'#',1X,A)
 903    FORMAT(1X,I2,1X,I2,3X,1P,E16.8,0P,3X,'#',1X,A)
 904    FORMAT('DECAY',1X,I9,3X,1P,E16.8,0P,3X,'#',1X,A)
 905    FORMAT(3X,1P,E16.8,0P,3X,I2,3X,I9,1X,I9,1X,2X,'#',1X,A)
 906    FORMAT('#',1X,A,3X,E16.8)

	END


	SUBROUTINE MHIGGS(PAR,IFAIL)

***********************************************************************
*
*	This subroutine computes the Higgs masses and couplings in
*	the NMSSM. The relevant input parameters are
*
*	PAR(1) = lambda
*	PAR(2) = kappa
*	PAR(3) = tan(beta)	at the scale M_top
*	PAR(4) = mu		effective mu term (=lambda*s)
*	PAR(5) = A_lambda
*	PAR(6) = A_kappa
*	PAR(7) = MQ3           Left-handed Squarks
*	PAR(8) = MU3           Right-handed Stops
*	PAR(9) = MD3           Right-handed Sbottoms
*	PAR(12) = A_TOP
*	PAR(13) = A_BOT         
*       PAR(20) = M1
*       PAR(21) = M2  
*
*	On output: 
*
*	SMASS(1-3): CP-even masses (ordered)
*
*	SCOMP(1-3,1-3): Mixing angles: if HB(I) are the bare states,
*	  HB(I) = Re(H1), Re(H2), Re(S), and HM(I) are the mass eigenstates, 
*	  the convention is HB(I) = SUM_(J=1,3) SCOMP(J,I)*HM(J)
*	  which is equivalent to HM(I) = SUM_(J=1,3) SCOMP(I,J)*HB(J)
*
*	PMASS(1-2): CP-odd masses (ordered)
*
*	PCOMP(1-2,1-2): Mixing angles: if AB(I) are the bare states,
*	  AB(I) = Im(H1), Im(H2), Im(S), and AM(I) are the mass eigenstates, 
*	  the convention is 
*	  AM(I) = PCOMP(I,1)*(COSBETA*AB(1)+SINBETA*AB(2))
*			+ PCOMP(I,2)*AB(3)
*
*	CMASS: Charged Higgs mass
*
*	IFAIL:  =   0	        OK
*		=   1,3,5,7	SMASS(1)**2 < 0
*		=   2,3,6,7	PMASS(1)**2 < 0
*		=   4,5,6,7	CMASS**2 < 0
*		=   8   	M_stop1**2 or M_sbot1**2 < 0
*
*	The precision in the computation of the effective potential is:
*
*	Terms ~ ht^4/hb^4 from one stop/sbottom-loops are computed
*	exactly in the mixing parameters.
*
*	Leading logs ~ g^2*(ht^2/hb^2)*log(m_squark/mtop) (where g is an
*	electro-weak gauge coupling) are taken into account due to the 
*	wave function renormalizations factors.
*
*	Leading logs ~ g^4 are added explicitely.
*
*	Leading double logs from two loops ~ ht^6 and ht^4*alpha_s are 
*	taken into account.
*
*	For heavy higgses (with masses mhh > mtop) the leading log
*	contributions ~ (ht^2/hb^2)*log(mhh/mtop) to the pole masses
*	are taken into account, but not the corresponding effects on the
*	mixing angles.
*
*	All mixing angles are at the scale m_top.
*
*	The dominant errors come from one loop terms ~ g^2*ht^2 without
*	large logs, and from two loop terms without large double logs.
*
************************************************************************

	IMPLICIT NONE

	INTEGER IFAIL,I

	DOUBLE PRECISION PAR(*),SMASS(3),SCOMP(3,3)
        DOUBLE PRECISION PMASS(2),PCOMP(2,2),CMASS
	DOUBLE PRECISION mp(3),mdia(3),moff(3)
	DOUBLE PRECISION rdiat(3),rofft(3),rdiab(3),roffb(3)
	DOUBLE PRECISION l,k,Al,Ak,tanb,mu,nu,B,h1,h2
	DOUBLE PRECISION Wt,Wb,mst1,mst2,msb1,msb2
	DOUBLE PRECISION ht,Zht,Mstop,dMst,At,Xt,s2t,emt,fmt,gmt
	DOUBLE PRECISION hb,Zhb,Msbot,dMsb,Ab,Xb,s2b,emb,fmb,gmb
	DOUBLE PRECISION Q2,t,pi,g,gam,C2TW,S2TW
	DOUBLE PRECISION h1q,h2q,tanbq,htq,hbq,mtopq,mbotq
	DOUBLE PRECISION sin2b,cos2b,ct,cb,c2,ALSMT,ALSQ,MASM2
	DOUBLE PRECISION M1,M2,mu1,mu2,mu12,Lmu,LM2,Lmu1,Lmu2,Lmu12
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW,xst

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/HIGGSPEC/SMASS,SCOMP,PMASS,PCOMP,CMASS
	COMMON/SQSPEC/MST1,MST2,MSB1,MSB2

	IFAIL=0
	pi=4.*DATAN(1.D0)

*   Weak scale parameters

	g=(g1+g2)/2.
	C2TW=(MW/MZ)**2
	S2TW=1.-C2TW
	gam=g2*MZ**2/(16.*pi**2*C2TW)
	l=PAR(1)
	k=PAR(2)
	tanb=PAR(3)
	cos2b=1./(1.+tanb**2)
	sin2b=tanb**2*cos2b
	h1=DSQRT(sin2b/(2.*DSQRT(2.D0)*GF))
	h2=h1/tanb
	mu=PAR(4)
	Al=PAR(5)
	Ak=PAR(6)
	M1=PAR(20)
	M2=PAR(21)
	Mstop=(PAR(7)**2+PAR(8)**2)/2.
	dMst=(PAR(7)**2-PAR(8)**2)/2.
	At=PAR(12)
	Msbot=(PAR(7)**2+PAR(9)**2)/2.
	dMsb=(PAR(7)**2-PAR(9)**2)/2.
	Ab=PAR(13)
	nu=k/l*mu
	B=Al+nu

*   At MB

	hb=MB/DABS(h2)
	
*   At MT (ht, hb: running MS_bar couplings)

	ALSMT=ALSMZ/(1.+23./(12.*PI)*ALSMZ*DLOG(MT**2/MZ**2))
	ht=MT/(1.+4.*ALSMT/(3.*PI)+11.*(ALSMT/PI)**2)/h1
	hb=hb*(1.-23./(12.*PI)*ALSMT*DLOG(MT**2/MB**2))**(12./23.)

      write(*,*) alsmt,MT,ht,hb

*   At Q~Msusy

	Q2=(Mstop+Msbot)/2.+MT**2
	t=DLOG(Q2/MT**2)
	Zht=1.+3.*ht**2/(16.*pi**2)*t
	h1q=h1/DSQRT(Zht)
	Zhb=1.+3.*hb**2/(16.*pi**2)*t
	h2q=h2/DSQRT(Zhb)
	tanbq=h1q/h2q
	htq=ht*(1.+7./(4.*PI)*ALSMT*t)**(-4./7.)
     C	 *(1.+(9.*ht**2+hb**2)/(64.*PI**2)*t)
	hbq=hb*(1.+7./(4.*PI)*ALSMT*t)**(-4./7.)
     C	 *(1.+(9.*hb**2+ht**2)/(64.*PI**2)*t)
      write(*,*) tanbq
*   MASM2: ~MSSM-like pseudoscalar mass**2 (if >> MZ)

	MASM2= mu*B*(tanb + 1/tanb)
     
        IF(MASM2.GT.MT**2) THEN
	  htq=htq*(1.+2.*hb**2*dlog(MASM2/MT**2)/(64.*PI**2))
	  hbq=hbq*(1.+2.*ht**2*dlog(MASM2/MT**2)/(64.*PI**2))
        END IF
     
	mtopq=htq*h1q
	Xt=At-mu/tanbq
	mbotq=hbq*h2q
	Xb=Ab-mu*tanbq
	ALSQ=ALSMT/(1.+7.*ALSMT/(4.*pi)*t)

*    Stop masses

	Wt=DSQRT(dMst**2+mtopq**2*Xt**2)
	mst1=Mstop+mtopq**2-Wt
	mst2=Mstop+mtopq**2+Wt
	
*    Sbottom masses

	Wb=DSQRT(dMsb**2+mbotq**2*Xb**2)
	msb1=Msbot+mbotq**2-Wb
	msb2=Msbot+mbotq**2+Wb

	IF(mst1.LT.0. .OR. msb1.LT.0. .OR. mu.EQ.0.)IFAIL=8
	IF(IFAIL.NE.0)GOTO 10

	ct=3.*htq**2/(32.*pi**2)
	IF(Wt.NE.0.)THEN
	 s2t=mtopq*Xt/Wt
	 fmt=(mst2*DLOG(mst2/Q2)-mst1*DLOG(mst1/Q2))/(mst2-mst1)-1.
	 gmt=s2t**2*((mst2+mst1)/(mst2-mst1)*DLOG(mst2/mst1)-2.)
	 emt=mtopq*s2t*DLOG(mst2/mst1)
	ELSE
	 s2t=DSIGN(1.D0,Xt)
	 fmt=DLOG(mst1/Q2)
	 gmt=0.
	 emt=0.
	ENDIF

	cb=3.*hbq**2/(32.*pi**2)
	IF(Wb.NE.0.)THEN
	 s2b=mbotq*Xb/Wb
	 fmb=(msb2*DLOG(msb2/Q2)-msb1*DLOG(msb1/Q2))/(msb2-msb1)-1.
	 gmb=s2b**2*((msb2+msb1)/(msb2-msb1)*DLOG(msb2/msb1)-2.)
	 emb=mbotq*s2b*DLOG(msb2/msb1)
	ELSE
	 s2b=DSIGN(1.D0,Xb)
	 fmb=DLOG(msb1/Q2)
	 gmb=0.
	 emb=0.
	ENDIF

	Al=Al+2.*ct*At*fmt+2.*cb*Ab*fmb
	
*   Tree level CP-even Higss mass matrix

	nu=k/l*mu
	B=Al+nu
	mdia(1) = g*h1q**2 + mu*B/tanbq
	mdia(2) = g*h2q**2 + mu*B*tanbq
	mdia(3) = 4.*nu**2 + l**2*Al*h1q*h2q/mu + Ak*nu
	moff(1) = (2.*l**2-g)*h1q*h2q - mu*B
	moff(2) = l*(2.*mu*h1q - (B+nu)*h2q)
	moff(3) = l*(2.*mu*h2q - (B+nu)*h1q)

*   1-loop radiative corrections

	rdiat(1) = - At**2*gmt + 4.*At*emt
     C		  + 4.*mtopq**2*DLOG(mst1*mst2/mtopq**4)
	rdiat(2) = - mu**2*gmt 
	rdiat(3) = -l**2*h2q**2*gmt
	rofft(1) = mu*(At*gmt - 2.*emt)	
	rofft(2) = l*h2q*(At*gmt - 2.*emt)
	rofft(3) = - l*h2q*mu*gmt + 4.*l*h2q*mu*fmt

	rdiab(1) = - mu**2*gmb
	rdiab(2) =  - Ab**2*gmb + 4.*Ab*emb
     C		  + 4.*mbotq**2*DLOG(msb1*msb2/mbotq**4)
	rdiab(3) = -l**2*h1q**2*gmb	
	roffb(1) = mu*(Ab*gmb - 2.*emb)	
	roffb(2) = - l*h1q*mu*gmb + 4.*l*h1q*mu*fmb
	roffb(3) = l*h1q*(Ab*gmb - 2.*emb)
	
	DO I=1,3
	 mdia(i) = mdia(i) + ct*rdiat(i)+ cb*rdiab(i)
	 moff(i) = moff(i) + ct*rofft(i)+ cb*roffb(i)
	ENDDO

*   2-loop terms

c	c2 = 3.*htq**4/(64.*pi**4)*(32.*pi*ALSQ-3.*htq**2/2.)
c	rdiat(1) = h1q**2*t**2
c	mdia(1) = mdia(1) + c2*rdiat(1)

*   Back to the weak scale

	mdia(1) = mdia(1)/Zht
	mdia(2) = mdia(2)/Zhb
	moff(1) = moff(1)/DSQRT(Zht*Zhb)
	moff(2) = moff(2)/DSQRT(Zht)
	moff(3) = moff(3)/DSQRT(Zhb)

*   Leading-log electroweak contributions (1 loop), 
*   without charginos/neutralinos

c	mdia(1) = mdia(1) + gam*t*sin2b*(4.-8.*S2TW+32./3.*S2TW**2)
c	mdia(2) = mdia(2) + gam*t*cos2b*(4.-8.*S2TW+32./3.*S2TW**2)
c	moff(1) = moff(1) + gam*t*DSQRT(sin2b*cos2b)
c     C                     *(-4.+8.*S2TW-32./3.*S2TW**2)

*   Large electroweak logs ~ ln(MA/MZ) neglecting terms ~ lambda**2 

c	IF(MASM2.GT.MZ**2)THEN
c	 mdia(1) = mdia(1) + gam/6.*sin2b*DLOG(MASM2/MZ**2)
c     C    *(-9.*(cos2b-sin2b)**4+(1.-2.*S2TW+2.*S2TW**2)
c     C    *(cos2b-sin2b)**2+10.-2.*S2TW+2.*S2TW**2)
c     	 mdia(2) = mdia(2) + gam/6.*cos2b*DLOG(MASM2/MZ**2)
c     C    *(-9.*(cos2b-sin2b)**4+(1.-2.*S2TW+2.*S2TW**2)
c     C    *(cos2b-sin2b)**2+10.-2.*S2TW+2.*S2TW**2)
c     	 moff(1) = moff(1) - gam/6.*DSQRT(cos2b*sin2b)*DLOG(MASM2/MZ**2)
c     C    *(-9.*(cos2b-sin2b)**4+(1.-2.*S2TW+2.*S2TW**2)
c     C    *(cos2b-sin2b)**2+8.-22.*S2TW+10.*S2TW**2)
c        ENDIF

*   Chargino/neutralino contributions ~ ln(M1/MZ), ln(M2/MZ), ln(mu/MZ)
*   neglecting terms ~ lambda**2, kappa**2 and mixings

	mu1 = MAX(DABS(mu),M1)
	mu2 = MAX(DABS(mu),M2)
	mu12 = MAX(mu1,mu2)
	Lmu = DLOG((mu/MZ)**2)
	LM2 = DLOG((M2/MZ)**2)
	Lmu1 = DLOG((mu1/MZ)**2)
	Lmu2 = DLOG((mu2/MZ)**2)
	Lmu12 = DLOG((mu12/MZ)**2)

c	mdia(1) = mdia(1) + gam*sin2b
c     C	 *(S2TW*(1.-2.*S2TW)*Lmu1 - 4.*S2TW*C2TW*Lmu12
c     C	 + C2TW*(3.-10.*C2TW)*Lmu2 - 4./3.*C2TW**2*LM2
c     C	 - 2./3.*(S2TW**2+C2TW**2)*Lmu)
c	mdia(2) = mdia(2) + gam*cos2b
c     C	 *(S2TW*(1.-2.*S2TW)*Lmu1 - 4.*S2TW*C2TW*Lmu12
c     C	 + C2TW*(3.-10.*C2TW)*Lmu2 - 4./3.*C2TW**2*LM2
c     C	 - 2./3.*(S2TW**2+C2TW**2)*Lmu)
c	moff(1) = moff(1) - gam*DSQRT(sin2b*cos2b)
c     C	 *(S2TW*(1.+2.*S2TW)*Lmu1 + 4.*S2TW*C2TW*Lmu12
c     C	 + C2TW*(3.+2.*C2TW)*Lmu2 - 4./3.*C2TW**2*LM2
c     C	 - 2./3.*(S2TW**2+C2TW**2)*Lmu)

*   Diagonalization

	CALL DIAG3(mdia,moff,SMASS,SCOMP)

*  For heavy Higgses: Up to the scale Mhiggs (-> pole mass)

	DO I=1,3
	 IF(SMASS(I).GT.MT**2)THEN
	  SMASS(I) = SMASS(I)/
     C	   (1.-(2.*ct*SCOMP(I,1)**2+2.*cb*SCOMP(I,2)**2)
     C	   *DLOG(SMASS(I)/MT**2))
	 ENDIF
	ENDDO

	IF(SMASS(1).LT.1.)THEN
	 IFAIL=1
	ELSE
	 DO I=1,3
	  SMASS(I)=DSQRT(SMASS(I))
	 ENDDO
	ENDIF

*   CP-odd Higgs mass matrix including
*   1-loop radiative corrections

	mp(1) = mu*B*(h1q/(Zhb*h2q)+h2q/(Zht*h1q))
	mp(2) = l**2*h1q*h2q/mu*(3.*nu+B)-3.*Ak*nu
	mp(3) = l*(B-3.*nu)*DSQRT(h1q**2/Zhb+h2q**2/Zht)

*   Diagonalization

	CALL DIAG2(mp,PMASS,PCOMP)

*  For heavy Higgses: Up to the scale Mhiggs (-> pole mass)

	DO I=1,2
	 IF(PMASS(I).GT.MT**2)THEN
	  PMASS(I)=PMASS(I)/
     C	   (1.-(2.*ct*PCOMP(I,1)**2*cos2b+2.*cb*PCOMP(I,1)**2*sin2b)
     C	   *DLOG(PMASS(I)/MT**2))
	 ENDIF
	ENDDO

	IF(PMASS(1).LT.1.)THEN
	 IFAIL=IFAIL+2
	ELSE
	 DO I=1,2
	  PMASS(I)=DSQRT(PMASS(I))
	 ENDDO
	ENDIF

*   Charged Higgs mass including 1-loop radiative corrections

        CMASS = ((g2/2.-l**2)*h1q*h2q+mu*B)
     C	      *(h1q**2*Zht+h2q**2*Zhb)/(h1q*h2q*Zht*Zhb)
     C	      +3.*ht**2*hb**2/(16.*DSQRT(2.D0)*PI**2*GF)*t
     C	      +g2*MW**2/(48.*PI**2)*(12.*t+3.*S2TW/C2TW*(Lmu1+4.*Lmu12)
     C	      -3.*Lmu2-4.*LM2-2.*Lmu)
     
*  For heavy Higgses: Up to the scale Mhiggs (-> pole mass)

	IF(CMASS.GT.MT**2)THEN
	 CMASS = CMASS/
     C	  (1.-(2.*ct*cos2b+2.*cb*sin2b)
     C	  *DLOG(CMASS/MT**2))
	ENDIF

	IF(CMASS.LT.1.)THEN
	 IFAIL=IFAIL+4
	ELSE
	 CMASS=DSQRT(CMASS)
	ENDIF
	
*  Pole Squark Masses (DR_bar)

	xst=PAR(22)**2/MST1
	MST1=DSQRT(MST1*(1.+2.*ALSQ/(3.*PI)*
     C  (1.+3.*xst+1./2.*(xst-1)**2*DLOG((xst-1)**2)-xst**2*DLOG(xst))))
	xst=PAR(22)**2/MST2
	MST2=DSQRT(MST2*(1.+2.*ALSQ/(3.*PI)*
     C  (1.+3.*xst+1./2.*(xst-1)**2*DLOG((xst-1)**2)-xst**2*DLOG(xst))))
	xst=PAR(22)**2/MSB1
	MSB1=DSQRT(MSB1*(1.+2.*ALSQ/(3.*PI)*
     C  (1.+3.*xst+1./2.*(xst-1)**2*DLOG((xst-1)**2)-xst**2*DLOG(xst))))
	xst=PAR(22)**2/MSB2
	MSB2=DSQRT(MSB2*(1.+2.*ALSQ/(3.*PI)*
     C  (1.+3.*xst+1./2.*(xst-1)**2*DLOG((xst-1)**2)-xst**2*DLOG(xst))))
	
 10	END


	SUBROUTINE DIAG3(mdia,moff,Smass,Scomp)

*   Subroutine to diagonalize the 3x3 CP even Higgs mass matrix

	IMPLICIT NONE

	INTEGER i,j

	DOUBLE PRECISION a1,a2,a3,p,q,D,phi,ang,norm,pi,X
	DOUBLE PRECISION mdia(3),moff(3),Smass(3),Scomp(3,3)

	IF(moff(1).EQ.0. .AND. moff(2).EQ.0. .AND. moff(3).EQ.0.)THEN

	 Smass(1)=mdia(1)
	 Scomp(1,1)=1.
	 Scomp(1,2)=0.
	 Scomp(1,3)=0.
	 Smass(2)=mdia(2)
	 Scomp(2,1)=0.
	 Scomp(2,2)=1.
	 Scomp(2,3)=0.
	 Smass(3)=mdia(3)
	 Scomp(3,1)=0.
	 Scomp(3,2)=0.
	 Scomp(3,3)=1.
	 
	ELSEIF(moff(1).EQ.0. .AND. moff(2).EQ.0.)THEN

	 Smass(1)=mdia(1)
	 Scomp(1,1)=1.
	 Scomp(1,2)=0.
	 Scomp(1,3)=0.
	 a1=mdia(2)+mdia(3)
	 a2=mdia(2)*mdia(3)-moff(3)**2
	 D=DSQRT(a1**2-4.*a2)
	 Smass(2)=(a1-D)/2.
	 Smass(3)=(a1+D)/2.
	 Scomp(2,1)=0.
	 Scomp(2,2)=moff(3)
	 Scomp(2,3)=Smass(2)-mdia(2)
	 Scomp(3,1)=0.
	 Scomp(3,2)=Smass(3)-mdia(3)
	 Scomp(3,3)=moff(3)
	 DO i=2,3
	  norm=DSQRT(Scomp(i,2)**2+Scomp(i,3)**2)
	  Scomp(i,2)=Scomp(i,2)/norm
	  Scomp(i,3)=Scomp(i,3)/norm
	 ENDDO

	ELSEIF(moff(1).EQ.0. .AND. moff(3).EQ.0.)THEN

	 Smass(1)=mdia(2)
	 Scomp(1,1)=0.
	 Scomp(1,2)=1.
	 Scomp(1,3)=0.
	 a1=mdia(1)+mdia(3)
	 a2=mdia(1)*mdia(3)-moff(2)**2
	 D=DSQRT(a1**2-4.*a2)
	 Smass(2)=(a1-D)/2.
	 Smass(3)=(a1+D)/2.
	 Scomp(2,1)=moff(2)
	 Scomp(2,2)=0.
	 Scomp(2,3)=Smass(2)-mdia(1)
	 Scomp(3,1)=Smass(3)-mdia(3)
	 Scomp(3,2)=0.
	 Scomp(3,3)=moff(2)
	 DO i=2,3
	  norm=DSQRT(Scomp(i,1)**2+Scomp(i,3)**2)
	  Scomp(i,1)=Scomp(i,1)/norm
	  Scomp(i,3)=Scomp(i,3)/norm
	 ENDDO

	ELSEIF(moff(2).EQ.0. .AND. moff(3).EQ.0.)THEN

	 Smass(1)=mdia(3)
	 Scomp(1,1)=0.
	 Scomp(1,2)=0.
	 Scomp(1,3)=1.
	 a1=mdia(1)+mdia(2)
	 a2=mdia(1)*mdia(2)-moff(1)**2
	 D=DSQRT(a1**2-4.*a2)
	 Smass(2)=(a1-D)/2.
	 Smass(3)=(a1+D)/2.
	 Scomp(2,1)=moff(1)
	 Scomp(2,2)=Smass(2)-mdia(1)
	 Scomp(2,3)=0.
	 Scomp(3,1)=Smass(3)-mdia(2)
	 Scomp(3,2)=moff(1)
	 Scomp(3,3)=0.
	 DO i=2,3
	  norm=DSQRT(Scomp(i,1)**2+Scomp(i,2)**2)
	  Scomp(i,1)=Scomp(i,1)/norm
	  Scomp(i,2)=Scomp(i,2)/norm
	 ENDDO

	ELSE

	pi=4.*DATAN(1.D0)
	a1=-(mdia(1)+mdia(2)+mdia(3))
	a2=mdia(1)*mdia(2)+mdia(1)*mdia(3)+mdia(2)*mdia(3)
     c	     -moff(1)**2-moff(2)**2-moff(3)**2
	a3=mdia(1)*moff(3)**2+mdia(2)*moff(2)**2+mdia(3)*moff(1)**2
     c	     -mdia(1)*mdia(2)*mdia(3)-2.*moff(1)*moff(2)*moff(3)
	p=(3.*a2-a1**2)/9.
	q=a1**3/27.-a1*a2/6.+a3/2.
	D=DSIGN(DSQRT(-p),q)
	phi=q/D**3
	IF(phi.GT.1.)phi=1.
	IF(phi.LT.-1.)phi=-1.
	phi=DACOS(phi)/3.
	ang=2.*pi/3.
	DO i=1,3
	 Smass(i)=-2.*D*DCOS(phi+DFLOAT(i)*ang)-a1/3.
	ENDDO

	 DO i=1,3
	  Scomp(i,1)=moff(1)*moff(3)-(mdia(2)-Smass(i))*moff(2)
	  Scomp(i,2)=moff(1)*moff(2)-(mdia(1)-Smass(i))*moff(3)
	  Scomp(i,3)=(mdia(1)-Smass(i))*(mdia(2)-Smass(i))-moff(1)**2
	 norm=DSQRT(Scomp(i,1)**2+Scomp(i,2)**2+Scomp(i,3)**2)
	 DO j=1,3
	  Scomp(i,j)=Scomp(i,j)/norm
	 ENDDO
	 ENDDO

	ENDIF

	IF(Smass(1).GT.Smass(2))THEN
	 X=Smass(2)
	 Smass(2)=Smass(1)
	 Smass(1)=X
	 DO I=1,3
	  X=Scomp(2,I)
	  Scomp(2,I)=Scomp(1,I)
	  Scomp(1,I)=X
	 ENDDO
	ENDIF
	IF(Smass(2).GT.Smass(3))THEN
	 X=Smass(3)
	 Smass(3)=Smass(2)
	 Smass(2)=X
	 DO I=1,3
	  X=Scomp(3,I)
	  Scomp(3,I)=Scomp(2,I)
	  Scomp(2,I)=X
	 ENDDO
	 IF(Smass(1).GT.Smass(2))THEN
	  X=Smass(2)
	  Smass(2)=Smass(1)
	  Smass(1)=X
	  DO I=1,3
	   X=Scomp(2,I)
	   Scomp(2,I)=Scomp(1,I)
	   Scomp(1,I)=X
	  ENDDO
	 ENDIF
	ENDIF

	END

	SUBROUTINE DIAG2(mp,Pmass,Pcomp)

*   Subroutine to diagonalize the 2x2 CP odd Higgs mass matrix

	IMPLICIT NONE

	INTEGER i

	DOUBLE PRECISION a1,a2,mp(3),norm,D
	DOUBLE PRECISION Pmass(2),Pcomp(2,2)

	IF(mp(3).EQ.0)THEN
	 IF(mp(1).LE.mp(2))THEN
	  Pmass(1)=mp(1)
	  Pmass(2)=mp(2)
	  Pcomp(1,1)=1.
	  Pcomp(1,2)=0.
	  Pcomp(2,1)=0.
	  Pcomp(2,2)=1.
	 ELSE
	  Pmass(1)=mp(2)
	  Pmass(2)=mp(1)
	  Pcomp(1,1)=0.
	  Pcomp(1,2)=1.
	  Pcomp(2,1)=-1.
	  Pcomp(2,2)=0.
	 ENDIF
	ELSE
	 a1=mp(1)+mp(2)
	 a2=mp(1)*mp(2)-mp(3)**2
	 D=DSQRT(a1**2-4.*a2)
	 Pmass(1)=(a1-D)/2.
	 Pmass(2)=(a1+D)/2.
	 Pcomp(1,1)=mp(3)
	 Pcomp(1,2)=Pmass(1)-mp(1)
	 Pcomp(2,1)=Pmass(2)-mp(2)
	 Pcomp(2,2)=mp(3)
	 DO i=1,2
	  norm=DSQRT(Pcomp(i,1)**2+Pcomp(i,2)**2)
	  Pcomp(i,1)=Pcomp(i,1)/norm
	  Pcomp(i,2)=Pcomp(i,2)/norm
	 ENDDO
	ENDIF

	END


	SUBROUTINE CHARGINO(PAR)

	IMPLICIT NONE

	INTEGER I

	DOUBLE PRECISION PAR(*),MCH(2),U(2,2),V(2,2),MNEU(5),N(5,5)
	DOUBLE PRECISION TanB,mu,M2,g2v1,g2v2,TrX2,detX,D,tU,tV
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION CU,SU,CV,SV

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/SUSYSPEC/MCH,U,V,MNEU,N

	TanB=PAR(3)
	mu=PAR(4)
	M2=PAR(21)
	g2v2=DSQRT(2./(1.+TanB**2))*MW
	g2v1=g2v2*TanB

	TrX2=2.*MW**2+M2**2+mu**2
	detX=mu*M2-2.*MW**2*g2v1*g2v2/(g2v1**2+g2v2**2)
	D=TrX2**2-4.*detX**2

	tU=(g2v2**2-g2v1**2-M2**2+mu**2-DSQRT(D))/2./(M2*g2v2+mu*g2v1)
	tV=(g2v1**2-g2v2**2-M2**2+mu**2-DSQRT(D))/2./(M2*g2v1+mu*g2v2)

	CU=DCOS(DATAN(tU))
	SU=DSIN(DATAN(tU))
	CV=DCOS(DATAN(tV))
	SV=DSIN(DATAN(tV))

	U(1,1)=CU
	U(1,2)=SU
	U(2,1)=-SU
	U(2,2)=CU

	V(1,1)=CV
	V(1,2)=SV
	V(2,1)=-SV
	V(2,2)=CV

	DO I=1,2
	 MCH(I)=g2v1*U(I,1)*V(I,2)+g2v2*U(I,2)*V(I,1)
     C	  +M2*U(I,1)*V(I,1)+mu*U(I,2)*V(I,2)
	ENDDO

	END


	SUBROUTINE NEUTRALINO(PAR)

	IMPLICIT NONE

	INTEGER I,J
	DOUBLE PRECISION PAR(*),MCHA(2),U(2,2),V(2,2),MNEU(5),NEU(5,5)
	DOUBLE PRECISION VALP(5),VECP(5,5),EPS
	DOUBLE PRECISION l,k,TanB,mu,M1,M2,v1,v2
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW

	COMMON/SUSYSPEC/MCHA,U,V,MNEU,NEU

	EPS=1.D-8

	l=PAR(1)
	k=PAR(2)
	TanB=PAR(3)
	mu=PAR(4)
	M1=PAR(20)
	M2=PAR(21)
	v2=1./DSQRT(2.*DSQRT(2.D0)*(1.+TanB**2)*GF)
	v1=v2*TanB

        NEU(1,1)=M1
        NEU(1,2)=0.
        NEU(1,3)=DSQRT(g1/2.)*v1
        NEU(1,4)=-DSQRT(g1/2.)*v2
        NEU(1,5)=0.
        NEU(2,2)=M2
        NEU(2,3)=-DSQRT(g2/2.)*v1
        NEU(2,4)=DSQRT(g2/2.)*v2
        NEU(2,5)=0.
        NEU(3,3)=0.
        NEU(3,4)=-mu
        NEU(3,5)=-l*v2
        NEU(4,4)=0.
        NEU(4,5)=-l*v1
        NEU(5,5)=2.*k/l*mu
	CALL DIAGN(5,NEU,VALP,VECP,EPS)
	CALL SORTN(5,VALP,VECP)
	DO I=1,5
	 MNEU(I)=VALP(I)
	 DO J=1,5
	  NEU(I,J)=VECP(J,I)
	 ENDDO
	ENDDO

	END

	SUBROUTINE DIAGN(N,A,D,V,EPS)

*   Diagonalization of a real symmetric NxN matrix
*   Using the Jacobi method (from Numerical Recipes)

	IMPLICIT NONE

	INTEGER N,I,J,IP,IQ,NMAX
	PARAMETER(NMAX=500)

	DOUBLE PRECISION A(N,N),D(N),V(N,N),B(N),Z(N)
	DOUBLE PRECISION EPS,SM,THR,G,H,C,S,T,THETA,TAU

	DO IP=1,N
	 DO IQ=1,N
	  V(IP,IQ)=0.
	 ENDDO
	 V(IP,IP)=1.
	 B(IP)=A(IP,IP)
	 D(IP)=B(IP)
	 Z(IP)=0.
	ENDDO

	DO I=1,NMAX

	 SM=0.
	 DO IP=1,N-1
	  DO IQ=IP+1,N
	   SM=SM+DABS(A(IP,IQ))
	  ENDDO
	 ENDDO
	 IF(SM.LE.EPS)RETURN

	 IF(I.LT.4)THEN
	  THR=.2D0*SM/N**2
	 ELSE
	  THR=0.
	 ENDIF

	 DO IP=1,N-1
	 DO IQ=IP+1,N
	  IF(DABS(A(IP,IQ)).LT.EPS*MIN(DABS(D(IP)),DABS(D(IQ)))
     .	     .AND. I.GT.4)THEN
	   A(IP,IQ)=0.
	  ELSEIF(DABS(A(IP,IQ)).GT.THR)THEN
	   H=D(IQ)-D(IP)
	   IF(DABS(A(IP,IQ)).LT.EPS*DABS(H))THEN
	    T=A(IP,IQ)/H
	   ELSE
	    THETA=.5D0*H/A(IP,IQ)
	    T=1./(DABS(THETA)+DSQRT(1.+THETA**2))
	    IF(THETA.LT.0.)T=-T
	   ENDIF
	   C=1./DSQRT(1.+T**2)
	   S=T*C
	   TAU=S/(1.+C)
	   H=T*A(IP,IQ)
	   Z(IP)=Z(IP)-H
	   Z(IQ)=Z(IQ)+H
	   D(IP)=D(IP)-H
	   D(IQ)=D(IQ)+H
	   A(IP,IQ)=0.
	   DO J=1,IP-1
	    G=A(J,IP)
	    H=A(J,IQ)
	    A(J,IP)=G-S*(H+G*TAU)
	    A(J,IQ)=H+S*(G-H*TAU)
	   ENDDO
	   DO J=IP+1,IQ-1
	    G=A(IP,J)
	    H=A(J,IQ)
	    A(IP,J)=G-S*(H+G*TAU)
	    A(J,IQ)=H+S*(G-H*TAU)
	   ENDDO
	   DO J=IQ+1,N
	    G=A(IP,J)
	    H=A(IQ,J)
	    A(IP,J)=G-S*(H+G*TAU)
	    A(IQ,J)=H+S*(G-H*TAU)
	   ENDDO
	   DO J=1,N
	    G=V(J,IP)
	    H=V(J,IQ)
	    V(J,IP)=G-S*(H+G*TAU)
	    V(J,IQ)=H+S*(G-H*TAU)
	   ENDDO
	  ENDIF
	 ENDDO
	 ENDDO

	 DO IP=1,N
	  B(IP)=B(IP)+Z(IP)
	  D(IP)=B(IP)
	  Z(IP)=0.
	 ENDDO

	ENDDO

	RETURN
	END


	SUBROUTINE SORTN(N,D,V)

*   Reordering of the absolute value of the eigenvalues D(I), I=1..N
*   and corresponding eigenvectors V(J,I), J=1..N in ascending order
*   (from Numerical Recipes)

	IMPLICIT NONE 

	INTEGER N,I,J,K
	DOUBLE PRECISION D(N),V(N,N),P

	DO I=1,N-1
	 K=I
	 P=D(I)
	 DO J=I+1,N
	  IF(DABS(D(J)).LT.DABS(P))THEN
	   K=J
	   P=D(J)
	  ENDIF
	 ENDDO
	 IF(K.NE.I)THEN
	  D(K)=D(I)
	  D(I)=P
	  DO J=1,N
	   P=V(J,I)
	   V(J,I)=V(J,K)
	   V(J,K)=P
	  ENDDO
	 ENDIF
	ENDDO

	RETURN
	END


	SUBROUTINE DECAY(PAR)

*   Subroutine to calculate the Higgs bosons BRs
*
*	CU,CD,CV,CG,CGA(i)	Reduced couplings of h1,h2,h3 (i=1,2,3) or
*				a1,a2 (i=4,5) to up type fermions, down type
*				fermions, gauge bosons, gluons and photons
*				Note: CV(4,5)=0
*
*	WIDTH(i) Total decay width of h1,h2,h3,a1,a2 (i=1..5)
*		with the following branching ratios:
*	BRJJ(i)  h1,h2,h3,a1,a2	-> gluon gluon
*	BRMM(i)		"	-> mu mu
*	BRLL(i)		"	-> tau tau
*	BRSS(i)		"	-> ss
*	BRCC(i)		"	-> cc
*	BRBB(i)		"	-> bb
*	BRTT(i)		"	-> tt
*	BRWW(i)		"	-> WW (BRWW(4,5)=0)
*	BRZZ(i)		"	-> ZZ (BRZZ(4,5)=0)
*	BRGG(i)		"	-> gamma gamma
*	BRZG(i)		"	-> Z gamma
*	BRHIGGS(i)	"	-> other Higgses, including:
*		BRHAA(i,j)	hi -> a1a1, a1a2, a2a2 (j=1,2,3)
*		BRHCHC(i)	hi -> h+h-
*		BRHAZ(i,j)	hi -> Zaj
*		BRHCW(i)	h1,h2,h3 -> h+W- (i=1,3), a1,a2 -> h+W- (i=1,2)
*		BRHHH(i)	h2 -> h1h1, h3-> h1h1, h1h2, h2h2 (i=1,2,3,4)
*		BRAHA(i)	a2 -> a1hi
*		BRAHZ(i,j)	ai -> Zhj
*	BRSUSY(i)	"	-> susy particles, including:
*		BRNEUNEU(i,j,k)	-> neutralinos j,k
*		BRCHACHA(i,j)	-> charginos 11, 12, 22 (j=1,2,3)
*
*	HCWIDTH		Total decay width of the charged Higgs
*		       with the following branching ratios:
*	HCBRM     h+ ->	mu nu_mu
*	HCBRL		tau nu_tau
*	HCBRSU		s u
*	HCBRBU		b u
*	HCBRSC		s c
*	HCBRBC		b c
*	HCBRBT		b t
*	HCBRWHT		neutral Higgs W+, including:
*		HCBRWH(i)	h1W+, h2W+, h3W+, a1W+, a2W+ (i=1..5)
*	HCBRSUSYT	susy particles,including
*		HCBRSUSY(i,j)	neutalino i chargino j
*
	IMPLICIT NONE

	INTEGER A,B,D,I,J,K,N0,NF,NFEXT,NFGG

	DOUBLE PRECISION MH,MA,PAR(*),SMASS(3),S(3,3)
	DOUBLE PRECISION PMASS(2),P2(2,2),P(2,3),CMASS,C(2)
	DOUBLE PRECISION CU(5),CD(5),CV(3),CG(5),CGA(5),RATCOUP,RAT
	DOUBLE PRECISION PI,HIGTOP,ASG,ASH,AS3,AS4,ASMB,ASMT
	DOUBLE PRECISION EPS,FQCD,XFAC,X,Y
	DOUBLE PRECISION HJJ,HMM,HLL,HSS,HCC,HBB,HTT,HWW,HZZ,HGG,HZG
	DOUBLE PRECISION DCC,DBB,HS1,HS2,HC1,HC2,HB1,HB2,HT1,HT2
	DOUBLE PRECISION DLD,DLU,XM1,XM2,CWW,CZZ,XX(4),YY(4)
	DOUBLE PRECISION HHH(4),HAA(3),HHA(3),HHCHC,HAZ(2),HHCW
	DOUBLE PRECISION HHZ(3),HNEU(5,5),HCHA(3),HTOT,STOT
	DOUBLE PRECISION BRJJ(5),BRMM(5),BRLL(5),BRSS(5),BRCC(5)
	DOUBLE PRECISION BRBB(5),BRTT(5),BRWW(3),BRZZ(3),BRGG(5),BRZG(5)
	DOUBLE PRECISION BRHHH(4),BRHAA(3,3),BRHCHC(3),BRHAZ(3,2)
	DOUBLE PRECISION BRAHA(3),BRAHZ(2,3),BRHCW(5),BRHIGGS(5)
	DOUBLE PRECISION BRNEU(5,5,5),BRCHA(5,3),BRSUSY(5),WIDTH(5)
	DOUBLE PRECISION GHHH,GHAA,GHCC,RH,CH
	DOUBLE PRECISION GHNEUNEU,GANEUNEU,GHCHACHA,GACHACHA
	DOUBLE PRECISION GHCNEUCHAL,GHCNEUCHAR
	DOUBLE PRECISION MCHA(2),U(2,2),V(2,2),MNEU(5),N(5,5)
	DOUBLE PRECISION TANBETA,SINBETA,COSBETA
	DOUBLE PRECISION XLAMBDA,MC0,MB0,MT0,RMS,RMC,RMB,RMT,MBP
	DOUBLE PRECISION SQR2,LAMBDA,KAPPA,ALAMBDA,AKAPPA,h1,h2,ss
	DOUBLE PRECISION HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC
	DOUBLE PRECISION HCBRBT,HCBRWH(5),HCBRWHT,HCBRSUSY(5,2)
	DOUBLE PRECISION HCBRSUSYT,HCWIDTH
	DOUBLE PRECISION HMN,HLN,HSU,HSU1,HSU2,HSC,HSC1,HSC2
	DOUBLE PRECISION HBC,HBC1,HBC2,HBU,HBU1,HBU2
	DOUBLE PRECISION HBT,HBT1,HBT2,HCWH(5),HCSUSY(5,2)
	DOUBLE PRECISION VUS,VCB,VUB,S2TW,C2TW,T2TW
	DOUBLE PRECISION HVV,HV,HFF,QCD0,HQCDM,HQCD,QCDH,TQCDH,HGGQCD
	DOUBLE PRECISION AFF,AQCDM,AQCD,QCDA,TQCDA,AGGQCD
	DOUBLE PRECISION QCDC,QCDCM,CQCD,CQCDM,QCDCI,QCDCMI
	DOUBLE PRECISION BETA,LAMB,SP,ALPHAS,RUNM,QQINT,FINT
	DOUBLE PRECISION T,Z,XI,BIJ,CFF,PIHHH,PIHAA,PIHCC
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION LQ,RADT,RADB,RADTB,FT,FB,ACOUP	

	DOUBLE COMPLEX CTT,CTB,CTC,CTL,CTW,CTHC,CTCH1,CTCH2,CF
	DOUBLE COMPLEX CAT,CAB,CAC,CAL,CAW,CAHC,CACH1,CACH2,CA,CB
	DOUBLE COMPLEX CLT,CLB,CLW,CLH
	DOUBLE COMPLEX CI1,CI2,CGZ,CATZ,CABZ,CAWZ,CAHZ

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/CKM/VUS,VCB,VUB
	COMMON/ALS/XLAMBDA,MC0,MB0,MT0,N0
	COMMON/BRN/BRJJ,BRMM,BRLL,BRSS,BRCC,BRBB,BRTT,BRWW,BRZZ,BRGG,
     C		BRZG,BRHHH,BRHAA,BRHCHC,BRHAZ,BRAHA,BRAHZ,BRHCW,
     C		BRHIGGS,BRNEU,BRCHA,BRSUSY,WIDTH
	COMMON/BRC/HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC,
     C		HCBRBT,HCBRWH,HCBRWHT,HCBRSUSY,HCBRSUSYT,HCWIDTH
	COMMON/REDCOUP/CU,CD,CV,CG,CGA
	COMMON/HIGGSPEC/SMASS,S,PMASS,P2,CMASS
	COMMON/SUSYSPEC/MCHA,U,V,MNEU,N

	QQINT(RAT,X,Y)= RAT**2*X+(1.-RAT**2)*Y
	BETA(X)= DSQRT(1.-4.*X)
	LAMB(X,Y)= DSQRT((1.-X-Y)**2-4.*X*Y)
	CF(CA)= -CDLOG(-(1.+CDSQRT(1.-CA))/(1.-CDSQRT(1.-CA)))**2/4.
	CGZ(CA)= CDSQRT(1.-CA)/2.*CDLOG(-(1.+CDSQRT(1.-CA))
     .	 / (1.-CDSQRT(1.-CA)))
	CI1(CA,CB)= CA*CB/2./(CA-CB)
     .	 + CA**2*CB**2/2/(CA-CB)**2*(CF(CA)-CF(CB))
     .	 + CA**2*CB/(CA-CB)**2*(CGZ(CA)-CGZ(CB))
	CI2(CA,CB)= -CA*CB/2./(CA-CB)*(CF(CA)-CF(CB))
	HV(X)= 3.*(1.-8.*X+20.*X**2)/DSQRT((4.*X-1.))
     .	 * DACOS((3.*X-1.)/2./DSQRT(X**3))
     .	 - (1.-X)*(47./2.*X-13./2.+1./X)
     .	 - 3./2.*(1.-6.*X+4.*X**2)*DLOG(X)
	HVV(X,Y)= GF/(4.*PI*SQR2)*X**3/2.*BETA(Y)
     .	 * (1.-4.*Y+12.*Y**2)
	HFF(X,Y)= GF/(4.*PI*SQR2)*X**3*Y*(BETA(Y))**3
	AFF(X,Y)= GF/(4.*PI*SQR2)*X**3*Y*(BETA(Y))
	CFF(Z,T,X,Y)= GF/(4.*PI*SQR2)*Z**3*LAMB(X,Y)
     .	 * ((1.-X-Y)*(X*T**2+Y/T**2)-4.*X*Y)
	QCD0(X)= (1.+X**2)*(4.*SP((1.-X)/(1.+X))+2.*SP((X-1.)/(X+1.))
     .	 - 3.*DLOG((1.+X)/(1.-X))*DLOG(2./(1.+X))
     .	 - 2.*DLOG((1.+X)/(1.-X))*DLOG(X))
     .	 - 3.*X*DLOG(4./(1.-X**2))-4.*X*DLOG(X)
	HQCDM(X)= QCD0(X)/X+(3.+34.*X**2-13.*X**4)/16./X**3
     .	 * DLOG((1.+X)/(1.-X))+3./8./X**2*(7.*X**2-1.)
	HQCD(X)= 5.67D0*ASH/PI
     .	 + (29.14D0+RATCOUP*(1.570D0-2.*DLOG(HIGTOP)/3.
     .	 + DLOG(X)**2/9.))*(ASH/PI)**2
     .	 + (164.14D0-25.77D0*5.+0.259D0*5.**2)*(ASH/PI)**3
	QCDH(X)= 1.+HQCD(X)
	TQCDH(X)= 1.+4./3.*HQCDM(BETA(X))*ASH/PI
	AQCDM(X)= QCD0(X)/X+(19.+2.*X**2+3.*X**4)/16./X
     .	 * DLOG((1.+X)/(1.-X))+3./8.*(7.-X**2)
	AQCD(X)= 5.67D0*ASH/PI
     .	 + (29.14D0+RATCOUP*(3.83D0-DLOG(HIGTOP)
     .	 + DLOG(X)**2/6.))*(ASH/PI)**2
     .	 + (164.14D0-25.77D0*5.+0.259D0*5.**2)*(ASH/PI)**3
	QCDA(X)= 1.+AQCD(X)
	TQCDA(X)= 1.+4./3.*AQCDM(BETA(X))*ASH/PI
	HGGQCD(ASG,NF)= 1.+ASG/PI*(95./4.-NF*7./6.)
	AGGQCD(ASG,NF)= 1.+ASG/PI*(97./4.-NF*7./6.)
	XI(X,Y)= 2.*X/(1.-X-Y+LAMB(X,Y))
	BIJ(X,Y)= (1.-X-Y)/LAMB(X,Y)
     .	 * (4.*SP(XI(X,Y)*XI(Y,X))
     .	 - 2.*SP(-XI(X,Y))-2.*SP(-XI(Y,X))
     .	 + 2.*DLOG(XI(X,Y)*XI(Y,X))*DLOG(1.-XI(X,Y)*XI(Y,X))
     .	 - DLOG(XI(X,Y))*DLOG(1.+XI(X,Y))
     .	 - DLOG(XI(Y,X))*DLOG(1.+XI(Y,X)))
     .	 - 4.*(DLOG(1-XI(X,Y)*XI(Y,X))
     .	 + XI(X,Y)*XI(Y,X)/(1.-XI(X,Y)*XI(Y,X))*DLOG(XI(X,Y)*XI(Y,X)))
     .	 + (LAMB(X,Y)+X-Y)/LAMB(X,Y)*(DLOG(1.+XI(X,Y))
     .	 - XI(X,Y)/(1.+XI(X,Y))*DLOG(XI(X,Y)))
     .	 + (LAMB(X,Y)-X+Y)/LAMB(X,Y)*(DLOG(1.+XI(Y,X))
     .	 - XI(Y,X)/(1.+XI(Y,X))*DLOG(XI(Y,X)))
	QCDC(X,Y)= 1.+4./3.*ASH/PI*(9./4.+(3.-2.*X+2.*Y)/4.*DLOG(X/Y)
     .	 + ((1.5D0-X-Y)*LAMB(X,Y)**2+5.*X*Y)/2./LAMB(X,Y)
     .	 / (1.-X-Y)*DLOG(XI(X,Y)*XI(Y,X))+BIJ(X,Y))
     .	 + ASH/PI*(2.*(4./3.-DLOG(X))
     .	 - (X*2.*(4./3.-DLOG(X))+Y*2.*(4./3.-DLOG(Y)))/(1.-X-Y)
     .	 - (X*2.*(4./3.-DLOG(X))*(1.-X+Y)
     .	 + Y*2.*(4./3.-DLOG(Y))*(1.+X-Y))/LAMB(X,Y)**2)
	QCDCI(X,Y)=1.+4./3.*ASH/PI*(3.+(Y-X)/2.*DLOG(X/Y)
     .	 + (2.*(1.-X-Y)+LAMB(X,Y)**2)/2./LAMB(X,Y)
     .	 * DLOG(XI(X,Y)*XI(Y,X))+BIJ(X,Y))
     .	 + ASH/PI*(2.*(4./3.-DLOG(X))+2.*(4./3.-DLOG(Y))
     .	 - (X*2.*(4./3.-DLOG(X))*(1.-X+Y)
     .	 + Y*2.*(4./3.-DLOG(Y))*(1.+X-Y))/LAMB(X,Y)**2)
	QCDCM(X,Y)=1.+4./3.*ASH/PI*(9./4.+(3.-2.*X+2.*Y)/4.*DLOG(X/Y)
     .	 + ((1.5D0-X-Y)*LAMB(X,Y)**2+5.*X*Y)/2./LAMB(X,Y)
     .	 / (1.-X-Y)*DLOG(4.*X*Y/(1.-X-Y+LAMB(X,Y))**2)
     .	 + BIJ(X,Y))
	QCDCMI(X,Y)=1.+4./3.*ASH/PI*(3.+(Y-X)/2.*DLOG(X/Y)
     .	 + (2.*(1.-X-Y)*LAMB(X,Y)**2)/2/LAMB(X,Y)
     .	 * DLOG(4.*X*Y/(1.-X-Y+LAMB(X,Y))**2)
     .	 + BIJ(X,Y))
	CQCD(Z,T,X,Y)= GF/(4.*PI*SQR2)*Z**3*LAMB(X,Y)
     .	 * ((1.-X-Y)*(X*T**2*QCDC(Y,X)
     .	 + Y/T**2*QCDC(X,Y))
     .	 - 4.*X*Y*QCDCI(X,Y))
	CQCDM(Z,T,X,Y)= GF/(4.*PI*SQR2)*Z**3*LAMB(X,Y)
     .	 * ((1.-X-Y)*(X*T**2*QCDCM(Y,X)
     .	 + Y/T**2*QCDCM(X,Y))
     .	 - 4.*X*Y*QCDCMI(X,Y))
	PIHHH(A,B,D,I,J,K)= S(A,I)*S(B,J)*S(D,K)+S(A,I)*S(D,J)*S(B,K)
     .	 + S(B,I)*S(A,J)*S(D,K)+S(B,I)*S(D,J)*S(A,K)
     .	 + S(D,I)*S(A,J)*S(B,K)+S(D,I)*S(B,J)*S(A,K)
	PIHAA(A,B,D,I,J,K)= S(A,I)*(P(B,J)*P(D,K)+P(B,K)*P(D,J))
	PIHCC(A,I,J,K)= 2.*S(A,I)*C(J)*C(K)
	RADT(X)=3.*SQR2*LQ*X**4/(16.*PI**2*h1**3)
 	RADB(X)=3.*SQR2*LQ*X**4/(16.*PI**2*h2**3)
	RADTB(X,Y)=3.*SQR2*LQ*X**2*Y**2/(16.*PI**2*h1**2*h2**2)
	GHHH(A,B,D)= (g1+g2)/(4.*SQR2)
     .	 * (h1*(PIHHH(A,B,D,1,1,1)-PIHHH(A,B,D,1,2,2))
     .	 + h2*(PIHHH(A,B,D,2,2,2)-PIHHH(A,B,D,2,1,1)))
     .	 - LAMBDA*ALAMBDA/SQR2*PIHHH(A,B,D,1,2,3)
     .	 + KAPPA*AKAPPA/(3.*SQR2)*PIHHH(A,B,D,3,3,3)
     .	 + LAMBDA**2/SQR2*(h1*(PIHHH(A,B,D,1,2,2)
     .	 + PIHHH(A,B,D,1,3,3))+h2*(PIHHH(A,B,D,2,1,1)
     .	 + PIHHH(A,B,D,2,3,3))+ss*(PIHHH(A,B,D,3,1,1)
     .	 + PIHHH(A,B,D,3,2,2)))
     .	 + KAPPA**2*SQR2*ss*PIHHH(A,B,D,3,3,3)
     .	 - LAMBDA*KAPPA/SQR2*(h1*PIHHH(A,B,D,3,2,3)
     .	 + h2*PIHHH(A,B,D,3,1,3)+2.*ss*PIHHH(A,B,D,1,2,3))
     .   +RADT(RMT)*PIHHH(A,B,D,1,1,1)
     .   +RADB(RMB)*PIHHH(A,B,D,2,2,2)
	GHAA(A,B,D)= (g1+g2)/(4.*SQR2)
     .	 * (h1*(PIHAA(A,B,D,1,1,1)-PIHAA(A,B,D,1,2,2))
     .	 + h2*(PIHAA(A,B,D,2,2,2)-PIHAA(A,B,D,2,1,1)))
     .	 + LAMBDA*ALAMBDA/SQR2*(PIHAA(A,B,D,1,2,3)
     .	 + PIHAA(A,B,D,2,1,3)+PIHAA(A,B,D,3,1,2))
     .	 - KAPPA*AKAPPA/SQR2*PIHAA(A,B,D,3,3,3)
     .	 + LAMBDA**2/SQR2*(h1*(PIHAA(A,B,D,1,2,2)
     .	 + PIHAA(A,B,D,1,3,3))+h2*(PIHAA(A,B,D,2,1,1)
     .	 + PIHAA(A,B,D,2,3,3))+ss*(PIHAA(A,B,D,3,1,1)
     .	 + PIHAA(A,B,D,3,2,2)))
     .	 + KAPPA**2*SQR2*ss*PIHAA(A,B,D,3,3,3)
     .	 + LAMBDA*KAPPA/SQR2*(h1*(PIHAA(A,B,D,2,3,3)
     .	 - 2.*PIHAA(A,B,D,3,2,3))+h2*(PIHAA(A,B,D,1,3,3)
     .	 - 2.*PIHAA(A,B,D,3,1,3))+2.*ss*(PIHAA(A,B,D,3,1,2)
     .	 - PIHAA(A,B,D,1,2,3)-PIHAA(A,B,D,2,1,3)))
     .   +RADT(RMT)*PIHAA(A,B,D,1,1,1)
     .   +RADB(RMB)*PIHAA(A,B,D,2,2,2)
	GHCC(A)= LAMBDA**2/SQR2*(ss*(PIHCC(A,3,1,1)
     .	 + PIHCC(A,3,2,2))-h1*PIHCC(A,2,1,2)
     .	 - h2*PIHCC(A,1,1,2))
     .	 + LAMBDA*KAPPA*SQR2*ss*PIHCC(A,3,1,2)
     .	 + LAMBDA*ALAMBDA/SQR2*PIHCC(A,3,1,2)
     .	 + g1/(4.*SQR2)*(h1*(PIHCC(A,1,1,1)
     .	 - PIHCC(A,1,2,2))+h2*(PIHCC(A,2,2,2)
     .	 - PIHCC(A,2,1,1)))
     .	 + g2/(4.*SQR2)*(h1*(PIHCC(A,1,1,1)
     .	 + PIHCC(A,1,2,2)+2.*PIHCC(A,2,1,2))
     .	 + h2*(PIHCC(A,2,1,1)+PIHCC(A,2,2,2)
     .	 + 2.*PIHCC(A,1,1,2)))
     .   +RADT(RMT)*PIHCC(A,1,1,1)
     .   +RADB(RMB)*PIHCC(A,2,2,2)
     .   +RADTB(RMT,RMB)*(H1*PIHCC(A,1,2,2)+H2*PIHCC(A,1,1,2)
     .                    +H2*PIHCC(A,2,1,1)+H1*PIHCC(A,2,1,2))     
	GHNEUNEU(A,B,D)= LAMBDA/SQR2
     .	 * (S(A,1)*(N(B,4)*N(D,5)+N(B,5)*N(D,4))
     .	 + S(A,2)*(N(B,3)*N(D,5)+N(B,5)*N(D,3))
     .	 + S(A,3)*(N(B,3)*N(D,4)+N(B,4)*N(D,3)))
     .	 - SQR2*KAPPA*S(A,3)*N(B,5)*N(D,5)
     .	 + DSQRT(g1)/2.*(-S(A,1)*(N(B,1)*N(D,3)+N(B,3)*N(D,1))
     .	 + S(A,2)*(N(B,1)*N(D,4)+N(B,4)*N(D,1)))
     .	 + DSQRT(g2)/2.*(S(A,1)*(N(B,2)*N(D,3)+N(B,3)*N(D,2))
     .	 - S(A,2)*(N(B,2)*N(D,4)+N(B,4)*N(D,2)))
	GANEUNEU(A,B,D)= LAMBDA/SQR2
     .	 * (P(A,1)*(N(B,4)*N(D,5)+N(B,5)*N(D,4))
     .	 + P(A,2)*(N(B,3)*N(D,5)+N(B,5)*N(D,3))
     .	 + P(A,3)*(N(B,3)*N(D,4)+N(B,4)*N(D,3)))
     .	 - SQR2*KAPPA*P(A,3)*N(B,5)*N(D,5)
     .	 - DSQRT(g1)/2.*(-P(A,1)*(N(B,1)*N(D,3)+N(B,3)*N(D,1))
     .	 + P(A,2)*(N(B,1)*N(D,4)+N(B,4)*N(D,1)))
     .	 - DSQRT(g2)/2.*(P(A,1)*(N(B,2)*N(D,3)+N(B,3)*N(D,2))
     .	 - P(A,2)*(N(B,2)*N(D,4)+N(B,4)*N(D,2)))
	GHCHACHA(A,B,D)= LAMBDA/SQR2*S(A,3)*U(B,2)*V(D,2)
     .	 + DSQRT(g2)/SQR2*(S(A,1)*U(B,1)*V(D,2)+S(A,2)*U(B,2)*V(D,1))
	GACHACHA(A,B,D)= LAMBDA/SQR2*P(A,3)*U(B,2)*V(D,2)
     .	 - DSQRT(g2)/SQR2*(P(A,1)*U(B,1)*V(D,2)+P(A,2)*U(B,2)*V(D,1))
	GHCNEUCHAL(A,B)= LAMBDA*COSBETA*N(A,5)*U(B,2)
     .	 - SINBETA/SQR2*(DSQRT(g1)*N(A,1)+DSQRT(g2)*N(A,2))*U(B,2)
     .	 + SINBETA*DSQRT(g2)*N(A,4)*U(B,1)
	GHCNEUCHAR(A,B)= LAMBDA*SINBETA*N(A,5)*V(B,2)
     .	 + COSBETA/SQR2*(DSQRT(g1)*N(A,1)+DSQRT(g2)*N(A,2))*V(B,2)
     .	 + COSBETA*DSQRT(g2)*N(A,3)*V(B,1)



	NFGG= 3
	EPS= 1.D-8
	PI= 4.*DATAN(1.D0)
	SQR2= DSQRT(2.D0)
	C2TW= (MW/MZ)**2
	S2TW= 1.-C2TW
	T2TW= S2TW/(1.-S2TW)
	LAMBDA= PAR(1)
	KAPPA= PAR(2)
	TANBETA= PAR(3)
	COSBETA= 1./DSQRT(1.+TANBETA**2)
	SINBETA= TANBETA*COSBETA
	h1= SINBETA/DSQRT(2.*SQR2*GF)
	h2= h1/TANBETA
	ss= PAR(4)/LAMBDA
	ALAMBDA= PAR(5)
	AKAPPA= PAR(6)
	DO I=1,2
	 P(I,1)= P2(I,1)*COSBETA
	 P(I,2)= P2(I,1)*SINBETA
	 P(I,3)= P2(I,2)
	ENDDO
	C(1)= COSBETA
	C(2)= SINBETA
	ASMB=ALPHAS(MB,2)
	ASMT=ALPHAS(MT,2)
	
*   For rad. corrections:

	LQ=DLOG((PAR(7)**2/2.+(PAR(8)**2+PAR(9)**2)/4.+MT**2)/MT**2)

*   Assumption: MB = Running bottom mass at MB; MBP = Pole mass:

	MBP= MB*(1.+4.*ASMB/(3.*PI)+12.4*(ASMB/PI)**2)

* Loop over h1, h2, h3
 
	DO I=1,3

	 MH= SMASS(I)
	 HIGTOP= (MH/MT)**2
	 MT0= 3.D8
	 ASH= ALPHAS(MH,2)
	 MC0= 1.D8
	 MB0= 2.D8
	 AS3= ALPHAS(MH,2)
	 MC0= MC
	 AS4= ALPHAS(MH,2)
	 MB0= MB
	 MT0= MT

*  Running quark masses

	 RMS= RUNM(MH,3)
	 RMC= RUNM(MH,4)

*   Assumption: MT = Pole mass; RMT = running mass at Mtop (MS_bar):

	 RMT= MT/(1.+4.*ASMT/(3.*PI)+11.*(ASMT/PI)**2)	       

*   Running bottom/top masses at MH:

	 IF(MH.GE.RMT)THEN
	  RMB= MB*(ASMT/ASMB)**(12./23.)
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(MH**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(MB/H2)**2+(RMT/H1)**2)/(64.*PI**2)
     C     *DLOG(MH**2/MT**2))
	  RMT= RMT
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(MH**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(RMT/H1)**2+(MB/H2)**2)/(64.*PI**2)
     C     *DLOG(MH**2/MT**2))
           IF(CMASS.GT.MT) THEN
	     RMB=RMB*(1.+2.*(MT/H1)**2/(64.*PI**2)
     C           *DLOG(MH**2/MT**2))
	     RMT=RMT*(1.+2.*(MB/H2)**2/(64.*PI**2)
     C           *DLOG(MH**2/MT**2))
           END IF
	 ELSE
	  RMB= MB
     C	   *(1.+23./(12.*PI)*ASMB*DLOG(MH**2/MB**2))**(-12./23.)
 	  RMT= RMT
     C	   *(1.+23./(12.*PI)*ASMT*DLOG(MH**2/MT**2))**(-12./23.)
 	 ENDIF

*  Couplings relative to the standard model

	 CV(I)= S(I,1)*SINBETA+S(I,2)*COSBETA
	 CU(I)= S(I,1)/SINBETA
	 CD(I)= S(I,2)/COSBETA
	 IF(CD(I).EQ.0.) CD(I)= EPS

*  Effective coupling to 2 gluons/2 photons

	 CTT= 4.*(RMT/MH)**2*DCMPLX(1.D0,-EPS)
	 CTB= 4.*(RMB/MH)**2*DCMPLX(1.D0,-EPS)
	 CTC= 4.*(RMC/MH)**2*DCMPLX(1.D0,-EPS)
	 CTL= 4.*(MTAU/MH)**2*DCMPLX(1.D0,-EPS)
	 CTW= 4.*(MW/MH)**2*DCMPLX(1.D0,-EPS)
	 CTHC= 4.*(CMASS/MH)**2*DCMPLX(1.D0,-EPS)
	 CTCH1= 4.*(MCHA(1)/MH)**2*DCMPLX(1.D0,-EPS)
	 CTCH2= 4.*(MCHA(2)/MH)**2*DCMPLX(1.D0,-EPS)
	 CAT= 2.*CTT*(1.+(1.-CTT)*CF(CTT))
	 CAB= 2.*CTB*(1.+(1.-CTB)*CF(CTB))
	 CAC= 2.*CTC*(1.+(1.-CTC)*CF(CTC))
	 CAL= 2.*CTL*(1.+(1.-CTL)*CF(CTL))
	 CAW= -(2.+3.*CTW+3.*CTW*(2.-CTW)*CF(CTW))
	 CAHC= CTHC*(CTHC*CF(CTHC)-1.)
	 CACH1= 2.*CTCH1*(1.+(1.-CTCH1)*CF(CTCH1))
	 CACH2= 2.*CTCH2*(1.+(1.-CTCH2)*CF(CTCH2))

*   Here CG and CGA are the actual couplings. Later we divide
*   them by the SM couplings in order to obtain reduced couplings

	 CG(I)= CDABS(CU(I)*(CAT+CAC) + CD(I)*CAB)

	 CGA(I)= CDABS(CU(I)*4./3.*(CAT+CAC) + CD(I)*(CAB/3.+CAL)
     C	  + CV(I)*CAW + GHCC(I)/(2.*CMASS**2*DSQRT(2.*SQR2*GF))*CAHC
     C	  + GHCHACHA(I,1,1)/(DSQRT(SQR2*GF)*MCHA(1))*CACH1
     C	  + GHCHACHA(I,2,2)/(DSQRT(SQR2*GF)*MCHA(2))*CACH2)

*  Partial widths

*   h -> gg

	 NFEXT= 3
	 ASG= AS3
	 FQCD= HGGQCD(ASG,NFEXT)
	 XFAC= CG(I)**2*FQCD
	 HJJ= HVV(MH,0.D0)*(ASG/PI)**2*XFAC/8.

*   h -> gg* -> gcc to be added to h -> cc

	 NFEXT= 4
	 ASG= AS4
	 FQCD= HGGQCD(ASG,NFEXT)
	 XFAC= CG(I)**2*FQCD
	 DCC= HVV(MH,0.D0)*(ASG/PI)**2*XFAC/8.-HJJ

*   h -> gg* -> gbb to be added to h -> bb

	 NFEXT= 5
	 ASG= ASH
	 FQCD= HGGQCD(ASG,NFEXT)
	 XFAC= CG(I)**2*FQCD
	 DBB= HVV(MH,0.D0)*(ASG/PI)**2*XFAC/8.-HJJ-DCC

	 IF(NFGG.EQ.5)THEN
	  HJJ= HJJ+DBB+DCC
	  DBB= 0.
	  DCC= 0.
	 ELSEIF(NFGG.EQ.4)THEN
	  HJJ= HJJ+DCC
	  DCC= 0.
	 ENDIF

*    Below CG becomes the REDUCED coupling
*    to 2 gluons as compared to the SM coupling

	 CG(I)= CG(I)/CDABS(CAT+CAC+CAB)

*   h -> mumu

	 IF(MH.LE.2.*MMUON)THEN
	  HMM= 0.
	 ELSE
	  HMM= CD(I)**2*HFF(MH,(MMUON/MH)**2)
	 ENDIF

*   h -> tautau

	 IF(MH.LE.2.*MTAU)THEN
	  HLL= 0.
	 ELSE
	  HLL= CD(I)**2*HFF(MH,(MTAU/MH)**2)
	 ENDIF

*   h -> ss

	 RATCOUP= CU(I)/CD(I)
	 IF(MH.LE.2.*MS)THEN
	  HSS= 0.
	 ELSE
	  HS1= 3.*HFF(MH,(MS/MH)**2)*CD(I)**2
     .       * TQCDH((MS/MH)**2)
	  HS2= 3.*HFF(MH,(RMS/MH)**2)*CD(I)**2
     .       * QCDH((RMS/MH)**2)
	  IF(HS2.LT.0.) HS2=0.
	  RAT= 2.*MS/MH
	  HSS= QQINT(RAT,HS1,HS2)
	 ENDIF

*   h -> cc

	 RATCOUP= 1.
	 IF(MH.LE.2.*MC)THEN
	  HCC= 0.
	 ELSE
	  HC1= 3.*HFF(MH,(MC/MH)**2)*CU(I)**2
     .       * TQCDH((MC/MH)**2)
	  HC2= 3.*HFF(MH,(RMC/MH)**2)*CU(I)**2
     .       * QCDH((RMC/MH)**2)
     .       + DCC
	  IF(HC2.LT.0.) HC2=0.
	  RAT= 2.*MC/MH
	  HCC= QQINT(RAT,HC1,HC2)
	 ENDIF

*   h -> bb

	 RATCOUP= CU(I)/CD(I)
	 IF(MH.LE.2.*MBP)THEN
	  HBB= 0.
	 ELSE
	  HB1= 3.*HFF(MH,(MBP/MH)**2)*CD(I)**2
     .       * TQCDH((MBP/MH)**2)
	  HB2= 3.*HFF(MH,(RMB/MH)**2)*CD(I)**2
     .       * QCDH((RMB/MH)**2)
     .       + DBB
	  IF(HB2.LT.0.) HB2=0.
	  RAT= 2.*MBP/MH
	  HBB= QQINT(RAT,HB1,HB2)
	 ENDIF

*   h -> tt

	 RATCOUP= 0.
	 IF (MH.LE.2.*MT)THEN
	  HTT= 0.
	 ELSE
	  HT1= 3.*HFF(MH,(MT/MH)**2)*CU(I)**2
     .	     * TQCDH((MT/MH)**2)
	  IF (MH.LE.2.*RMT)THEN
	   HT2= 0.
	  ELSE
	   HT2= 3.*HFF(MH,(RMT/MH)**2)*CU(I)**2
     .	      * QCDH((RMT/MH)**2)
	  ENDIF
	  IF(HT2.LT.0.) HT2=0.
	  RAT= 2.*MT/MH
	  HTT= QQINT(RAT,HT1,HT2)
	 ENDIF

*   h -> WW

	 DLD= 2.
	 DLU= 2.
	 XM1= 2.*MW-DLD
	 XM2= 2.*MW+DLU
	 IF(MH.LE.MW)THEN
	  HWW= 0.
	 ELSEIF(MH.LE.XM1)THEN
	  CWW= 3.*GF**2*MW**4/16./PI**3
	  HWW= CV(I)**2*HV((MW/MH)**2)*CWW*MH
	 ELSEIF(MH.LT.XM2)THEN
	  CWW= 3.*GF**2*MW**4/16./PI**3
	  XX(1)= XM1-1.
	  XX(2)= XM1
	  XX(3)= XM2
	  XX(4)= XM2+1.
	  YY(1)= HV((MW/XX(1))**2)*CWW*XX(1)
	  YY(2)= HV((MW/XX(2))**2)*CWW*XX(2)
	  YY(3)= HVV(XX(3),(MW/XX(3))**2)
	  YY(4)= HVV(XX(4),(MW/XX(4))**2)
	  HWW= CV(I)**2*FINT(MH,XX,YY)
	 ELSE
	  HWW= CV(I)**2*HVV(MH,(MW/MH)**2)
	 ENDIF

*   h -> ZZ

	 DLD= 2.
	 DLU= 2.
	 XM1= 2.*MZ-DLD
	 XM2= 2.*MZ+DLU
	 IF(MH.LE.MZ)THEN
	  HZZ= 0.
	 ELSEIF(MH.LE.XM1)THEN
	  CZZ= 3.*GF**2*MZ**4/192./PI**3
     .	     * (7.-40./3.*S2TW+160./9.*S2TW**2)
	  HZZ= CV(I)**2*HV((MZ/MH)**2)*CZZ*MH
	 ELSEIF(MH.LT.XM2)THEN
	  CZZ= 3.*GF**2*MZ**4/192./PI**3
     .	     * (7.-40./3.*S2TW+160./9.*S2TW**2)
	  XX(1)= XM1-1.
	  XX(2)= XM1
	  XX(3)= XM2
	  XX(4)= XM2+1.
	  YY(1)= HV((MZ/XX(1))**2)*CZZ*XX(1)
	  YY(2)= HV((MZ/XX(2))**2)*CZZ*XX(2)
	  YY(3)= HVV(XX(3),(MZ/XX(3))**2)/2.
	  YY(4)= HVV(XX(4),(MZ/XX(4))**2)/2.
	  HZZ= CV(I)**2*FINT(MH,XX,YY)
	 ELSE
	  HZZ= CV(I)**2*HVV(MH,(MZ/MH)**2)/2.
	 ENDIF
 
*   h -> gamma gamma

	 XFAC= CGA(I)**2
	 HGG= HVV(MH,0.D0)*(ALP0/PI)**2/16.*XFAC

*    Below CGA becomes the REDUCDED coupling
*    to 2 photons as compared to the SM coupling

	 CGA(I)= CGA(I)/CDABS(4./3.*(CAT+CAC)+(CAB/3.+CAL)+CAW)
	 
*  h -> Z gamma

	IF(MH.LE.MZ)THEN
	 HZG= 0.
	ELSE
	 FT= -2.*(1.-8./3.*S2TW)/DSQRT(S2TW*C2TW)*CU(I)
	 FB= (-1.+4./3.*S2TW)/DSQRT(S2TW*C2TW)*CD(I)
	 CLT= 4.*(RMT/MZ)**2*DCMPLX(1.D0,-EPS)
	 CLB= 4.*(RMB/MZ)**2*DCMPLX(1.D0,-EPS)
	 CLW= 4.*(MW/MZ)**2*DCMPLX(1.D0,-EPS)
	 CLH= 4.*(CMASS/MZ)**2*DCMPLX(1.D0,-EPS)
	 CATZ= FT*(CI1(CTT,CLT) - CI2(CTT,CLT))
	 CABZ= FB*(CI1(CTB,CLB) - CI2(CTB,CLB))
	 CAWZ= -1./DSQRT(T2TW)*(4.*(3.-T2TW)*CI2(CTW,CLW)
     .	     + ((1.+2./CTW)*T2TW - (5.+2./CTW))*CI1(CTW,CLW))*CV(I)
	 CAHZ= (1.-2.*S2TW)/(DSQRT(S2TW*C2TW)*2.*CMASS**2)
     .	     * CI1(CTHC,CLH)*GHCC(I)/(DSQRT(2.*SQR2*GF))
	 XFAC= CDABS(CATZ+CABZ+CAWZ+CAHZ)**2
	 ACOUP= SQR2*GF*MZ**2*S2TW*C2TW/PI**2
	 HZG= GF/(4.*PI*SQR2)*MH**3*(ALP0/PI)*ACOUP/16.
     .	    * XFAC*(1.-(MZ/MH)**2)**3
	ENDIF

*   h -> hh
	write(*,*) 'couplings'
      write(*,*) GHHH(1,1,1)
      write(*,*) GHHH(1,1,2)
      write(*,*) GHHH(1,2,2)
      write(*,*) GHHH(2,2,3)
      write(*,*) GHAA(1,1,1)
      write(*,*) GHAA(2,1,1)

	 HTOT= 0.

	 IF(I.EQ.2)THEN
	  IF(MH.LE.2.*SMASS(1))THEN
	   HHH(1)= 0.
	  ELSE
	   RH= GHHH(2,1,1)
	   CH= BETA((SMASS(1)/MH)**2)
	   HHH(1)= CH*RH**2/(32.*PI*MH)
	   HTOT= HTOT+HHH(1)
	  ENDIF
	 ENDIF

	 IF(I.EQ.3)THEN
	  IF(MH.LE.2.*SMASS(1))THEN
	   HHH(2)= 0.
	  ELSE
	   RH= GHHH(3,1,1)
	   CH= BETA((SMASS(1)/MH)**2)
	   HHH(2)= CH*RH**2/(32.*PI*MH)
	   HTOT= HTOT+HHH(2)
	  ENDIF

	  IF(MH.LE.SMASS(1)+SMASS(2))THEN
	   HHH(3)= 0.
	  ELSE
	   RH= GHHH(3,1,2)
	   CH= LAMB((SMASS(1)/MH)**2,(SMASS(2)/MH)**2)
	   HHH(3)= CH*RH**2/(16.*PI*MH)
	   HTOT= HTOT+HHH(3)
	  ENDIF

	  IF(MH.LE.2.*SMASS(2))THEN
	   HHH(4)= 0.
	  ELSE
	   RH= GHHH(3,2,2)
	   CH= BETA((SMASS(2)/MH)**2)
	   HHH(4)= CH*RH**2/(32.*PI*MH)
	   HTOT= HTOT+HHH(4)
	  ENDIF
	 ENDIF

*   h -> aa

	 IF(MH.LE.2.*PMASS(1))THEN
	  HAA(1)= 0.
	 ELSE
	  RH= GHAA(I,1,1)
	  CH= BETA((PMASS(1)/MH)**2)
	  HAA(1)= CH*RH**2/(32.*PI*MH)
	  HTOT= HTOT+HAA(1)
	 ENDIF

	 IF(MH.LE.PMASS(1)+PMASS(2))THEN
	  HAA(2)= 0.
	 ELSE
	  RH= GHAA(I,1,2)
	  CH= LAMB((PMASS(1)/MH)**2,(PMASS(2)/MH)**2)
	  HAA(2)= CH*RH**2/(16.*PI*MH)
	  HTOT= HTOT+HAA(2)
	 ENDIF

	 IF(MH.LE.2.*PMASS(2))THEN
	  HAA(3)= 0.
	 ELSE
	  RH= GHAA(I,2,2)
	  CH= BETA((PMASS(2)/MH)**2)
	  HAA(3)= CH*RH**2/(32.*PI*MH)
	  HTOT= HTOT+HAA(3)
	 ENDIF

*   h -> h+h-

	 IF(MH.LE.2.*CMASS)THEN
	  HHCHC= 0.
	 ELSE
	  RH= GHCC(I)
	  CH= BETA((CMASS/MH)**2)
	  HHCHC= CH*RH**2/(16.*PI*MH)
	  HTOT= HTOT+HHCHC
	 ENDIF

*   h -> aZ

	 DO J=1,2
	  IF(MH.LE.PMASS(J)+MZ)THEN
	   HAZ(J)= 0.
	  ELSE
	   RH= (S(I,1)*COSBETA-S(I,2)*SINBETA)
     C	     * (P(J,1)*COSBETA+P(J,2)*SINBETA)
	   CH= LAMB((PMASS(J)/MH)**2,(MZ/MH)**2)
     C	     * LAMB((MH/MZ)**2,(PMASS(J)/MZ)**2)**2
	   HAZ(J)= GF/8./SQR2/PI*MZ**4/MH*CH*RH**2
	   HTOT= HTOT+HAZ(J)
	  ENDIF
	 ENDDO

*   h -> h+W-

	 IF(MH.LE.CMASS+MW)THEN
	  HHCW= 0.
	 ELSE
	  RH= S(I,1)*COSBETA-S(I,2)*SINBETA
	  CH= LAMB((CMASS/MH)**2,(MW/MH)**2)
     C	    * LAMB((MH/MW)**2,(CMASS/MW)**2)**2
	  HHCW= GF/4./SQR2/PI*MW**4/MH*CH*RH**2
	  HTOT= HTOT+HHCW
	 ENDIF

*   h -> neutralinos/charginos

	 STOT= 0.

	 DO J=1,5
	 DO K=J,5
	  IF(MH.LE.DABS(MNEU(J))+DABS(MNEU(K)))THEN
	   HNEU(J,K)= 0.
	  ELSE
	   HNEU(J,K)= 1./(16.*PI)*(MH**2-(MNEU(J)+MNEU(K))**2)/MH
     C	        * LAMB((MNEU(J)/MH)**2,(MNEU(K)/MH)**2)
     C	        * GHNEUNEU(I,J,K)**2
	   IF(J.NE.K)HNEU(J,K)= 2.*HNEU(J,K)
	   STOT= STOT+HNEU(J,K)
	  ENDIF
	  HNEU(K,J)= HNEU(J,K)
	 ENDDO
	 ENDDO

	 IF(MH.LE.2.*DABS(MCHA(1)))THEN
	  HCHA(1)= 0.
	 ELSE
	  HCHA(1)= 1./(8.*PI)*MH
     C	        * LAMB((MCHA(1)/MH)**2,(MCHA(1)/MH)**2)**3
     C	        * GHCHACHA(I,1,1)**2
	  STOT= STOT+HCHA(1)
	 ENDIF

	 IF(MH.LE.DABS(MCHA(1))+DABS(MCHA(2)))THEN
	  HCHA(2)= 0.
	 ELSE
	  HCHA(2)= 1./(16.*PI)*MH
     C	        * LAMB((MCHA(1)/MH)**2,(MCHA(2)/MH)**2)
     C	        * ((GHCHACHA(I,1,2)**2+GHCHACHA(I,2,1)**2)
     C	        * (1.-(MCHA(1)/MH)**2-(MCHA(2)/MH)**2)
     C	        - 4.*GHCHACHA(I,1,2)*GHCHACHA(I,2,1)
     C	        * MCHA(1)*MCHA(2)/MH**2)
	  STOT= STOT+HCHA(2)
	 ENDIF

	 IF(MH.LE.2.*DABS(MCHA(2)))THEN
	  HCHA(3)= 0.
	 ELSE
	  HCHA(3)= 1./(8.*PI)*MH
     C	        * LAMB((MCHA(2)/MH)**2,(MCHA(2)/MH)**2)**3
     C	        * GHCHACHA(I,2,2)**2
	  STOT= STOT+HCHA(3)
	 ENDIF

*  Branching ratios

	 WIDTH(I)= HJJ+HMM+HLL+HSS+HCC+HBB+HTT+HWW+HZZ+HGG+HZG+HTOT+STOT
	 BRJJ(I)= HJJ/WIDTH(I)
	 BRMM(I)= HMM/WIDTH(I)
	 BRLL(I)= HLL/WIDTH(I)
	 BRSS(I)= HSS/WIDTH(I)
	 BRCC(I)= HCC/WIDTH(I)
	 BRBB(I)= HBB/WIDTH(I)
	 BRTT(I)= HTT/WIDTH(I)
	 BRWW(I)= HWW/WIDTH(I)
	 BRZZ(I)= HZZ/WIDTH(I)
	 BRGG(I)= HGG/WIDTH(I)
	 BRZG(I)= HZG/WIDTH(I)
	 BRHIGGS(I)= HTOT/WIDTH(I)
	 BRSUSY(I)= STOT/WIDTH(I)
	 DO J=1,3
	  BRHAA(I,J)=HAA(J)/WIDTH(I)
	 ENDDO
	 BRHCHC(I)= HHCHC/WIDTH(I)
	 DO J=1,2
	  BRHAZ(I,J)=HAZ(J)/WIDTH(I)
	 ENDDO
	 BRHCW(I)= HHCW/WIDTH(I)
	 DO J=1,5
	 DO K=1,5
	  BRNEU(I,J,K)=HNEU(J,K)/WIDTH(I)
	 ENDDO
	 ENDDO
	 DO J=1,3
	  BRCHA(I,J)=HCHA(J)/WIDTH(I)
	 ENDDO

	ENDDO

	BRHHH(1)=HHH(1)/WIDTH(2)
	BRHHH(2)=HHH(2)/WIDTH(3)
	BRHHH(3)=HHH(3)/WIDTH(3)
	BRHHH(4)=HHH(4)/WIDTH(3)

* Loop over a1, a2

	DO I=1,2

	 MA= PMASS(I)
	 HIGTOP= (MA/MT)**2
	 MT0= 3.D8
	 ASH= ALPHAS(MA,2)
	 MC0= 1.D8
	 MB0= 2.D8
	 AS3= ALPHAS(MA,2)
	 MC0= MC
	 AS4= ALPHAS(MA,2)
	 MB0= MB
	 MT0= MT

*  Running quark masses

	 RMS= RUNM(MA,3)
	 RMC= RUNM(MA,4)

*   Assumption: MT = Pole mass; RMT = running mass at Mtop (MS_bar):

	 RMT= MT/(1.+4.*ASMT/(3.*PI)+11.*(ASMT/PI)**2)	       

*   Running bottom/top masses at MA:

	 IF(MA.GE.RMT)THEN
	  RMB= MB*(ASMT/ASMB)**(12./23.)
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(MA**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(MB/H2)**2+(RMT/H1)**2)/(64.*PI**2)
     C     *DLOG(MA**2/MT**2))
	  RMT= RMT
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(MA**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(RMT/H1)**2+(MB/H2)**2)/(64.*PI**2)
     C     *DLOG(MA**2/MT**2))
           IF(CMASS.GT.MT) THEN
	     RMB=RMB*(1.+2.*(MT/H1)**2/(64.*PI**2)
     C           *DLOG(MA**2/MT**2))
	     RMT=RMT*(1.+2.*(MB/H2)**2/(64.*PI**2)
     C           *DLOG(MA**2/MT**2))
           END IF
	 ELSE
	  RMB= MB
     C	   *(1.+23./(12.*PI)*ASMB*DLOG(MA**2/MB**2))**(-12./23.)
 	  RMT= RMT
     C	   *(1.+23./(12.*PI)*ASMT*DLOG(MA**2/MT**2))**(-12./23.)
 	 ENDIF

*  Relative couplings compared with equivalent SM

	 CU(I+3)= P(I,1)/SINBETA
	 CD(I+3)= P(I,2)/COSBETA
	 IF(CD(I+3).EQ.0.) CD(I+3)= EPS

*  Effective coupling to 2 gluons/2 photons

	 CTT= 4.*(RMT/MA)**2*DCMPLX(1.D0,-EPS)
	 CTB= 4.*(RMB/MA)**2*DCMPLX(1.D0,-EPS)
	 CTC= 4.*(RMC/MA)**2*DCMPLX(1.D0,-EPS)
	 CTL= 4.*(MTAU/MA)**2*DCMPLX(1.D0,-EPS)
	 CTCH1= 4.*(MCHA(1)/MH)**2*DCMPLX(1.D0,-EPS)
	 CTCH2= 4.*(MCHA(2)/MH)**2*DCMPLX(1.D0,-EPS)
	 CAT= CTT*CF(CTT)
	 CAB= CTB*CF(CTB)
	 CAC= CTC*CF(CTC)
	 CAL= CTL*CF(CTL)
	 CACH1= CTCH1*CF(CTCH1)
	 CACH2= CTCH2*CF(CTCH2)

*   Here CG and CGA are the actual couplings. Later we divide
*   them by the SM couplings in order to obtain the reduced couplings

	 CG(I+3)= CDABS(CU(I+3)*(CAT+CAC) + CD(I+3)*CAB)

	 CGA(I+3)= CDABS(CU(I+3)*4./3.*(CAT+CAC) + CD(I+3)*(CAB/3.+CAL)
     C	  + GACHACHA(I,1,1)/(DSQRT(SQR2*GF)*MCHA(1))*CACH1
     C	  + GACHACHA(I,2,2)/(DSQRT(SQR2*GF)*MCHA(2))*CACH2)

*  Partial widths

*   a -> gg

	 NFEXT= 3
	 ASG= AS3
	 FQCD= AGGQCD(ASG,NFEXT)
	 XFAC= CG(I+3)**2*FQCD
	 HJJ= GF/(16.*PI*SQR2)*MA**3*(ASG/PI)**2*XFAC

*   a -> gg* -> gcc to be added to a -> cc

	 NFEXT= 4
	 ASG= AS4
	 FQCD= AGGQCD(ASG,NFEXT)
	 XFAC= CG(I+3)**2*FQCD
	 DCC= GF/(16.*PI*SQR2)*MA**3*(ASG/PI)**2*XFAC-HJJ

*   a -> gg* -> gbb to be added to a -> bb

	 NFEXT= 5
	 ASG= ASH
	 FQCD= AGGQCD(ASG,NFEXT)
	 XFAC= CG(I+3)**2*FQCD
	 DBB= GF/(16.*PI*SQR2)*MA**3*(ASG/PI)**2*XFAC-HJJ-DCC

	 IF(NFGG.EQ.5)THEN
	  HJJ= HJJ+DBB+DCC
	  DBB= 0.
	  DCC= 0.
	 ELSEIF(NFGG.EQ.4)THEN
	  HJJ= HJJ+DCC
	  DCC= 0.
	 ENDIF

*    Below CG becomes the REDUCED coupling
*    to 2 gluons as compared to the SM coupling

	 CG(I+3)= CG(I+3)/CDABS(CAT+CAC+CAB)

*   a -> mumu

	 IF(MA.LE.2*MMUON)THEN
	 HMM= 0.
	 ELSE
	  HMM= CD(I+3)**2*AFF(MA,(MMUON/MA)**2)
	 ENDIF

*   a -> tautau

	 IF(MA.LE.2*MTAU)THEN
	  HLL= 0.
	 ELSE
	  HLL= CD(I+3)**2*AFF(MA,(MTAU/MA)**2)
	 ENDIF

*   a -> ss

	 RATCOUP= CU(I+3)/CD(I+3)
	 IF(MA.LE.2.*MS)THEN
	  HSS= 0.
	 ELSE
	  HS1= 3.*AFF(MA,(MS/MA)**2)*CD(I+3)**2
     .       * TQCDA((MS/MA)**2)
	  HS2= 3.*AFF(MA,(RMS/MA)**2)*CD(I+3)**2
     .       * QCDA((RMS/MA)**2)
	  IF(HS2.LT.0.) HS2=0.
	  RAT= 2.*MS/MA
	  HSS= QQINT(RAT,HS1,HS2)
	 ENDIF

*   a -> cc

	 RATCOUP= 1.
	 IF(MA.LE.2*MC)THEN
	  HCC= 0.
	 ELSE
	  HC1= 3.*AFF(MA,(MC/MA)**2)*CU(I+3)**2
     .	     * TQCDA((MC/MA)**2)
	  HC2= 3.*AFF(MA,(RMC/MA)**2)*CU(I+3)**2
     .	     * QCDA((RMC/MA)**2)
     .	     + DCC
	  IF(HC2.LT.0.) HC2=0.
	  RAT= 2.*MC/MA
	  HCC= QQINT(RAT,HC1,HC2)
	 ENDIF

*   a -> bb

	 RATCOUP= CU(I+3)/CD(I+3)
	 IF(MA.LE.2*MBP)THEN
	  HBB= 0.
	 ELSE
	  HB1= 3.*AFF(MA,(MBP/MA)**2)*CD(I+3)**2
     .	     * TQCDA((MBP/MA)**2)
	  HB2= 3.*AFF(MA,(RMB/MA)**2)*CD(I+3)**2
     .	     * QCDA((RMB/MA)**2)
     .	     + DBB
	  IF(HB2.LT.0.) HB2=0.
	  RAT= 2.*MBP/MA
	  HBB= QQINT(RAT,HB1,HB2)
	 ENDIF

*   a -> tt
 
	 RATCOUP= 0.
	 IF (MA.LE.2.*MT)THEN
	  HTT= 0.
	 ELSE
	  HT1= 3.*AFF(MA,(MT/MA)**2)*CU(I+3)**2
     .	     * TQCDA((MT/MA)**2)
	  IF(MA.LE.2.*RMT)THEN
	   HT2= 0.
	  ELSE
	   HT2= 3.*AFF(MA,(RMT/MA)**2)*CU(I+3)**2
     .	      * QCDA((RMT/MA)**2)
	  ENDIF
	  IF(HT2.LT.0.) HT2=0.
	  RAT= 2.*MT/MA
	  HTT= QQINT(RAT,HT1,HT2)
	 ENDIF
 
*   a -> gamma gamma

	 XFAC= CGA(I+3)**2
	 HGG= GF/(32.*PI*SQR2)*MA**3*(ALP0/PI)**2*XFAC

*    Below CGA becomes the REDUCDED coupling
*    to 2 photons as compared to the SM coupling

	 CGA(I+3)= CGA(I+3)/CDABS(4./3.*(CAT+CAC)+(CAB/3.+CAL))
	 
*   a -> Z gamma

	IF(MA.LE.MZ)THEN
	 HZG= 0.
	ELSE
	 FT= -2.*(1.-8./3.*S2TW)/DSQRT(S2TW*C2TW)*CU(I+3)
	 FB= (-1.+4./3.*S2TW)/DSQRT(S2TW*C2TW)*CD(I+3)
	 CLT= 4.*(RMT/MZ)**2*DCMPLX(1.D0,-EPS)
	 CLB= 4.*(RMB/MZ)**2*DCMPLX(1.D0,-EPS)
	 CAT= FT*(-CI2(CTT,CLT))
	 CAB= FB*(-CI2(CTB,CLB))
	 XFAC= CDABS(CAT+CAB)**2
	 ACOUP= SQR2*GF*MZ**2*S2TW*C2TW/PI**2
	 HZG= GF/(4.*PI*SQR2)*MA**3*(ALP0/PI)*ACOUP/16.
     .	    * XFAC*(1.-(MZ/MA)**2)**3
	ENDIF

*   a -> ha

	 HTOT= 0.

	 IF(I.EQ.2)THEN
	  DO J=1,3
	   IF(MA.LE.SMASS(J)+PMASS(1))THEN
	    HHA(J)= 0.
	   ELSE
	    RH= GHAA(J,1,I)
	    CH= LAMB((SMASS(J)/MA)**2,(PMASS(1)/MA)**2)
	    HHA(J)= CH*RH**2/(16.*PI*MA)
	    HTOT= HTOT+HHA(J)
	   ENDIF
	  ENDDO
	 ENDIF

*   a -> hZ

	 DO J=1,3
	  IF(MA.LE.SMASS(J)+MZ)THEN
	   HHZ(J)= 0.
	  ELSE
	   RH= (S(J,1)*COSBETA-S(J,2)*SINBETA)*P2(I,1)
	   CH= LAMB((SMASS(J)/MA)**2,(MZ/MA)**2)
     C	     * LAMB((MA/MZ)**2,(SMASS(J)/MZ)**2)**2
	   HHZ(J)= GF/8./SQR2/PI*MZ**4/MA*CH*RH**2
	   HTOT= HTOT+HHZ(J)
	  ENDIF
	 ENDDO

*   a -> h+W-

	 IF(MA.LE.CMASS+MW)THEN
	  HHCW= 0.
	 ELSE
	  RH= P2(I,1)
	  CH= LAMB((CMASS/MA)**2,(MW/MA)**2)
     C	    * LAMB((MA/MW)**2,(CMASS/MW)**2)**2
	  HHCW= GF/4./SQR2/PI*MW**4/MA*CH*RH**2
	  HTOT= HTOT+HHCW
	 ENDIF

*   a -> neutralinos/charginos

	 STOT= 0.

	 DO J=1,5
	 DO K=J,5
	  IF(MA.LE.DABS(MNEU(J))+DABS(MNEU(K)))THEN
	   HNEU(J,K)= 0.
	  ELSE
	   HNEU(J,K)= 1./(16.*PI)*(MA**2-(MNEU(J)-MNEU(K))**2)/MA
     C	        * LAMB((MNEU(J)/MA)**2,(MNEU(K)/MA)**2)
     C	        * GANEUNEU(I,J,K)**2
	   IF(J.NE.K)HNEU(J,K)= 2.*HNEU(J,K)
	   STOT= STOT+HNEU(J,K)
	  ENDIF
	  HNEU(K,J)= HNEU(J,K)
	 ENDDO
	 ENDDO

	 IF(MA.LE.2.*DABS(MCHA(1)))THEN
	  HCHA(1)= 0.
	 ELSE
	  HCHA(1)= 1./(8.*PI)*MA
     C	       * LAMB((MCHA(1)/MA)**2,(MCHA(1)/MA)**2)
     C	       * GACHACHA(I,1,1)**2
	  STOT= STOT+HCHA(1)
	 ENDIF

	 IF(MA.LE.DABS(MCHA(1))+DABS(MCHA(2)))THEN
	  HCHA(2)= 0.
	 ELSE
	  HCHA(2)= 1./(16.*PI)*MA
     C	       * LAMB((MCHA(1)/MA)**2,(MCHA(2)/MA)**2)
     C	       * ((GACHACHA(I,1,2)**2+GACHACHA(I,2,1)**2)
     C	       * (1.-(MCHA(1)/MA)**2-(MCHA(2)/MA)**2)
     C	       + 4.*GACHACHA(I,1,2)*GACHACHA(I,2,1)
     C	       * MCHA(1)*MCHA(2)/MA**2)
	  STOT= STOT+HCHA(2)
	 ENDIF

	 IF(MA.LE.2.*DABS(MCHA(2)))THEN
	  HCHA(3)= 0.
	 ELSE
	  HCHA(3)= 1./(8.*PI)*MA
     C	       * LAMB((MCHA(2)/MA)**2,(MCHA(2)/MA)**2)
     C	       * GACHACHA(I,2,2)**2
	  STOT= STOT+HCHA(3)
	 ENDIF

*  Branching ratios

	 WIDTH(I+3)= HJJ+HMM+HLL+HSS+HCC+HBB+HTT+HGG+HZG+HTOT+STOT
	 BRJJ(I+3)= HJJ/WIDTH(I+3)
	 BRMM(I+3)= HMM/WIDTH(I+3)
	 BRLL(I+3)= HLL/WIDTH(I+3)
	 BRSS(I+3)= HSS/WIDTH(I+3)
	 BRCC(I+3)= HCC/WIDTH(I+3)
	 BRBB(I+3)= HBB/WIDTH(I+3)
	 BRTT(I+3)= HTT/WIDTH(I+3)
	 BRGG(I+3)= HGG/WIDTH(I+3)
	 BRZG(I+3)= HZG/WIDTH(I+3)
	 BRHIGGS(I+3)= HTOT/WIDTH(I+3)
	 BRSUSY(I+3)= STOT/WIDTH(I+3)
	 DO J=1,3
	  BRAHZ(I,J)= HHZ(J)/WIDTH(I+3)
	 ENDDO
	 BRHCW(I+3)= HHCW/WIDTH(I+3)
	 DO J=1,5
	 DO K=1,5
	  BRNEU(I+3,J,K)=HNEU(J,K)/WIDTH(I+3)
	 ENDDO
	 ENDDO
	 DO J=1,3
	  BRCHA(I+3,J)=HCHA(J)/WIDTH(I+3)
	 ENDDO

	ENDDO

	DO J=1,3
	 BRAHA(J)= HHA(J)/WIDTH(5)
	ENDDO

* Charged Higgs decays

	ASH= ALPHAS(CMASS,2)

*  Running quark masses

	RMS= RUNM(CMASS,3)
	RMC= RUNM(CMASS,4)

*   Assumption: MT = Pole mass; RMT = running mass at Mtop (MS_bar):

	RMT= MT/(1.+4.*ASMT/(3.*PI)+11.*(ASMT/PI)**2)	       

*   Running bottom/top masses at CMASS:

	 IF(CMASS.GE.MT)THEN
	  RMB= MB*(ASMT/ASMB)**(12./23.)
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(CMASS**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(MB/H2)**2+3.*(RMT/H1)**2)/(64.*PI**2)
     C     *DLOG(CMASS**2/MT**2))
	  RMT= RMT
     C	   *(1.+7./(4.*PI)*ASMT*DLOG(CMASS**2/MT**2))**(-4./7.)
     C	   *(1.+(9.*(RMT/H1)**2+3.*(MB/H2)**2)/(64.*PI**2)
     C     *DLOG(CMASS**2/MT**2))
	 ELSE
	  RMB= MB
     C	   *(1.+23./(12.*PI)*ASMB*DLOG(CMASS**2/MB**2))**(-12./23.)
 	  RMT= RMT
     C	   *(1.+23./(12.*PI)*ASMT*DLOG(CMASS**2/MT**2))**(-12./23.)
 	 ENDIF

*  Partial widths

*   h+ -> munu

	IF(CMASS.LE.MMUON)THEN
	 HMN= 0.
	ELSE
	 HMN= CFF(CMASS,TANBETA,(MMUON/CMASS)**2,0.D0)
	ENDIF

*   h+ -> taunu

	IF(CMASS.LE.MTAU)THEN
	 HLN= 0.
	ELSE
	 HLN= CFF(CMASS,TANBETA,(MTAU/CMASS)**2,0.D0)
	ENDIF

*   h+ -> su

	IF(CMASS.LE.MS+EPS)THEN
	 HSU= 0.
	ELSE
	 HSU1= 3.*VUS**2*CQCDM(CMASS,TANBETA,(MS/CMASS)**2,EPS)
	 HSU2= 3.*VUS**2*CQCD(CMASS,TANBETA,(RMS/CMASS)**2,EPS)
	 IF(HSU2.LT.0.) HSU2= 0.
	 RAT= MS/CMASS
	 HSU= QQINT(RAT,HSU1,HSU2)
	ENDIF

*   h+ -> cs

	IF(CMASS.LE.MS+MC)THEN
	 HSC= 0.
	ELSE
	 HSC1= 3.*CQCDM(CMASS,TANBETA,(MS/CMASS)**2,(MC/CMASS)**2)
	 HSC2= 3.*CQCD(CMASS,TANBETA,(RMS/CMASS)**2,(RMC/CMASS)**2)
	 IF(HSC2.LT.0.) HSC2= 0.
	 RAT= (MS+MC)/CMASS
	 HSC= QQINT(RAT,HSC1,HSC2)
	ENDIF

*   h+ -> cb

	IF(CMASS.LE.MBP+MC)THEN
	 HBC= 0.
	ELSE
	 HBC1= 3.*VCB**2
     C	  *CQCDM(CMASS,TANBETA,(MBP/CMASS)**2,(MC/CMASS)**2)
	 HBC2= 3.*VCB**2
     C	  *CQCD(CMASS,TANBETA,(RMB/CMASS)**2,(RMC/CMASS)**2)
	 IF(HBC2.LT.0.) HBC2= 0.
	 RAT= (MBP+MC)/CMASS
	 HBC= QQINT(RAT,HBC1,HBC2)
	ENDIF

*   h+ -> bu

	IF(CMASS.LE.MBP+EPS)THEN
	 HBU= 0.
	ELSE
	 HBU1= 3.*VUB**2*CQCDM(CMASS,TANBETA,(MBP/CMASS)**2,EPS)
	 HBU2= 3.*VUB**2*CQCD(CMASS,TANBETA,(RMB/CMASS)**2,EPS)
	 IF(HBU2.LT.0.) HBU2= 0.
	 RAT= MBP/CMASS
	 HBU= QQINT(RAT,HBU1,HBU2)
	ENDIF

*   h+ -> tb :

	IF(CMASS.LE.MT+MBP)THEN
	 HBT= 0.
	ELSE
	 HBT1= 3.*CQCDM(CMASS,TANBETA,(MBP/CMASS)**2,(MT/CMASS)**2)
	 IF(CMASS.LE.RMT+RMB)THEN
	  HBT2= 0.
	 ELSE
	  HBT2= 3.*CQCD(CMASS,TANBETA,(RMB/CMASS)**2,(RMT/CMASS)**2)
	 ENDIF
	 IF(HBT2.LT.0.) HBT2= 0.
	 RAT= (MBP+MT)/CMASS
	 HBT= QQINT(RAT,HBT1,HBT2)
	ENDIF

*   h+ -> hW

	HTOT= 0.

	DO I=1,3
	 IF(CMASS.LE.MW+SMASS(I))THEN
	  HCWH(I)= 0.
	 ELSE
	  RH= S(I,1)*COSBETA-S(I,2)*SINBETA
	  CH= LAMB((SMASS(I)/CMASS)**2,(MW/CMASS)**2)
     C	   *LAMB((CMASS/MW)**2,(SMASS(I)/MW)**2)**2
	  HCWH(I)= GF/8./SQR2/PI*MW**4/CMASS*CH*RH**2
	 HTOT= HTOT+HCWH(I)
	 ENDIF
	ENDDO

*   h+ -> aW

	DO I=1,2
	 IF(CMASS.LE.MW+PMASS(I))THEN
	  HCWH(I+3)= 0.
	 ELSE
	  RH= P2(I,1)
	  CH= LAMB((PMASS(I)/CMASS)**2,(MW/CMASS)**2)
     C	    * LAMB((CMASS/MW)**2,(PMASS(I)/MW)**2)**2
	  HCWH(I+3)= GF/8./SQR2/PI*MW**4/CMASS*CH*RH**2
	  HTOT= HTOT+HCWH(I+3)
	 ENDIF
	ENDDO

*   h+ -> charginos+neutralinos

	STOT= 0.
	DO I=1,5
	DO J=1,2
	 IF (CMASS.LE.DABS(MNEU(I))+DABS(MCHA(J)))THEN
	  HCSUSY(I,J)= 0.
	 ELSE
	  HCSUSY(I,J)= 1./(16.*PI)*CMASS
     C	       * LAMB((MNEU(I)/CMASS)**2,(MCHA(J)/CMASS)**2)
     C	       * ((GHCNEUCHAL(I,J)**2+GHCNEUCHAR(I,J)**2)
     C	       * (1.-(MNEU(I)/CMASS)**2-(MCHA(J)/CMASS)**2)
     C	       - 4.*GHCNEUCHAL(I,J)*GHCNEUCHAR(I,J)
     C	       * MNEU(I)*MCHA(J)/CMASS**2)

	  STOT= STOT+HCSUSY(I,J)
	 ENDIF
	ENDDO
	ENDDO

*  Branching ratios

	HCWIDTH= HLN+HMN+HSU+HBU+HSC+HBC+HBT+HTOT+STOT
      
	HCBRM= HMN/HCWIDTH
	HCBRL= HLN/HCWIDTH
	HCBRSU= HSU/HCWIDTH
	HCBRBU= HBU/HCWIDTH
	HCBRSC= HSC/HCWIDTH
	HCBRBC= HBC/HCWIDTH
	HCBRBT= HBT/HCWIDTH
	HCBRWHT= HTOT/HCWIDTH
	HCBRSUSYT= STOT/HCWIDTH
	DO I=1,5
	 HCBRWH(I)= HCWH(I)/HCWIDTH
	ENDDO
	DO I=1,5
	 DO J=1,2
	  HCBRSUSY(I,J)= HCSUSY(I,J)/HCWIDTH
	 ENDDO
	ENDDO

	END


	SUBROUTINE SUBLEP(PAR,PROB)

*   Subroutine to check LEP2 constraints

	IMPLICIT NONE

	INTEGER IFAIL,I,J,K,PROB(*)
	INTEGER NhZind,NhZbb,NhZll,NhZinv,NhZjj,NhZgg
	INTEGER NhA4b,NhA4tau,NAAA6b,NAAZ4b
	INTEGER Ncccc02,Ncccc04,Ncccc05,Ncccc06,Ncccc08,Ncccc1
	INTEGER Nccgg02,Nccgg04,Nccgg05,Nccgg06,Nccgg08,Nccgg1
	INTEGER Ncctt02,Ncctt04,Ncctt05,Ncctt06,Ncctt08,Ncctt1
	INTEGER Ngggg02,Ngggg04,Ngggg05,Ngggg06,Ngggg08,Ngggg1
	INTEGER Nttgg02,Nttgg04,Nttgg05,Nttgg06,Nttgg08,Nttgg1
	INTEGER Ntttt02,Ntttt04,Ntttt05,Ntttt06,Ntttt08,Ntttt1

	DOUBLE PRECISION PAR(*),MCH(2),U(2,2),V(2,2),MNEU(5),NEU(5,5)
	DOUBLE PRECISION PI,S,SQRS,SQR2,CONV
	DOUBLE PRECISION SMASS(3),SCOMP(3,3),PMASS(2),PCOMP(2,2),CMASS
	DOUBLE PRECISION BRJJ(5),BRMM(5),BRLL(5),BRSS(5),BRCC(5)
	DOUBLE PRECISION BRBB(5),BRTT(5),BRWW(3),BRZZ(3),BRGG(5)
	DOUBLE PRECISION BRZG(5),BRHHH(4),BRHAA(3,3),BRHCHC(3)
	DOUBLE PRECISION BRHAZ(3,2),BRAHA(3),BRAHZ(2,3),BRHCW(5)
	DOUBLE PRECISION BRHIGGS(5),BRNEU(5,5,5),BRCHA(5,3),BRSUSY(5)
	DOUBLE PRECISION WIDTH(5)
	DOUBLE PRECISION hZind(1000,2),hZbb(1000,2),hZll(1000,2)
	DOUBLE PRECISION hZinv(1000,2),hZjj(1000,2),hZgg(1000,2)
	DOUBLE PRECISION hA4b(10000,3),hA4tau(10000,3)
	DOUBLE PRECISION AAA6b(10000,3),AAZ4b(10000,3)
	DOUBLE PRECISION MH,MA,M,TANB,h1,h2,R,Rmax,BRINV,M1,M2
	DOUBLE PRECISION GZINV,GZINVMAX,MCHAMIN,MCMIN,SIGNEU1,SIGNEU
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2,g,SW
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION LAMBDA,X,Y,Z,Q,O,DZ,E1,E2,SIG
	DOUBLE PRECISION MST1,MST2,MSB1,MSB2,MSQL
	DOUBLE PRECISION cccc02(1000,2),cccc04(1000,2),cccc05(1000,2)
	DOUBLE PRECISION cccc06(1000,2),cccc08(1000,2),cccc1(1000,2)
	DOUBLE PRECISION ccgg02(1000,2),ccgg04(1000,2),ccgg05(1000,2)
	DOUBLE PRECISION ccgg06(1000,2),ccgg08(1000,2),ccgg1(1000,2)
	DOUBLE PRECISION cctt02(1000,2),cctt04(1000,2),cctt05(1000,2)
	DOUBLE PRECISION cctt06(1000,2),cctt08(1000,2),cctt1(1000,2)
	DOUBLE PRECISION gggg02(1000,2),gggg04(1000,2),gggg05(1000,2)
	DOUBLE PRECISION gggg06(1000,2),gggg08(1000,2),gggg1(1000,2)
	DOUBLE PRECISION ttgg02(1000,2),ttgg04(1000,2),ttgg05(1000,2)
	DOUBLE PRECISION ttgg06(1000,2),ttgg08(1000,2),ttgg1(1000,2)
	DOUBLE PRECISION tttt02(1000,2),tttt04(1000,2),tttt05(1000,2)
	DOUBLE PRECISION tttt06(1000,2),tttt08(1000,2),tttt1(1000,2)
        DOUBLE PRECISION MAtest,ceff

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW
	COMMON/SUSYSPEC/MCH,U,V,MNEU,NEU
	COMMON/CONTR/GZINVMAX,MCHAMIN,MCMIN,SIGNEU1,SIGNEU
	COMMON/LEP/hZind,hZbb,hZll,hZinv,hZjj,hZgg,
     C		hA4b,hA4tau,AAA6b,AAZ4b,
     C          cccc02,cccc04,cccc05,cccc06,cccc08,cccc1,
     C          ccgg02,ccgg04,ccgg05,ccgg06,ccgg08,ccgg1,
     C          cctt02,cctt04,cctt05,cctt06,cctt08,cctt1,
     C          gggg02,gggg04,gggg05,gggg06,gggg08,gggg1,
     C          ttgg02,ttgg04,ttgg05,ttgg06,ttgg08,ttgg1,
     C          tttt02,tttt04,tttt05,tttt06,tttt08,tttt1,
     C		NhZind,NhZbb,NhZll,NhZinv,NhZjj,NhZgg,
     C		NhA4b,NhA4tau,NAAA6b,NAAZ4b,
     C          Ncccc02,Ncccc04,Ncccc05,Ncccc06,Ncccc08,Ncccc1,
     C          Nccgg02,Nccgg04,Nccgg05,Nccgg06,Nccgg08,Nccgg1,
     C          Ncctt02,Ncctt04,Ncctt05,Ncctt06,Ncctt08,Ncctt1,
     C          Ngggg02,Ngggg04,Ngggg05,Ngggg06,Ngggg08,Ngggg1,
     C          Nttgg02,Nttgg04,Nttgg05,Nttgg06,Nttgg08,Nttgg1,
     C          Ntttt02,Ntttt04,Ntttt05,Ntttt06,Ntttt08,Ntttt1
	COMMON/BRN/BRJJ,BRMM,BRLL,BRSS,BRCC,BRBB,BRTT,BRWW,BRZZ,BRGG,
     C		BRZG,BRHHH,BRHAA,BRHCHC,BRHAZ,BRAHA,BRAHZ,BRHCW,
     C		BRHIGGS,BRNEU,BRCHA,BRSUSY,WIDTH
	COMMON/HIGGSPEC/SMASS,SCOMP,PMASS,PCOMP,CMASS
	COMMON/SQSPEC/MST1,MST2,MSB1,MSB2


	LAMBDA(X,Y,Z)= DSQRT(X**2+Y**2+Z**2-2.*X*Y-2.*X*Z-2.*Y*Z)

        PI=4.D0*DATAN(1D0)
	SQR2=DSQRT(2.D0)
	SQRS=209.
	S=SQRS**2
	CONV=.3894D9
	g=(g1+g2)/2.
	SW=1.-(MW/MZ)**2
	TANB=PAR(3)
	h2=1./DSQRT(2.*DSQRT(2.D0)*(1.+TANB**2)*GF)
	h1=h2*TANB

* Test on chargino mass

	IF(DABS(MCH(1)).LT.MCHAMIN) PROB(1)=1

* Test on Z width into neutralinos

	IF(DABS(MNEU(1)).LT.MZ/2.)THEN
	 GZINV=MZ**3*GF/(12.*DSQRT(2.D0)*PI)
     C     *(NEU(1,3)**2-NEU(1,4)**2)**2*(1.-4*MNEU(1)**2/MZ**2)**1.5
 	 IF(GZINV.GT.GZINVMAX) PROB(2)=1
	ENDIF

* Test on neutralinos

	DZ = 1./(S-MZ**2)**2
	DO I=1,5
	 DO J=I,5
	  IF(DABS(MNEU(I))+DABS(MNEU(J)).LT.SQRS)THEN
	   O= (NEU(I,3)*NEU(J,3)-NEU(I,4)*NEU(J,4))
	   E1= (S+MNEU(I)**2-MNEU(J)**2)/(2.*SQRS)
	   E2= (S+MNEU(J)**2-MNEU(I)**2)/(2.*SQRS)
	   Q= lambda(S,MNEU(I)**2,MNEU(J)**2)/(2.*SQRS)
	   SIG= 1./(4.*PI)*Q/SQRS*DZ*(E1*E2+Q**2/3.-MNEU(I)*MNEU(J))
     C	       * g**2*O**2*(.25D0-SW+2.*SW**2)
	   SIG= CONV*SIG
	   IF(I.EQ.J)SIG= SIG/2.
	   IF(I.EQ.1)THEN
	    IF(J.GT.1 .AND. SIG.GT.SIGNEU1)PROB(2)=1
	   ELSE
	    IF(SIG.GT.SIGNEU)PROB(2)=1
	   ENDIF
	  ENDIF
	 ENDDO
	ENDDO

* Test on charged Higgs mass

	 IF(CMASS.LT.MCMIN) PROB(3)=1

* Test on Higgs decays into squarks
* First: MSQL = lightest squark mass

	MSQL=MST1
	IF(MSB1.LE.MST1) MSQL=MSB1
	
	IF(MSQL.LT.SMASS(1)/2.) PROB(14)=1
	IF(MSQL.LT.SMASS(2)/2.) PROB(15)=1
	IF(MSQL.LT.SMASS(3)/2.) PROB(16)=1
	IF(MSQL.LT.PMASS(1)/2.) PROB(17)=1
	IF(MSQL.LT.PMASS(2)/2.) PROB(18)=1
	IF(MST1+MSB1.LT.CMASS) PROB(19)=1
	
* Higgs Strahlung

	DO I=1,3
	MH=SMASS(I)
	IF(MH+MZ.LT.SQRS)THEN

	 R=(SCOMP(I,1)*h1+SCOMP(I,2)*h2)**2/(h1**2+h2**2)

*  ee -> hZ flavor independent

	 Rmax=1.
	 J=1
	 DOWHILE(hZind(J,1).LE.MH .AND. J.LT.NhZind)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZind(NhZind,1))
     C    Rmax=hZind(J-1,2)+(MH-hZind(J-1,1))/(hZind(J,1)
     C         -hZind(J-1,1))*(hZind(J,2)-hZind(J-1,2))
	 IF(R.GT.Rmax) PROB(4)=1

*  ee -> hZ, h -> bb

	 Rmax=1.
	 J=1
	 DOWHILE(hZbb(J,1).LE.MH .AND. J.LT.NhZbb)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZbb(NhZbb,1))
     C    Rmax=hZbb(J-1,2)+(MH-hZbb(J-1,1))/(hZbb(J,1)-hZbb(J-1,1))
     C         *(hZbb(J,2)-hZbb(J-1,2))
	 IF(R*BRBB(I).GT.Rmax) PROB(5)=1

*  ee -> hZ, h -> tautau

	 Rmax=1.
	 J=1
	 DOWHILE(hZll(J,1).LE.MH .AND. J.LT.NhZll)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZll(NhZll,1))
     C    Rmax=hZll(J-1,2)+(MH-hZll(J-1,1))/(hZll(J,1)-hZll(J-1,1))
     C         *(hZll(J,2)-hZll(J-1,2))
	 IF(R*BRLL(I).GT.Rmax) PROB(6)=1

*  ee -> hZ, h -> invisible

	 Rmax=1.
	 J=1
	 DOWHILE(hZinv(J,1).LE.MH .AND. J.LT.NhZinv)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZinv(NhZinv,1))
     C    Rmax=hZinv(J-1,2)+(MH-hZinv(J-1,1))/(hZinv(J,1)-hZinv(J-1,1))
     C         *(hZinv(J,2)-hZinv(J-1,2))
	 IF(I.EQ.1)
     C	  BRINV=BRNEU(1,1,1)+BRHAA(1,1)*BRNEU(4,1,1)**2
	 IF(I.EQ.2)
     C	  BRINV=BRNEU(2,1,1)+BRHAA(2,1)*BRNEU(4,1,1)**2
     C		+BRHHH(1)*BRNEU(1,1,1)**2
	 IF(I.EQ.3)
     C	  BRINV=BRNEU(3,1,1)+BRHAA(3,1)*BRNEU(4,1,1)**2
     C		+BRHHH(2)*BRNEU(1,1,1)**2
	 IF(R*BRINV.GT.Rmax) PROB(7)=1

*  ee -> hZ, h -> 2jets

	 Rmax=1.
	 J=1
	 DOWHILE(hZjj(J,1).LE.MH .AND. J.LT.NhZjj)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZjj(NhZjj,1))
     C    Rmax=hZjj(J-1,2)+(MH-hZjj(J-1,1))/(hZjj(J,1)-hZjj(J-1,1))
     C         *(hZjj(J,2)-hZjj(J-1,2))
	 IF(R*(BRJJ(I)+BRSS(I)+BRCC(I)+BRBB(I)).GT.Rmax) PROB(8)=1

*  ee -> hZ, h -> 2photons

	 Rmax=1.
	 J=1
	 DOWHILE(hZgg(J,1).LE.MH .AND. J.LT.NhZgg)
	  J=J+1
	 ENDDO
	 IF(J.GT.1 .AND. MH.LT.hZgg(NhZgg,1))
     C    Rmax=hZgg(J-1,2)+(MH-hZgg(J-1,1))/(hZgg(J,1)-hZgg(J-1,1))
     C         *(hZgg(J,2)-hZgg(J-1,2))
	 IF(R*BRGG(I).GT.Rmax) PROB(9)=1

*  ee -> hZ, h -> AA -> 4bs

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  Rmax=1.
	  DO K=1,NAAZ4b
	   IF(AINT(MA).EQ.AAZ4b(K,1) .AND. AINT(MH).EQ.AAZ4b(K,2))THEN
	    Rmax=AAZ4b(K,3)
	    GOTO 1
	   ENDIF
	  ENDDO
 1	  IF(R*BRHAA(I,1)*BRBB(4)**2.GT.Rmax) PROB(10)=1
	 ENDIF

	 MA=SMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  Rmax=1.
	  DO K=1,NAAZ4b
	   IF(AINT(MA).EQ.AAZ4b(K,1) .AND. AINT(MH).EQ.AAZ4b(K,2))THEN
	    Rmax=AAZ4b(K,3)
	    GOTO 2
	   ENDIF
	  ENDDO
 2	  IF(R*BRHHH(I-1)*BRBB(1)**2.GT.Rmax) PROB(10)=1
	 ENDIF

*  ee -> hZ, h -> AA -> cccc

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.OR.MH.GT.90.) GOTO 502
	  IF(MA.LT.2.OR.MA.GT.12.) GOTO 502
	  ceff=R*BRHAA(I,1)*BRCC(4)**2
	  IF(ceff.GE.0.2.AND.ceff.LT.0.4) GOTO 102
	  IF(ceff.GE.0.4.AND.ceff.LT.0.5) GOTO 202
	  IF(ceff.GE.0.5.AND.ceff.LT.0.6) GOTO 2025
	  IF(ceff.GE.0.6.AND.ceff.LT.0.8) GOTO 302
	  IF(ceff.GE.0.8.AND.ceff.LT.1) GOTO 402
	  GOTO 502
	 ENDIF

 102	 Rmax=.2
	 J=1
	 DOWHILE(cccc02(J,1).LE.MH .AND. J.LT.Ncccc02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cccc02(Ncccc02,1))
     C	  MAtest=cccc02(J-1,2)+(MH-cccc02(J-1,1))/
     C	   (cccc02(J,1)-cccc02(J-1,1))
     C	   *(cccc02(J,2)-cccc02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 502

 202	 Rmax=.4
	 J=1
	 DOWHILE(cccc04(J,1).LE.MH .AND. J.LT.Ncccc04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cccc04(Ncccc04,1))
     C	  MAtest=cccc04(J-1,2)+(MH-cccc04(J-1,1))/
     C	   (cccc04(J,1)-cccc04(J-1,1))
     C	   *(cccc04(J,2)-cccc04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 502

 2025	 Rmax=.5
	 J=1
	 DOWHILE(cccc05(J,1).LE.MH .AND. J.LT.Ncccc05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cccc05(Ncccc05,1))
     C	  MAtest=cccc05(J-1,2)+(MH-cccc05(J-1,1))/
     C	   (cccc05(J,1)-cccc05(J-1,1))
     C	   *(cccc05(J,2)-cccc05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 502

 302	 Rmax=.6
	 J=1
	 DOWHILE(cccc06(J,1).LE.MH .AND. J.LT.Ncccc06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cccc06(Ncccc06,1))
     C	  MAtest=cccc06(J-1,2)+(MH-cccc06(J-1,1))/
     C	   (cccc06(J,1)-cccc06(J-1,1))
     C	   *(cccc06(J,2)-cccc06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 502

 402	 Rmax=.8
	 J=1
	 DOWHILE(cccc08(J,1).LE.MH .AND. J.LT.Ncccc08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cccc08(Ncccc08,1))
     C	  MAtest=cccc08(J-1,2)+(MH-cccc08(J-1,1))/
     C	   (cccc08(J,1)-cccc08(J-1,1))
     C	   *(cccc08(J,2)-cccc08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
 502	 CONTINUE

*  ee -> hZ, h -> AA -> ccjj

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.or.MH.GT.90.) GOTO 512
	  IF(MA.LT.2.or.MA.GT.12.) GOTO 512
	  ceff=R*BRHAA(I,1)*BRCC(4)*(BRJJ(4)+BRSS(4))*2.
	  IF(ceff.GE.0.2.and.ceff.LT.0.4) GOTO 112
	  IF(ceff.GE.0.4.and.ceff.LT.0.5) GOTO 212
	  IF(ceff.GE.0.5.and.ceff.LT.0.6) GOTO 2125
	  IF(ceff.GE.0.6.and.ceff.LT.0.8) GOTO 312
	  IF(ceff.GE.0.8.and.ceff.LT.1) GOTO 412
	  GOTO 512
	 ENDIF

 112	 Rmax=.2
	 J=1
	 DOWHILE(ccgg02(J,1).LE.MH .AND. J.LT.Nccgg02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ccgg02(Nccgg02,1))
     C	  MAtest=ccgg02(J-1,2)+(MH-ccgg02(J-1,1))/
     C	   (ccgg02(J,1)-ccgg02(J-1,1))
     C	   *(ccgg02(J,2)-ccgg02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 512

 212	 Rmax=.4
	 J=1
	 DOWHILE(ccgg04(J,1).LE.MH .AND. J.LT.Nccgg04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ccgg04(Nccgg04,1))
     C	  MAtest=ccgg04(J-1,2)+(MH-ccgg04(J-1,1))/
     C	   (ccgg04(J,1)-ccgg04(J-1,1))
     C	   *(ccgg04(J,2)-ccgg04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 512

 2125	 Rmax=.5
	 J=1
	 DOWHILE(ccgg05(J,1).LE.MH .AND. J.LT.Nccgg05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ccgg05(Nccgg05,1))
     C	  MAtest=ccgg05(J-1,2)+(MH-ccgg05(J-1,1))/
     C	   (ccgg05(J,1)-ccgg05(J-1,1))
     C	   *(ccgg05(J,2)-ccgg05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1	
	 GOTO 512

 312	 Rmax=.6
	 J=1
	 DOWHILE(ccgg06(J,1).LE.MH .AND. J.LT.Nccgg06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ccgg06(Nccgg06,1))
     C    MAtest=ccgg06(J-1,2)+(MH-ccgg06(J-1,1))/
     C	   (ccgg06(J,1)-ccgg06(J-1,1))
     C	   *(ccgg06(J,2)-ccgg06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 512

 412	 Rmax=.8
	 J=1
	 DOWHILE(ccgg08(J,1).LE.MH .AND. J.LT.Nccgg08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ccgg08(Nccgg08,1))
     C	  MAtest=ccgg08(J-1,2)+(MH-ccgg08(J-1,1))/
     C	   (ccgg08(J,1)-ccgg08(J-1,1))
     C	   *(ccgg08(J,2)-ccgg08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
 512	 CONTINUE

*  ee -> hZ, h -> AA -> cctautau

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.or.MH.GT.90.) GOTO 522
	  IF(MA.LT.3.4.or.MA.GT.12.) GOTO 522
	  ceff=R*BRHAA(I,1)*BRCC(4)*BRLL(4)*2.
	  IF(ceff.GE.0.2.and.ceff.LT.0.4) GOTO 122
	  IF(ceff.GE.0.4.and.ceff.LT.0.5) GOTO 222
	  IF(ceff.GE.0.5.and.ceff.LT.0.6) GOTO 2225
	  IF(ceff.GE.0.6.and.ceff.LT.0.8) GOTO 322
	  IF(ceff.GE.0.8.and.ceff.LT.1) GOTO 422
	  GOTO 522
	 ENDIF

 122	 Rmax=.2
	 J=1
	 DOWHILE(cctt02(J,1).LE.MH .AND. J.LT.Ncctt02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cctt02(Ncctt02,1))
     C	  MAtest=cctt02(J-1,2)+(MH-cctt02(J-1,1))/
     C	   (cctt02(J,1)-cctt02(J-1,1))
     C	   *(cctt02(J,2)-cctt02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 522

 222	 Rmax=.4
	 J=1
	 DOWHILE(cctt04(J,1).LE.MH .AND. J.LT.Ncctt04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cctt04(Ncctt04,1))
     C	  MAtest=cctt04(J-1,2)+(MH-cctt04(J-1,1))/
     C	   (cctt04(J,1)-cctt04(J-1,1))
     C	   *(cctt04(J,2)-cctt04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 522

 2225	 Rmax=.5
	 J=1
	 DOWHILE(cctt05(J,1).LE.MH .AND. J.LT.Ncctt05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cctt05(Ncctt05,1))
     C	  MAtest=cctt05(J-1,2)+(MH-cctt05(J-1,1))/
     C	   (cctt05(J,1)-cctt05(J-1,1))
     C	   *(cctt05(J,2)-cctt05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 522

 322	 Rmax=.6
	 J=1
	 DOWHILE(cctt06(J,1).LE.MH .AND. J.LT.Ncctt06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cctt06(Ncctt06,1))
     C	  MAtest=cctt06(J-1,2)+(MH-cctt06(J-1,1))/
     C	   (cctt06(J,1)-cctt06(J-1,1))
     C	   *(cctt06(J,2)-cctt06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 522

 422	 Rmax=.8
	 J=1
	 DOWHILE(cctt08(J,1).LE.MH .AND. J.LT.Ncctt08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.cctt08(Ncctt08,1))
     C	  MAtest=cctt08(J-1,2)+(MH-cctt08(J-1,1))/
     C	   (cctt08(J,1)-cctt08(J-1,1))
     C	   *(cctt08(J,2)-cctt08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
 522	 CONTINUE

*  ee -> hZ, h -> AA -> jjjj

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.or.MH.GT.90.) GOTO 532
	  IF(MA.LT.0.2.or.MA.GT.12.) GOTO 532
	  ceff=R*BRHAA(I,1)*(BRJJ(4)+BRSS(4))**2
	  IF(ceff.GE.0.2.and.ceff.LT.0.4) GOTO 132
	  IF(ceff.GE.0.4.and.ceff.LT.0.5) GOTO 232
	  IF(ceff.GE.0.5.and.ceff.LT.0.6) GOTO 2325
	  IF(ceff.GE.0.6.and.ceff.LT.0.8) GOTO 332
	  IF(ceff.GE.0.8.and.ceff.LT.1) GOTO 432
	  GOTO 532
	 ENDIF

 132	 Rmax=.2
	 J=1
	 DOWHILE(gggg02(J,1).LE.MH .AND. J.LT.Ngggg02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.gggg02(Ngggg02,1))
     C	  MAtest=gggg02(J-1,2)+(MH-gggg02(J-1,1))/
     C	   (gggg02(J,1)-gggg02(J-1,1))
     C	   *(gggg02(J,2)-gggg02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 532

 232	 Rmax=.4
	 J=1
	 DOWHILE(gggg04(J,1).LE.MH .AND. J.LT.Ngggg04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.gggg04(Ngggg04,1))
     C	  MAtest=gggg04(J-1,2)+(MH-gggg04(J-1,1))/
     C	   (gggg04(J,1)-gggg04(J-1,1))
     C	   *(gggg04(J,2)-gggg04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 532

 2325	 Rmax=.5
	 J=1
	 DOWHILE(gggg05(J,1).LE.MH .AND. J.LT.Ngggg05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.gggg05(Ngggg05,1))
     C	  MAtest=gggg05(J-1,2)+(MH-gggg05(J-1,1))/
     C	   (gggg05(J,1)-gggg05(J-1,1))
     C	   *(gggg05(J,2)-gggg05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 532

 332	 Rmax=.6
	 J=1
	 DOWHILE(gggg06(J,1).LE.MH .AND. J.LT.Ngggg06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.gggg06(Ngggg06,1))
     C	  MAtest=gggg06(J-1,2)+(MH-gggg06(J-1,1))/
     C	   (gggg06(J,1)-gggg06(J-1,1))
     C	   *(gggg06(J,2)-gggg06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 532

 432	 Rmax=.8
	 J=1
	 DOWHILE(gggg08(J,1).LE.MH .AND. J.LT.Ngggg08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.gggg08(Ngggg08,1))
     C	  MAtest=gggg08(J-1,2)+(MH-gggg08(J-1,1))/
     C	   (gggg08(J,1)-gggg08(J-1,1))
     C	   *(gggg08(J,2)-gggg08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
 532	 CONTINUE

*  ee -> hZ, h -> AA -> tautaujj

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.or.MH.GT.90.) GOTO 542
	  IF(MA.LT.3.4.or.MA.GT.12.) GOTO 542
	  ceff=R*BRHAA(I,1)*BRLL(4)*(BRJJ(4)+BRSS(4))*2.
	  IF(ceff.GE.0.2.and.ceff.LT.0.4) GOTO 142
	  IF(ceff.GE.0.4.and.ceff.LT.0.5) GOTO 242
	  IF(ceff.GE.0.5.and.ceff.LT.0.6) GOTO 2425
	  IF(ceff.GE.0.6.and.ceff.LT.0.8) GOTO 342
	  IF(ceff.GE.0.8.and.ceff.LT.1) GOTO 442
	  GOTO 542
	 ENDIF

 142	 Rmax=.2
	 J=1
	 DOWHILE(ttgg02(J,1).LE.MH .AND. J.LT.Nttgg02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ttgg02(Nttgg02,1))
     C	  MAtest=ttgg02(J-1,2)+(MH-ttgg02(J-1,1))/
     C	   (ttgg02(J,1)-ttgg02(J-1,1))
     C	   *(ttgg02(J,2)-ttgg02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 542

 242	 Rmax=.4
	 J=1
	 DOWHILE(ttgg04(J,1).LE.MH .AND. J.LT.Nttgg04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ttgg04(Nttgg04,1))
     C	  MAtest=ttgg04(J-1,2)+(MH-ttgg04(J-1,1))/
     C	   (ttgg04(J,1)-ttgg04(J-1,1))
     C	   *(ttgg04(J,2)-ttgg04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 542

 2425	 Rmax=.5
	 J=1
	 DOWHILE(ttgg05(J,1).LE.MH .AND. J.LT.Nttgg05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ttgg05(Nttgg05,1))
     C	  MAtest=ttgg05(J-1,2)+(MH-ttgg05(J-1,1))/
     C	   (ttgg05(J,1)-ttgg05(J-1,1))
     C	   *(ttgg05(J,2)-ttgg05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 542

 342	 Rmax=.6
	 J=1
	 DOWHILE(ttgg06(J,1).LE.MH .AND. J.LT.Nttgg06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ttgg06(Nttgg06,1))
     C	  MAtest=ttgg06(J-1,2)+(MH-ttgg06(J-1,1))/
     C	   (ttgg06(J,1)-ttgg06(J-1,1))
     C	   *(ttgg06(J,2)-ttgg06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 542

 442	 Rmax=.8
	 J=1
	 DOWHILE(ttgg08(J,1).LE.MH .AND. J.LT.Nttgg08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.ttgg08(Nttgg08,1))
     C	  MAtest=ttgg08(J-1,2)+(MH-ttgg08(J-1,1))/
     C	   (ttgg08(J,1)-ttgg08(J-1,1))
     C	   *(ttgg08(J,2)-ttgg08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
 542	 CONTINUE

*  ee -> hZ, h -> AA -> tautautautau

	 MA=PMASS(1)
	 IF(MH.GT.2.*MA)THEN
	  IF(MH.LT.40.or.MH.GT.90.) GOTO 552
	  IF(MA.LT.3.4.or.MA.GT.12.) GOTO 552
	  ceff=R*BRHAA(I,1)*BRLL(4)**2
	  IF(ceff.GE.0.2.and.ceff.LT.0.4) GOTO 152
	  IF(ceff.GE.0.4.and.ceff.LT.0.5) GOTO 252
	  IF(ceff.GE.0.5.and.ceff.LT.0.6) GOTO 2525
	  IF(ceff.GE.0.6.and.ceff.LT.0.8) GOTO 352
	  IF(ceff.GE.0.8.and.ceff.LT.1) GOTO 452
	  GOTO 552
	 ENDIF

 152	 Rmax=.2
	 J=1
	 DOWHILE(tttt02(J,1).LE.MH .AND. J.LT.Ntttt02)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.tttt02(Ntttt02,1))
     C	  MAtest=tttt02(J-1,2)+(MH-tttt02(J-1,1))/
     C	   (tttt02(J,1)-tttt02(J-1,1))
     C	   *(tttt02(J,2)-tttt02(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 552

 252	 Rmax=.4
	 J=1
	 DOWHILE(tttt04(J,1).LE.MH .AND. J.LT.Ntttt04)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.tttt04(Ntttt04,1))
     C	  MAtest=tttt04(J-1,2)+(MH-tttt04(J-1,1))/
     C	   (tttt04(J,1)-tttt04(J-1,1))
     C	   *(tttt04(J,2)-tttt04(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 552

 2525	 Rmax=.5
	 J=1
	 DOWHILE(tttt05(J,1).LE.MH .AND. J.LT.Ntttt05)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.tttt05(Ntttt05,1))
     C	  MAtest=tttt05(J-1,2)+(MH-tttt05(J-1,1))/
     C	   (tttt05(J,1)-tttt05(J-1,1))
     C	   *(tttt05(J,2)-tttt05(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	 GOTO 552

 352	 Rmax=.6
	 J=1
	 DOWHILE(tttt06(J,1).LE.MH .AND. J.LT.Ntttt06)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.tttt06(Ntttt06,1))
     C	  MAtest=tttt06(J-1,2)+(MH-tttt06(J-1,1))/
     C	   (tttt06(J,1)-tttt06(J-1,1))
     C	   *(tttt06(J,2)-tttt06(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	GOTO 552

 452	 Rmax=.8
	 J=1
	 DOWHILE(tttt08(J,1).LE.MH .AND. J.LT.Ntttt08)
	  J=J+1
	 ENDDO
	 MAtest=0.
	 IF(J.GT.1 .AND. MH.LT.tttt08(Ntttt08,1))
     C	  MAtest=tttt08(J-1,2)+(MH-tttt08(J-1,1))/
     C	   (tttt08(J,1)-tttt08(J-1,1))
     C	   *(tttt08(J,2)-tttt08(J-1,2))
	 IF(ceff.GT.Rmax.and.MA.LE.MAtest) PROB(22)=1
	
552	 CONTINUE

	ENDIF
	ENDDO

* Associated production

	DO I=1,3
	DO J=1,2
	MH=SMASS(I)
	MA=PMASS(J)
	IF(MH+MA.LT.SQRS)THEN

	 R=PCOMP(J,1)**2*(SCOMP(I,1)*h2-SCOMP(I,2)*h1)**2
     C	   /(h1**2+h2**2)

	 M1=MIN(MH,MA)
	 M2=MAX(MH,MA)

*  ee -> hA -> 4bs

	 Rmax=1.
	 DO K=1,NhA4b
	  IF(AINT(M1).EQ.hA4b(K,1) .AND. AINT(M2).EQ.hA4b(K,2))THEN
	   Rmax=hA4b(K,3)
	   GOTO 3
	  ENDIF
	 ENDDO
 3	 IF(R*BRBB(I)*BRBB(J).GT.Rmax) PROB(11)=1

*  ee -> hA -> 4taus

	 Rmax=1.
	 DO K=1,NhA4tau
	  IF(AINT(M1).EQ.hA4tau(K,1) .AND. AINT(M2).EQ.hA4tau(K,2))THEN
	   Rmax=hA4tau(K,3)
	   GOTO 4
	  ENDIF
	 ENDDO
 4	 IF(R*BRLL(I)*BRLL(J).GT.Rmax) PROB(12)=1

*  ee -> hA -> AAA -> 6bs

	 IF(MH.GT.2.*MA)THEN
	  Rmax=1.
	  DO K=1,NAAA6b
	   IF(AINT(MA).EQ.AAA6b(K,1) .AND. AINT(MH).EQ.AAA6b(K,2))THEN
	    Rmax=AAA6b(K,3)
	    GOTO 5
	   ENDIF
	  ENDDO
 5	  IF(R*BRHAA(I,2*J-1)*BRBB(J)**3.GT.Rmax) PROB(13)=1
	 ENDIF


	ENDIF
	ENDDO	  
	ENDDO

	END


	SUBROUTINE RGES(PAR,PROB,NPROB)

*   Subroutine to check if there is a Landau Pole
*   for the couplings lambda, kappa, htop and hbot

	IMPLICIT NONE

	INTEGER I,IFAIL,NN,NPROB,PROB(*),STEP
	PARAMETER (NN=7)

	DOUBLE PRECISION PAR(*),EPS,X1,X2,Y(NN)
	DOUBLE PRECISION MBOT,MTOP,HBOT,HTOP,NF,PI
	DOUBLE PRECISION LAMBDA,KAPPA,TANBETA,h1,h2,YMAX
	DOUBLE PRECISION g1z,g2z,g3z,g1t,g2t,g3t,g1s,g2s,g3s
	DOUBLE PRECISION BETA,g,b,q,ALSMT
	DOUBLE PRECISION ALSMZ,ALP0,GF,g1,g2,ALPMZ,S2TW
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW,MASM2

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW

	EXTERNAL DERIVS,RKQS

	BETA(g,b,q)=g/(1.-b*g/(16.*PI**2)*DLOG(q**2))

	EPS=1.D-8
	PI=4.*DATAN(1.D0)
	ALPMZ=1./128.92D0
	S2TW=.2312D0
	LAMBDA=PAR(1)
	KAPPA=PAR(2)
	TANBETA=PAR(3)
	Q=DSQRT((2.*PAR(7)**2+PAR(8)**2+PAR(9)**2)/4.+MT**2)

*  At MZ

	h2=1/DSQRT(2.*DSQRT(2.D0)*(1.+TANBETA**2)*GF)
	h1=h2*TANBETA
	g1z=4.*PI*ALPMZ/(1.-S2TW)
	g2z=4.*PI*ALPMZ/S2TW
	g3z=4.*PI*ALSMZ

*   At MB

	HBOT=MB/DABS(h2)
	
*  At MTOP

	NF=5.
	g1t=BETA(g1z,2.*NF/3.-1./10.,MT/MZ)
	g2t=BETA(g2z,2.*NF/3.-22./3.+1./6.,MT/MZ)
	g3t=BETA(g3z,2.*NF/3.-11.,MT/MZ)
	ALSMT=g3t/(4.*PI)
	HTOP=MT/(1.+4.*ALSMT/(3.*PI)+11.*(ALSMT/PI)**2)/h1
	HBOT=HBOT*(1.-23./(12.*PI)*ALSMT*DLOG(MT**2/MB**2))**(12./23.)

*  At Q~Msusy

	NF=6.
	g1s=BETA(g1t,2.*NF/3.-1./10.,Q/MT)
	g2s=BETA(g2t,2.*NF/3.-43./6.,Q/MT)
	g3s=BETA(g3t,2.*NF/3.-11.,Q/MT)
	HBOT=HBOT*(1.+7./(4.*PI)*ALSMT*DLOG(Q**2/MT**2))**(-4./7.)
     C	 *(1.+(9.*HBOT**2+HTOP**2)/(64.*PI**2)*DLOG(Q**2/MT**2))	
	HTOP=HTOP*(1.+7./(4.*PI)*ALSMT*DLOG(Q**2/MT**2))**(-4./7.)
     C	 *(1.+(9.*HTOP**2+HBOT**2)/(64.*PI**2)*DLOG(Q**2/MT**2))	

*   MASM2: ~MSSM-like pseudoscalar mass**2 (if >> MZ)

	MASM2= PAR(4)*(PAR(5)+PAR(2)*PAR(4)/PAR(1))*(TANBETA+1/TANBETA)
     
        IF(MASM2.GT.MT**2) THEN
	  HTOP=HTOP*(1.+2.*HBOT**2*DLOG(MASM2/MT**2)/(64.*PI**2))
	  HBOT=HBOT*(1.+2.*HTOP**2*DLOG(MASM2/MT**2)/(64.*PI**2))
        END IF

	STEP=2
	X1=0.
	X2=1./2.**STEP
 1	Y(1)=g1s
	Y(2)=g2s
	Y(3)=g3s
	Y(4)=LAMBDA**2
	Y(5)=KAPPA**2
	Y(6)=HTOP**2
	Y(7)=HBOT**2
	CALL ODEINT(Y,NN,X1,X2,EPS,DERIVS,RKQS,IFAIL)
	IF(DABS(Y(2)-5./3*Y(1)).GT.EPS .AND. X2*2.**STEP.LT.1./EPS)THEN
	 STEP=STEP+1
	 X2=X2+DSIGN(1.D0/2.**STEP,Y(2)-5./3*Y(1))
	 GOTO 1
	ENDIF
	YMAX=0.
	DO I=4,7
	 YMAX=MAX(YMAX,Y(I))
	ENDDO
	IF(IFAIL.NE.0 .OR. YMAX.GT.4.*PI) PROB(NPROB-1)=1

	END


	SUBROUTINE ODEINT(YSTART,NVAR,X1,X2,EPS,DERIVS,RKQS,IFAIL)

*   Driver subroutine to integrate Ordinary Differential Equations
*   using the Runge-Kutta 5th order method with adaptative step size
*   (from Numerical Recipes)

	IMPLICIT NONE

	INTEGER NVAR,IFAIL,MAXSTP,NSTP,I
	PARAMETER(MAXSTP=10000)

	DOUBLE PRECISION YSTART(NVAR),Y(NVAR),YSCAL(NVAR)
	DOUBLE PRECISION DYDX(NVAR),X,X1,X2,EPS,TINY
	DOUBLE PRECISION H1,HMIN,H,HDID,HNEXT

	EXTERNAL DERIVS,RKQS

	IFAIL=0
	TINY=EPS**4
	H1=DSQRT(EPS)*DABS(X2-X1)
	HMIN=EPS**2*DABS(X2-X1)

	X=X1
	H=DSIGN(H1,X2-X1)
	DO I=1,NVAR
	 Y(I)=YSTART(I)
	ENDDO

	DO NSTP=1,MAXSTP

	 CALL DERIVS(NVAR,X,Y,DYDX)

	 DO I=1,NVAR
	  YSCAL(I)=DABS(Y(I))+DABS(H*DYDX(I))+TINY
	 ENDDO

	 IF((X+H-X2)*(X+H-X1).GT.0.) H=X2-X

	 CALL RKQS(Y,DYDX,NVAR,X,H,EPS,YSCAL,HDID,HNEXT,DERIVS,IFAIL)

	 IF(IFAIL.NE.0)RETURN

	 IF((X-X2)*(X2-X1).GE.0.)THEN
	  DO I=1,NVAR
	   YSTART(I)=Y(I)
	  ENDDO
	  RETURN
	 ENDIF

	 IF(ABS(HNEXT).LT.HMIN)THEN
	  IFAIL=1
	  RETURN
	 ENDIF

	 H=HNEXT

	ENDDO

	IFAIL=2

	RETURN
	END


	SUBROUTINE DERIVS(N,X,Y,F)

*   2-loop Renormalization group equations for g1, g2, g3,
*   lambda, kappa, htop and hbot, to be integrated by ODEINT

	IMPLICIT NONE

	INTEGER N
	DOUBLE PRECISION X,Y(N),F(N),PI,c2

	PI=4.*DATAN(1.D0)
	c2=1./(16.*PI**2)

	F(1)= 11.*Y(1)**2
     .	    + c2*Y(1)**2*(199./9.*Y(1) + 9.*Y(2) + 88./3.*Y(3)
     .	    - 2.*Y(4) - 26./3.*Y(6) - 14./3.*Y(7))

	F(2) = Y(2)**2
     .	    + c2*Y(2)**2*(3.*Y(1) + 25.*Y(2) + 24.*Y(3)
     .	    - 2.*Y(4) - 6.*Y(6) - 6.*Y(7))

	F(3)= -3.*Y(3)**2
     .	    + c2*Y(3)**2*(11./3.*Y(1) + 9.*Y(2) + 14.*Y(3)
     .	    - 4 .*Y(6) - 4.*Y(7))

	F(4)= Y(4)*(-Y(1) - 3.*Y(2)
     .	    + 4.*Y(4) + 2.*Y(5) + 3.*Y(6) + 3.*Y(7))
     .	    + c2*Y(4)*(Y(4)*(2.*Y(1) + 6.*Y(2)
     .	    - 10.*Y(4) - 12.*Y(5) - 9.*Y(6) - 9.*Y(7))
     .	    + 23./2.*Y(1)**2 + 15./2.*Y(2)**2 + 3.*Y(1)*Y(2)
     .	    + 4./3.*Y(1)*Y(6) - 2./3.*Y(1)*Y(7)
     .	    + 16.*Y(3)*Y(6) + 16.*Y(3)*Y(7)
     .	    - 9.*Y(6)**2 - 9.*Y(7)**2 - 6.*Y(6)*Y(7) - 8.*Y(5)**2)

	F(5)= 6.*Y(5)*(Y(4) + Y(5))
     .	    + c2*Y(5)*(24.*Y(5)*(Y(4) - Y(5))
     .	    + 6.*Y(4)*(Y(1) + 3.*Y(2) - 2.*Y(4) + 3.*Y(6) + 3.*Y(7)))

	F(6)= Y(6)*(-13./9.*Y(1) - 3.*Y(2) - 16./3.*Y(3)
     .	    + Y(4) + 6.*Y(6) + Y(7))
     .	    + c2*Y(6)*(Y(6)*(2.*Y(1) + 6.*Y(2) + 16.*Y(3)
     .	    - 3.*Y(4) - 22.*Y(6) - 5.*Y(7))
     .	    + 2743./162.*Y(1)**2 + 15./2.*Y(2)** 2- 16./9.*Y(3)**2
     .	    + 5./3.*Y(1)*Y(2) + 136./27.*Y(1)*Y(3) + 8.*Y(2)*Y(3)
     .	    + 2./3.*Y(1)*Y(7) - 3.*Y(4)**2 - 2.*Y(4)*Y(5)
     .	    - 4.*Y(4)*Y(7) - 5.*Y(7)**2)

	F(7)= Y(7)*(-7./9.*Y(1) - 3.*Y(2) - 16./3.*Y(3)
     .	    + Y(4) + Y(6) + 6.*Y(7))
     .	    + c2*Y(7)*(Y(7)*(2./3.*Y(1) + 6.*Y(2) + 16.*Y(3)
     .	    - 3.*Y(4) - 5.*Y(6) - 22.*Y(7))
     .	    + 1435./162.*Y(1)**2 + 15./2.*Y(2)**2 - 16./9.*Y(3)**2
     .	    + 5./3.*Y(1)*Y(2) + 40./27.*Y(1)*Y(3) + 8.*Y(2)*Y(3)
     .	    + 4./3.*Y(1)*Y(6) - 3.*Y(4)**2 - 2.*Y(4)*Y(5)
     .	    - 4.*Y(4)*Y(6) - 5.*Y(6)**2)

	RETURN
	END


	SUBROUTINE RKQS(Y,DYDX,N,X,HTRY,EPS,YSCAL,HDID,HNEXT,DERIVS,
     .			IFAIL)

*   Stepper subroutine for ODEINT

	IMPLICIT NONE

	INTEGER IFAIL,I,N
	DOUBLE PRECISION EPS,HDID,HNEXT,HTRY,X,DYDX(N),Y(N),YSCAL(N)
	DOUBLE PRECISION ERRMAX,H,HTEMP,XNEW,YERR(N),YTEMP(N)
	DOUBLE PRECISION SAFETY,PGROW,PSHRNK,ERRCON

	EXTERNAL DERIVS

	SAFETY=.9D0
	PGROW=-.2D0
	PSHRNK=-.25D0
	ERRCON=(5./SAFETY)**(1./PGROW)

	H=HTRY

1	CALL RKCK(Y,DYDX,N,X,H,YTEMP,YERR,DERIVS)

	ERRMAX=0.
	DO I=1,N
	 ERRMAX=MAX(ERRMAX,DABS(YERR(I)/YSCAL(I)))
	ENDDO
	ERRMAX=ERRMAX/EPS

	IF(ERRMAX.GT.1.)THEN

	 HTEMP=SAFETY*H*(ERRMAX**PSHRNK)
	 H=SIGN(MAX(DABS(HTEMP),.1D0*DABS(H)),H)
	 XNEW=X+H
	 IF(XNEW.EQ.X)THEN
	  IFAIL=3
	  RETURN
	 ENDIF
	 GOTO 1

	ELSE

	 IF(ERRMAX.GT.ERRCON)THEN
	  HNEXT=SAFETY*H*(ERRMAX**PGROW)
	 ELSE
	   HNEXT=5.*H
	 ENDIF
	 HDID=H
	 X=X+H
	 DO I=1,N
	  Y(I)=YTEMP(I)
	 ENDDO
	 RETURN

	ENDIF

	END


	SUBROUTINE RKCK(Y,DYDX,N,X,H,YOUT,YERR,DERIVS)

*   Algorithm subroutine for ODEINT

	IMPLICIT NONE

	INTEGER I,N,NMAX

	DOUBLE PRECISION H,X,DYDX(N),Y(N),YERR(N),YOUT(N)
	DOUBLE PRECISION AK2(N),AK3(N),AK4(N),AK5(N),AK6(N),YTEMP(N)
	DOUBLE PRECISION A2,A3,A4,A5,A6,C1,C3,C4,C6
	DOUBLE PRECISION B21,B31,B32,B41,B42,B43,B51,B52,B53,B54
	DOUBLE PRECISION B61,B62,B63,B64,B65,DC1,DC3,DC4,DC5,DC6

	PARAMETER(A2=.2D0, A3=.3D0, A4=.6D0, A5=1., A6=.875D0, B21=.2D0,
     .	 B31=3./40., B32=9./40., B41=.3D0, B42=-.9D0, B43=1.2D0,
     .	 B51=-11./54., B52=2.5D0, B53=-70./27., B54=35./27.,
     .	 B61=1631./55296.,B62=175./512.,B63=575./13824.,
     .	 B64=44275./110592., B65=253./4096., C1=37./378., C3=250./621.,
     .	 C4=125./594., C6=512./1771., DC1=C1-2825./27648.,
     .	 DC3=C3-18575./48384., DC4=C4-13525./55296., DC5=-277./14336.,
     .	 DC6=C6-.25D0)

	EXTERNAL DERIVS

	DO I=1,N
	 YTEMP(I)=Y(I)+B21*H*DYDX(I)
	ENDDO
	CALL DERIVS(N,X+A2*H,YTEMP,AK2)
	DO I=1,N
	 YTEMP(I)=Y(I)+H*(B31*DYDX(I)+B32*AK2(I))
	ENDDO
	CALL DERIVS(N,X+A3*H,YTEMP,AK3)
	DO I=1,N
	 YTEMP(I)=Y(I)+H*(B41*DYDX(I)+B42*AK2(I)+B43*AK3(I))
	ENDDO
	CALL DERIVS(N,X+A4*H,YTEMP,AK4)
	DO I=1,N
	 YTEMP(I)=Y(I)+H*(B51*DYDX(I)+B52*AK2(I)+B53*AK3(I)+B54*AK4(I))
	ENDDO
	CALL DERIVS(N,X+A5*H,YTEMP,AK5)
	DO I=1,N
	 YTEMP(I)=Y(I)+H*(B61*DYDX(I)+B62*AK2(I)+B63*AK3(I)+B64*AK4(I)+
     .	  B65*AK5(I))
	ENDDO
	CALL DERIVS(N,X+A6*H,YTEMP,AK6)
	DO I=1,N
	 YOUT(I)=Y(I)+H*(C1*DYDX(I)+C3*AK3(I)+C4*AK4(I)+C6*AK6(I))
	ENDDO
	DO I=1,N
	 YERR(I)=H*(DC1*DYDX(I)+DC3*AK3(I)+DC4*AK4(I)+DC5*AK5(I)+DC6*
     .	  AK6(I))
	ENDDO

	RETURN
	END


	SUBROUTINE CHECKMIN(PAR,PROB,NPROB)

	IMPLICIT NONE

	INTEGER NPROB,PROB(*)

	DOUBLE PRECISION PAR(*)
	DOUBLE PRECISION l,k,Al,Ak,tanb,sin2b,mu,nu,B,h1,h2
	DOUBLE PRECISION EPS,Q2,t,pi,cc,mQ3,mU3,mD3,At,Ab
	DOUBLE PRECISION Wt,mst1,mst2,Wb,msb1,msb2
	DOUBLE PRECISION ht,Zht,Mstop,dMst,Xt
	DOUBLE PRECISION hb,Zhb,Msbot,dMsb,Xb
	DOUBLE PRECISION ct,fmt1,fmt2,fmt,gmt
	DOUBLE PRECISION cb,fmb1,fmb2,fmb,gmb
	DOUBLE PRECISION tanbq,htq,hbq,mtopq,mbotq
	DOUBLE PRECISION V,V0,V1,V2,V3,D,mh1,mh2,mss
	DOUBLE PRECISION ALSMT,ALSQ,ALSMZ,ALP0,GF,g1,g2,g
	DOUBLE PRECISION MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW,MASM2

	COMMON/GAUGE/ALSMZ,ALP0,GF,g1,g2
	COMMON/SMSPEC/MSB,MS,MC,MB,MT,MTAU,MMUON,MZ,MW

	pi=4.*DATAN(1.D0)
	cc=3./(32.*pi**2)
	EPS=1.D-6

*   Weak scale parameters

	g=(g1+g2)/2.
	l=PAR(1)
	k=PAR(2)
	tanb=PAR(3)
	sin2b=tanb**2/(1.+tanb**2)
	h1=DSQRT(sin2b/(2.*DSQRT(2.D0)*GF))
	h2=h1/tanb
	mu=PAR(4)
	Al=PAR(5)
	Ak=PAR(6)
	mQ3=PAR(7)
	mU3=PAR(8)
	mD3=PAR(9)
	Mstop=(mQ3**2+mU3**2)/2.
	dMst=(mQ3**2-mU3**2)/2.
	At=PAR(12)
	Msbot=(mQ3**2+mD3**2)/2.
	dMsb=(mQ3**2-mD3**2)/2.
	Ab=PAR(13)

	IF(mu.EQ.0. .OR. l.EQ.0. .OR. k.EQ.0.)RETURN

*   At MB

	hb=MB/DABS(h2)
	
*   At MT

	ALSMT=ALSMZ/(1.+23./(12.*PI)*ALSMZ*DLOG(MT**2/MZ**2))
	ht=MT/(1.+5.*ALSMT/(3.*PI)+11.*(ALSMT/PI)**2)/h1
	hb=hb*(1.-23./(12.*PI)*ALSMT*DLOG(MT**2/MB**2))**(12./23.)

*   At Q~Msusy

	Q2=(Mstop+Msbot)/2.+MT**2
	t=DLOG(Q2/MT**2)
	Zht=1.+3.*ht**2/(16.*pi**2)*t
	h1=h1/DSQRT(Zht)
	Zhb=1.+3.*hb**2/(16.*pi**2)*t
	h2=h2/DSQRT(Zhb)
	tanbq=h1/h2
	htq=ht*(1.+7./(4.*PI)*ALSMT*t)**(-4./7.)
     C	 *(1.+(9.*ht**2+hb**2)*t/(64.*PI**2))
	mtopq=htq*h1
	Xt=At-mu/tanbq
	hbq=hb*(1.+7./(4.*PI)*ALSMT*t)**(-4./7.)
     C	 *(1.+(9.*hb**2+ht**2)*t/(64.*PI**2))
	mbotq=hbq*h2
	Xb=Ab-mu*tanbq
	
*   MASM2: ~MSSM-like pseudoscalar mass**2 (if >> MZ)

	MASM2= PAR(4)*(PAR(5)+PAR(2)*PAR(4)/PAR(1))*(tanb+1/tanb)
     
        IF(MASM2.GT.MT**2) THEN
	  htq=htq*(1.+2.*hbq**2*DLOG(MASM2/MT**2)/(64.*PI**2))
	  hbq=hbq*(1.+2.*htq**2*DLOG(MASM2/MT**2)/(64.*PI**2))
        END IF
	
*    Stop masses

	Wt=DSQRT(dMst**2+mtopq**2*Xt**2)
	mst1=Mstop+mtopq**2-Wt
	mst2=Mstop+mtopq**2+Wt
	
*    Sbottom masses

	Wb=DSQRT(dMsb**2+mbotq**2*Xb**2)
	msb1=Msbot+mbotq**2-Wb
	msb2=Msbot+mbotq**2+Wb
	
	IF(mst1.LE.0. .OR. msb1.LE.0.)RETURN

	ct=3.*htq**2/(16.*pi**2)
	fmt1=mst1*(DLOG(mst1/Q2)-1.)
	fmt2=mst2*(DLOG(mst2/Q2)-1.)
	fmt=mtopq**2*(DLOG(mtopq**2/Q2)-1.)
	IF(Wt.NE.0.)THEN
	 gmt=(fmt2-fmt1)/(mst2-mst1)
	ELSE
	 gmt=DLOG(mst1/Q2)
	ENDIF

	cb=3.*hbq**2/(16.*pi**2)
	fmb1=msb1*(DLOG(msb1/Q2)-1.)
	fmb2=msb2*(DLOG(msb2/Q2)-1.)
	fmb=mbotq**2*(DLOG(mbotq**2/Q2)-1.)
	IF(Wt.NE.0.)THEN
	 gmb=(fmb2-fmb1)/(msb2-msb1)
	ELSE
	 gmb=DLOG(msb1/Q2)
	ENDIF

*   Soft masses

	nu=k/l*mu
	B=Al+nu
	mh1= -l**2*h2**2 - mu**2 + mu*B/tanbq + g/2.*(h2**2-h1**2)
     C	   - ct*(fmt1+fmt2-2.*fmt+At*Xt*gmt)
     C	   + cb*mu/tanbq*Xb*gmb
	mh2= -l**2*h1**2 - mu**2 + mu*B*tanbq + g/2.*(h1**2-h2**2)
     C	   + ct*mu*tanbq*Xt*gmt
     C	   - cb*(fmb1+fmb2-2.*fmb+Ab*Xb*gmb)
	mss= -l**2*(h1**2+h2**2) - 2.*nu**2
     C	   + l**2*h1*h2/mu*(B+nu) - nu*Ak
     C	   + ct*l**2*h1*h2/mu*Xt*gmt
     C	   + cb*l**2*h1*h2/mu*Xb*gmb

*   Physical minimum

	V= mu**2*(h1**2+h2**2) + l**2*h1**2*h2**2 + nu**4/k**2
     C	  - 2.*nu*mu*h1*h2 - 2.*Al*mu*h1*h2 + 2./3.*Ak*nu**3/k**2
     C    +g/4.*(h1**2-h2**2)**2
     C	  + mh1*h1**2 + mh2*h2**2 + mss*mu**2/l**2
     C	  + cc*
     C	  ( mst1**2*(DLOG(mst1/Q2)-1.5D0)
     C	  + mst2**2*(DLOG(mst2/Q2)-1.5D0)
     C	  - 2.*mtopq**4*(DLOG(mtopq**2/Q2)-1.5D0)
     C	  + msb1**2*(DLOG(msb1/Q2)-1.5D0)
     C	  + msb2**2*(DLOG(msb2/Q2)-1.5D0)
     C	  - 2.*mbotq**4*(DLOG(mbotq**2/Q2)-1.5D0))

*   Minimum with h1=h2=s=0

	mst1=mU3**2
	mst2=mQ3**2
	msb1=mD3**2
	msb2=mQ3**2
	V0= 0.
     C	  + cc*
     C	  ( mst1**2*(DLOG(mst1/Q2)-1.5D0)
     C	  + mst2**2*(DLOG(mst2/Q2)-1.5D0)
     C	  + msb1**2*(DLOG(msb1/Q2)-1.5D0)
     C	  + msb2**2*(DLOG(msb2/Q2)-1.5D0))
	IF(V0-V.LT.EPS*DABS(V))PROB(NPROB)=1

*   Minimum with h2=s=0

	IF(mh1.LT.0)THEN
	 mtopq=htq*DSQRT(-2.*mh1/g)
	 Wt=DSQRT(dMst**2+mtopq**2*At**2)
	 mst1=Mstop+mtopq**2-Wt
	 mst2=Mstop+mtopq**2+Wt
	 msb1=mD3**2
	 msb2=mQ3**2
	 IF(mst1.LE.0.) GOTO 1
	 V1= -mh1**2/g
     C	   + cc*
     C	   ( mst1**2*(DLOG(mst1/Q2)-1.5D0)
     C	   + mst2**2*(DLOG(mst2/Q2)-1.5D0)
     C	   - 2.*mtopq**4*(DLOG(mtopq**2/Q2)-1.5D0)
     C	   + msb1**2*(DLOG(msb1/Q2)-1.5D0)
     C	   + msb2**2*(DLOG(msb2/Q2)-1.5D0))
	 IF(V1-V.LT.EPS*DABS(V))PROB(NPROB)=1
 1	ENDIF

*   Minimum with h1=s=0

	IF(mh2.LT.0)THEN
	 mst1=mU3**2
	 mst2=mQ3**2
	 mbotq=hbq*DSQRT(-2.*mh2/g)
	 Wb=DSQRT(dMsb**2+mbotq**2*Ab**2)
	 msb1=Msbot+mbotq**2-Wb
	 msb2=Msbot+mbotq**2+Wb
	 IF(msb1.LE.0.) GOTO 2
	 V2= -mh2**2/g
     C	   + cc*
     C	   ( mst1**2*(DLOG(mst1/Q2)-1.5D0)
     C	   + mst2**2*(DLOG(mst2/Q2)-1.5D0)
     C	   + msb1**2*(DLOG(msb1/Q2)-1.5D0)
     C	   + msb2**2*(DLOG(msb2/Q2)-1.5D0)
     C	   - 2.*mbotq**4*(DLOG(mbotq**2/Q2)-1.5D0))
	 IF(V2-V.LT.EPS*DABS(V))PROB(NPROB)=1
 2	ENDIF

*   Minimum with h1=h2=0

	D=Ak**2-8.*ms
	IF(D.GT.0)THEN
	 mst1=mU3**2
	 mst2=mQ3**2
	 msb1=mD3**2
	 msb2=mQ3**2
	 IF(Ak.GT.0.)nu=(-Ak-DSQRT(D))/4.
	 IF(Ak.LT.0.)nu=(-Ak+DSQRT(D))/4.
	 V3= (nu**2+2./3.*Ak*nu+mss)*nu**2/k**2
     C	   + cc*
     C	   ( mst1**2*(DLOG(mst1/Q2)-1.5D0)
     C	   + mst2**2*(DLOG(mst2/Q2)-1.5D0)
     C	   + msb1**2*(DLOG(msb1/Q2)-1.5D0)
     C	   + msb2**2*(DLOG(msb2/Q2)-1.5D0))
	 IF(V3-V.LT.EPS*DABS(V))PROB(NPROB)=1
	ENDIF

	END


      DOUBLE PRECISION FUNCTION RUNM(Q,NF)

*   Subroutine to calculate the quark running masses

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      PARAMETER (NN=6)
      PARAMETER (ZETA3 = 1.202056903159594D0)

      DIMENSION AM(NN),YMSB(NN)

      COMMON/ALS/XLAMBDA,AMCA,AMBA,AMTA,N0A
      COMMON/GAUGE/ALSMZ,ALP0,GF,gg1,gg2
      COMMON/SMSPEC/AMSB,AMS,AMC,AMB,AMT,AMTAU,AMMUON,AMZ,AMW
      SAVE ISTRANGE

      B0(NF)= (33.D0-2.D0*NF)/12D0
      B1(NF)= (102D0-38D0/3D0*NF)/16D0
      B2(NF)= (2857D0/2D0-5033D0/18D0*NF+325D0/54D0*NF**2)/64D0
      G0(NF)= 1D0
      G1(NF)= (202D0/3D0-20D0/9D0*NF)/16D0
      G2(NF)= (1249D0-(2216D0/27D0+160D0/3D0*ZETA3)*NF
     .      - 140D0/81D0*NF**2)/64D0
      C1(NF)= G1(NF)/B0(NF) - B1(NF)*G0(NF)/B0(NF)**2
      C2(NF)= ((G1(NF)/B0(NF) - B1(NF)*G0(NF)/B0(NF)**2)**2
     .      + G2(NF)/B0(NF) + B1(NF)**2*G0(NF)/B0(NF)**3
     .      - B1(NF)*G1(NF)/B0(NF)**2 - B2(NF)*G0(NF)/B0(NF)**2)/2D0
      TRAN(X,XK)= 1D0+4D0/3D0*ALPHAS(X,2)/PI+XK*(ALPHAS(X,2)/PI)**2
      CQ(X,NF)= (2D0*B0(NF)*X)**(G0(NF)/B0(NF))
     .        * (1D0+C1(NF)*X+C2(NF)*X**2)
      DATA ISTRANGE/0/
      PI= 4D0*DATAN(1D0)
      ACC= 1.D-8
      AM(1)= 0
      AM(2)= 0
      AM(3)= AMSB
      AM(4)= AMC
      AM(5)= AMB
      AM(6)= AMT
      XK= 16.11D0
      DO 1 I=1,NF-1
       XK= XK - 1.04D0*(1.D0-AM(I)/AM(NF))
1     CONTINUE
      IF(NF.GE.4)THEN
       XMSB= AM(NF)/TRAN(AM(NF),0D0)
       XMHAT= XMSB/CQ(ALPHAS(AM(NF),2)/PI,NF)
      ELSE
       XMSB= 0
       XMHAT= 0
      ENDIF
      YMSB(3)= AMSB
      IF(NF.EQ.3)THEN
       YMSB(4)= YMSB(3)*CQ(ALPHAS(AM(4),2)/PI,3)/
     .                  CQ(ALPHAS(1.D0,2)/PI,3)
       YMSB(5)= YMSB(4)*CQ(ALPHAS(AM(5),2)/PI,4)/
     .                  CQ(ALPHAS(AM(4),2)/PI,4)
       YMSB(6)= YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                  CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.4)THEN
       YMSB(4)= XMSB
       YMSB(5)= YMSB(4)*CQ(ALPHAS(AM(5),2)/PI,4)/
     .                  CQ(ALPHAS(AM(4),2)/PI,4)
       YMSB(6)= YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                  CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.5)THEN
       YMSB(5)= XMSB
       YMSB(4)= YMSB(5)*CQ(ALPHAS(AM(4),2)/PI,4)/
     .                  CQ(ALPHAS(AM(5),2)/PI,4)
       YMSB(6)= YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                  CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.6)THEN
       YMSB(6)= XMSB
       YMSB(5)= YMSB(6)*CQ(ALPHAS(AM(5),2)/PI,5)/
     .                  CQ(ALPHAS(AM(6),2)/PI,5)
       YMSB(4)= YMSB(5)*CQ(ALPHAS(AM(4),2)/PI,4)/
     .                  CQ(ALPHAS(AM(5),2)/PI,4)
      ENDIF
      IF(Q.LT.AMC)THEN
       N0= 3
       Q0= 1.D0
      ELSEIF(Q.LE.AMB)THEN
       N0= 4
       Q0= AMC
      ELSEIF(Q.LE.AMT)THEN
       N0= 5
       Q0= AMB
      ELSE
       N0= 6
       Q0= AMT
      ENDIF
      IF(NF.GT.3)THEN
       XKFAC= TRAN(AM(NF),0D0)/TRAN(AM(NF),XK)
      ELSE
       XKFAC= 1D0
      ENDIF
      RUNM= YMSB(N0)*CQ(ALPHAS(Q,2)/PI,N0)/
     .               CQ(ALPHAS(Q0,2)/PI,N0)
     .       * XKFAC

      RETURN
      END


*  Running alpha_s and aux. subroutines/functions as in HDECAY

      DOUBLE PRECISION FUNCTION ALPHAS(Q,N)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION XLB(6)

      COMMON/ALSLAM/XLB1(6),XLB2(6)
      COMMON/ALS/XLAMBDA,AMC,AMB,AMT,N0

      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS1(NF,X)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB(NF)**2))
      ALS2(NF,X)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB(NF)**2))
     .          *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB(NF)**2))
     .           /DLOG(X**2/XLB(NF)**2))

  
      PI=4.D0*DATAN(1.D0)

      IF(N.EQ.1)THEN
       DO 1 I=1,6
        XLB(I)=XLB1(I)
1      CONTINUE
      ELSE
       DO 2 I=1,6
        XLB(I)=XLB2(I)
2      CONTINUE
      ENDIF
      IF(Q.LT.AMC)THEN
       NF=3
      ELSEIF(Q.LE.AMB)THEN
       NF=4
      ELSEIF(Q.LE.AMT)THEN
       NF=5
      ELSE
       NF=6
      ENDIF
      IF(N.EQ.1)THEN
        ALPHAS=ALS1(NF,Q)
      ELSE
        ALPHAS=ALS2(NF,Q)
      ENDIF

      RETURN
      END


      SUBROUTINE ALSINI(ACC)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION XLB(6)

      COMMON/ALSLAM/XLB1(6),XLB2(6)
      COMMON/ALS/XLAMBDA,AMC,AMB,AMT,N0

      PI=4.D0*DATAN(1.D0)
      XLB1(1)=0.D0
      XLB1(2)=0.D0
      XLB2(1)=0.D0
      XLB2(2)=0.D0
      IF(N0.EQ.3)THEN
       XLB(3)=XLAMBDA
       XLB(4)=XLB(3)*(XLB(3)/AMC)**(2.D0/25.D0)
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.4)THEN
       XLB(4)=XLAMBDA
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.5)THEN
       XLB(5)=XLAMBDA
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.6)THEN
       XLB(6)=XLAMBDA
       XLB(5)=XLB(6)*(XLB(6)/AMT)**(-2.D0/23.D0)
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
      ENDIF
      DO 1 I=1,6
       XLB1(I)=XLB(I)
1     CONTINUE
      IF(N0.EQ.3)THEN
       XLB(3)=XLAMBDA
       XLB(4)=XLB(3)*(XLB(3)/AMC)**(2.D0/25.D0)
     .             *(2.D0*DLOG(AMC/XLB(3)))**(-107.D0/1875.D0)
       XLB(4)=XITER(AMC,XLB(3),3,XLB(4),4,ACC)
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
     .             *(2.D0*DLOG(AMB/XLB(4)))**(-963.D0/13225.D0)
       XLB(5)=XITER(AMB,XLB(4),4,XLB(5),5,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.4)THEN
       XLB(4)=XLAMBDA
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
     .             *(2.D0*DLOG(AMB/XLB(4)))**(-963.D0/13225.D0)
       XLB(5)=XITER(AMB,XLB(4),4,XLB(5),5,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.5)THEN
       XLB(5)=XLAMBDA
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
     .             *(2.D0*DLOG(AMB/XLB(5)))**(963.D0/14375.D0)
       XLB(4)=XITER(AMB,XLB(5),5,XLB(4),4,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.6)THEN
       XLB(6)=XLAMBDA
       XLB(5)=XLB(6)*(XLB(6)/AMT)**(-2.D0/23.D0)
     .            *(2.D0*DLOG(AMT/XLB(6)))**(321.D0/3703.D0)
       XLB(5)=XITER(AMT,XLB(6),6,XLB(5),5,ACC)
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
     .             *(2.D0*DLOG(AMB/XLB(5)))**(963.D0/14375.D0)
       XLB(4)=XITER(AMB,XLB(5),5,XLB(4),4,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
      ENDIF
      DO 2 I=1,6
       XLB2(I)=XLB(I)
2     CONTINUE

      RETURN
      END


      DOUBLE PRECISION FUNCTION XITER(Q,XLB1,NF1,XLB,NF2,ACC)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS2(NF,X,XLB)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB**2))
     .              *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB**2))
     .              /DLOG(X**2/XLB**2))
      AA(NF)=12D0*PI/B0(NF)
      BB(NF)=B1(NF)/AA(NF)
      XIT(A,B,X)=A/2.D0*(1D0+DSQRT(1D0-4D0*B*DLOG(X)))
      PI=4.D0*DATAN(1.D0)
      XLB2=XLB
      II=0
1     II=II+1
      X=DLOG(Q**2/XLB2**2)
      ALP=ALS2(NF1,Q,XLB1)
      A=AA(NF2)/ALP
      B=BB(NF2)*ALP
      XX=XIT(A,B,X)
      XLB2=Q*DEXP(-XX/2.D0)
      Y1=ALS2(NF1,Q,XLB1)
      Y2=ALS2(NF2,Q,XLB2)
      DY=DABS(Y2-Y1)/Y1
      IF(DY.GE.ACC) GOTO 1
      XITER=XLB2

      RETURN
      END


      DOUBLE PRECISION FUNCTION XITLA(NO,ALP,ACC)

*  Iteration routine to determine improved Lambda's

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      COMMON/GAUGE/ALSMZ,ALP0,GF,gg1,gg2
      COMMON/SMSPEC/AMSB,AMS,AMC,AMB,AMT,AMTAU,AMMUON,AMZ,AMW

      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS2(NF,X,XLB)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB**2))
     .              *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB**2))
     .              /DLOG(X**2/XLB**2))
      AA(NF)=12D0*PI/B0(NF)
      BB(NF)=B1(NF)/AA(NF)
      XIT(A,B,X)=A/2.D0*(1D0+DSQRT(1D0-4D0*B*DLOG(X)))
      PI=4.D0*DATAN(1.D0)
      NF=5
      Q=AMZ
      XLB=Q*DEXP(-AA(NF)/ALP/2.D0)
      IF(NO.EQ.1)GOTO 111
      II=0
1     II=II+1
      X=DLOG(Q**2/XLB**2)
      A=AA(NF)/ALP
      B=BB(NF)*ALP
      XX=XIT(A,B,X)
      XLB=Q*DEXP(-XX/2.D0)
      Y1=ALP
      Y2=ALS2(NF,Q,XLB)
      DY=DABS(Y2-Y1)/Y1
      IF(DY.GE.ACC) GOTO 1
111   XITLA=XLB

      RETURN
      END


      DOUBLE PRECISION FUNCTION FINT(Z,XX,YY)

*  One-dimensional cubic interpolation
*  Z  = wanted point
*  XX = array of 4 discrete x-values around Z
*  YY = array of 4 discrete function-values around Z

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION XX(4),YY(4)

      X = DLOG(Z)
      X0=DLOG(XX(1))
      X1=DLOG(XX(2))
      X2=DLOG(XX(3))
      X3=DLOG(XX(4))
      Y0=DLOG(YY(1))
      Y1=DLOG(YY(2))
      Y2=DLOG(YY(3))
      Y3=DLOG(YY(4))
      A0=(X-X1)*(X-X2)*(X-X3)/(X0-X1)/(X0-X2)/(X0-X3)
      A1=(X-X0)*(X-X2)*(X-X3)/(X1-X0)/(X1-X2)/(X1-X3)
      A2=(X-X0)*(X-X1)*(X-X3)/(X2-X0)/(X2-X1)/(X2-X3)
      A3=(X-X0)*(X-X1)*(X-X2)/(X3-X0)/(X3-X1)/(X3-X2)
      FINT=DEXP(A0*Y0+A1*Y1+A2*Y2+A3*Y3)

      RETURN
      END


*   Spence function and auxiliary functions as in HDECAY

      DOUBLE PRECISION FUNCTION SP(X)

*  REAL dilogarithm (Spence-function)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 CX,LI2

      CX = DCMPLX(X,0.D0)
      SP = DREAL(LI2(CX))

      RETURN
      END


      COMPLEX FUNCTION LI2*16(X)

*  COMPLEX dilogarithm (Spence-function)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X,Y,CLI2

      COMMON/CONST/ZETA2,ZETA3

      ZERO=1.D-16
      XR=DREAL(X)
      XI=DIMAG(X)
      R2=XR*XR+XI*XI
      LI2=0
      IF(R2.LE.ZERO)THEN
        LI2=X
        RETURN
      ENDIF
      RR=XR/R2
      IF(R2.EQ.1.D0.AND.XI.EQ.0.D0)THEN
        IF(XR.EQ.1.D0)THEN
          LI2=DCMPLX(ZETA2)
        ELSE
          LI2=-DCMPLX(ZETA2/2.D0)
        ENDIF
        RETURN
      ELSEIF(R2.GT.1.D0.AND.RR.GT.0.5D0)THEN
        Y=(X-1.D0)/X
        LI2=CLI2(Y)+ZETA2-CDLOG(X)*CDLOG(1.D0-X)+0.5D0*CDLOG(X)**2
        RETURN
      ELSEIF(R2.GT.1.D0.AND.RR.LE.0.5D0)THEN
        Y=1.D0/X
        LI2=-CLI2(Y)-ZETA2-0.5D0*CDLOG(-X)**2
        RETURN
      ELSEIF(R2.LE.1.D0.AND.XR.GT.0.5D0)THEN
        Y=1.D0-X
        LI2=-CLI2(Y)+ZETA2-CDLOG(X)*CDLOG(1.D0-X)
       RETURN
      ELSEIF(R2.LE.1.D0.AND.XR.LE.0.5D0)THEN
        Y=X
        LI2=CLI2(Y)
        RETURN

      ENDIF
      END
 

      COMPLEX FUNCTION CLI2*16(X)

*  Taylor-expansion for complex dilogarithm (Spence-function)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X,Z

      COMMON/BERNOULLI/B2(18),B12(18),B3(18)
      COMMON/POLY/NBER

      N=NBER-1
      Z=-CDLOG(1.D0-X)
      CLI2=B2(NBER)
      DO 111 I=N,1,-1
        CLI2=Z*CLI2+B2(I)
111   CONTINUE
      CLI2=Z**2*CLI2+Z

      RETURN
      END

 
      DOUBLE PRECISION FUNCTION FACULT(N)

*  DOUBLE PRECISION version of FACULTY

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      FACULT=1.D0
      IF(N.EQ.0)RETURN
      DO 999 I=1,N
        FACULT=FACULT*DFLOAT(I)
999   CONTINUE

      RETURN
      END

 
      SUBROUTINE BERNINI(N)

*  Initialization of coefficients for polylogarithms

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION B(18),PB(19)

      COMMON/BERNOULLI/B2(18),B12(18),B3(18)
      COMMON/CONST/ZETA2,ZETA3
      COMMON/POLY/NBER

      NBER=N
      PI=4.D0*DATAN(1.D0)
 
      B(1)=-1.D0/2.D0
      B(2)=1.D0/6.D0
      B(3)=0.D0
      B(4)=-1.D0/30.D0
      B(5)=0.D0
      B(6)=1.D0/42.D0
      B(7)=0.D0
      B(8)=-1.D0/30.D0
      B(9)=0.D0
      B(10)=5.D0/66.D0
      B(11)=0.D0
      B(12)=-691.D0/2730.D0
      B(13)=0.D0
      B(14)=7.D0/6.D0
      B(15)=0.D0
      B(16)=-3617.D0/510.D0
      B(17)=0.D0
      B(18)=43867.D0/798.D0
      ZETA2=PI**2/6.D0
      ZETA3=1.202056903159594D0
 
      DO 995 I=1,18
        B2(I)=B(I)/FACULT(I+1)
        B12(I)=DFLOAT(I+1)/FACULT(I+2)*B(I)/2.D0
        PB(I+1)=B(I)
        B3(I)=0.D0
995   CONTINUE
      PB(1)=1.D0
      DO 996 I=1,18
      DO 996 J=0,I
       B3(I)=B3(I)+PB(J+1)*PB(I-J+1)/FACULT(I-J)/FACULT(J+1)/DFLOAT(I+1)
996   CONTINUE

      RETURN
      END
